#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import deb822
import glob
import json
import logging
import os
import subprocess
import sys
import tempfile

from argparse import ArgumentParser
from ubuntu_system_tests import (
    config,
    results,
    ssh,
    testdesc
)
from ubuntu_system_tests.helpers.file_system import clean_dir


logger = logging.getLogger(__name__)
_ADT_RUN_SHORT_TIMEOUT = 600  # 10 minutes.
_ADT_RUN_TEST_TIMEOUT = 86400  # 24 hours

# apt-cacher-ng definitions
APT_CACHER = 'apt-cacher-ng'
APT_CACHER_CACHE_DIR = '/var/cache/apt-cacher-ng'
APT_CACHER_LOG_DIR = '/var/log/apt-cacher-ng/'
APT_CACHER_INIT_FILE = '/etc/init/apt-cacher-ng.conf'
APT_PROXY = 'Acquire::http { Proxy "http://127.0.0.1:3142"; };'
APT_PROXY_FILE = '/etc/apt/apt.conf.d/01proxy'
EXT_CACHE_DIR = '/userdata/apt-cacher-ng/cache'
EXT_LOG_DIR = '/userdata/apt-cacher-ng/logs'
PERMISSION = '755'


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    args = _parse_arguments(argv)
    return run_system_tests(args.config, args.suites,
                            args.update_apt, args.network_ready,
                            args.touch_visualization, args.install_deps,
                            args.install_tests, args.silent, args.cache_deps,
                            args.verbose)


def _parse_arguments(argv=None):
    parser = ArgumentParser('Run system tests for Ubuntu Touch.')
    parser.add_argument('--config', '-c',
                        help='The config file to use for this run.',
                        default='ubuntu-system-tests.conf')
    parser.add_argument(
        '--update-apt', '-u', action='store_true',
        help='Update apt lists before installing the dependencies',
        default=False)
    parser.add_argument(
        '--touch-visualization', '-t', action='store_true',
        help='Enable visualization of touch points on screen.')
    parser.add_argument(
        '--network-ready', '-n', action='store_true',
        help='Assume that the network connection is already setup.')
    parser.add_argument(
        '--install-deps', '-p', action='store_true',
        help='Install only updated production dependencies permanently in the '
             'device instead of in the /tmp directory. This will not install '
             'any test dependencies.',
        default=False)
    parser.add_argument(
        '--install-tests', '-i', action='store_true',
        help='Install all dependencies permanently in the device instead of '
             'in the /tmp directory. This includes all production and test '
             'dependencies.',
        default=False)
    parser.add_argument(
        '--silent', '-s', action='store_true',
        help='Do not prompt for missing user config options, use default '
        'values instead.')
    parser.add_argument(
        '--cache-deps', '-d', action='store_true',
        help='Cache dependencies on the device so they don\'t have to be '
             'downloaded for every test run.')
    parser.add_argument(
        'suites', nargs='*', help='The list of test suites to run.')
    parser.add_argument(
        '--verbose', '-v', action='store_true',
        help='Provide verbose logging.', default=False)
    return parser.parse_args(args=argv)


def run_system_tests(conf_name, tests_to_run, update_apt=False,
                     network_ready=False, touch_visualization=False,
                     install_deps=False, install_tests=False, silent=False,
                     cache_deps=False, verbose=False):
    """Run system tests in a device connected via USB and report the results.
    :param conf_name: The name of the configuration file that will be used
        for the system tests.
    :param tests_to_run: A list of tests to run.
    :param update_apt: Boolean representing whether apt-get update should be
        run on the device before installing dependencies.
    :param network_ready: Boolean representing whether phablet-network should
    be run to provision the network on the device.
    :param install_deps: Install the test dependencies permanently in the
    device instead of in the /tmp directory
    :param touch_visualization: Enable visualization of touch points on screen
    :param silent: If True then do not prompt for user config options, instead
                   use the default value.
    :param cache_deps: If True then cache the downloaded dependencies so that
                       they don't have to be downloaded for every test run.
    :returns: 0 when successful, other value when failed.

    """
    config_stack = _get_config_stack(conf_name, tests_to_run, silent)
    _add_touch_visualization(config_stack, touch_visualization)
    print_warnings()
    clean_output_directory(config_stack)

    enable_autopilot_introspection(config_stack)
    return_code = run_autopkg_test(config_stack, update_apt, network_ready,
                                   install_deps, install_tests, cache_deps,
                                   verbose)

    if return_code <= 6:
        # Return codes higher than 6 mean that there was a problem with adt or
        # the test bed, not a failure in the tests. So there will be no file
        # with results.
        generate_user_report(config_stack)

    return return_code


def get_camera_warning():
    return '- Please ensure device is raised from surface to allow rear ' \
           'camera to take a photograph during camera tests.'


def print_warnings():
    print_separator('*')
    logger.warning(get_camera_warning())
    print_separator('*')


def print_separator(char):
    logger.warning(char * 125)


def clean_output_directory(config_stack):
    """
    Clean the output directory specified in config_stack.

    This is required so that adt-run does not fail due to existing content
    in the output directory from a previous test run.

    """
    path = os.path.abspath(config_stack.get('output_dir'))
    if os.path.isdir(path):
        clean_dir(path)


def _get_config_stack(conf_name, tests_to_run, silent=False):
    """Get the configuration values for the system tests.

    This function registers the declared options, opens the config store
    using the specified file name, adds the options coming from the command
    line, and asks the user for the missing options.

    :param conf_name: File name of config file to use.
    :param tests_to_run: List of tests to be saved to config and executed.
    :param silent: If True then don't prompt the user for any missing config
                   options, use default values instead. If False then prompt
                   the user for any missing option values.

    """
    config.register_test_options()

    config_stack = config.UbuntuSystemTestsStack(conf_name)

    # We set the values passed from the command line to send them from the host
    # to the testbed.
    config_stack.set('tests_to_run', ' '.join(tests_to_run or []))
    config_stack.get_missing_options(silent)
    ssh.configure_config_stack(config_stack)

    return config_stack


def _add_touch_visualization(config_stack, touch_visualization):
    """Add touch_visualization parameter to config and save."""
    config_stack.set('touch_visualization', str(touch_visualization).lower())
    config_stack.save()


def _get_serial_argument(config_stack):
    device_serial = config_stack.get('device_serial')
    if device_serial:
        return ['-s', device_serial]
    else:
        # No serial in the config, run the commands as if only one device is
        # connected.
        return []


def enable_autopilot_introspection(config_stack):
    command = _build_enable_autopilot_introspection_command(config_stack)
    subprocess.check_output(command)


def _build_enable_autopilot_introspection_command(config_stack):
    command = (
        ['phablet-config'] +
        _get_serial_argument(config_stack) +
        ['autopilot', '--dbus-probe', 'enable']
    )
    return command


def run_autopkg_test(config_stack, update_apt, network_ready, install_deps,
                     install_tests, cache_deps, verbose):
    """ Run the autopkgtest in the revice for the tests selected
    :param config_stack: The config objects
    :param update_apt: True if the packages hasve to be updated
    :param network_ready: True if the network set in the config has to be
    configured in the device
    :param install_deps: True if the tests dependencies have to be installed
    permanently in the device instead of install those in the tmp dir
    :param cache_deps: If True then cache the downloaded dependencies so that
                       they don't have to be downloaded for every test run.
    :param verbose: True to provide verbose logging.
    :return: the autopkgtest execution exit code
    """
    extra_depends = _get_extra_dependencies(
        config_stack.get('tests_to_run', '').split()
    )
    if cache_deps:
        adt_run_cmd = _build_adt_run_test_command(
            config_stack,
            extra_depends=None,
            update_apt=update_apt,
            network_ready=network_ready,
            add_ppa=False,
            install_deps=False,
            install_tests=False,
            cache_deps=True,
            verbose=verbose)
        subprocess.call(adt_run_cmd)
        clean_output_directory(config_stack)
        logger.info('apt-cacher-ng installed on device')
    add_ppa = True
    if install_deps or install_tests:
        adt_run_cmd = _build_adt_run_test_command(
            config_stack,
            extra_depends=extra_depends,
            update_apt=update_apt,
            network_ready=network_ready,
            add_ppa=add_ppa,
            install_deps=install_deps,
            install_tests=install_tests,
            verbose=verbose)
        subprocess.call(adt_run_cmd)

        clean_output_directory(config_stack)
        add_ppa = False
        update_apt = False
        install_deps = False
        install_tests = False

        logger.info('Install dependencies command finished')

    adt_run_cmd = _build_adt_run_test_command(
        config_stack,
        extra_depends=extra_depends,
        update_apt=update_apt,
        network_ready=network_ready,
        add_ppa=add_ppa,
        install_deps=install_deps,
        install_tests=install_tests,
        verbose=verbose)
    return subprocess.call(adt_run_cmd)


def _get_extra_dependencies(tests_to_run):
    """
    Get any extra dependencies required to run the list of
    tests provided

    :param tests_to_run: The list of tests to run
    :returns: A list of the extra dependencies
    """
    depends_map = _read_dependency_files(
        os.path.realpath('ubuntu_system_tests/external_tests')
    )
    extra_depends = []
    for test in tests_to_run:
        extra_depends.extend(_extra_dependencies_for_test(test, depends_map))
    return list(set(extra_depends))


def _read_dependency_files(deps_path):
    """
    Read the information about how dependencies map
    to test names from the .json files in the path
    provided.

    :param deps_path: The path to the directory containing the files
    :returns: A dictionary associating test prefixes with lists of dependencies
    """
    depends_map = {}
    for file in glob.glob(deps_path + '/*.json'):
        with open(file) as deps_file:
            for depends in json.loads(deps_file.read())['test_dependencies']:
                depends_map[depends['id']] = depends['depends']
    return depends_map


def _extra_dependencies_for_test(test, depends_map):
    """
    Calculates the extra dependencies required for a particular test

    :param test: The test to get the dependencies for
    :param depends_map: The dependency mapping
    :returns: A list of the extra dependencies
    """
    extra_dependencies = []
    for id in depends_map.keys():
        if test.startswith(id):
            extra_dependencies.append(depends_map[id])
    return extra_dependencies


def _create_temp_control_file(dependencies,
                              control_file='debian/tests/control'):
    """Create a copy of the test suite control file, appending the list of
    dependencies

    :param dependencies: List of dependencies to append to file
    :returns: File path to the new control file
    """
    # create a temporary file
    _, tmp_path = tempfile.mkstemp(text=True)
    # open existing control file
    with open(os.path.realpath(control_file), 'r') as tests_control:
        new_content = _append_dependencies(
            tests_control, dependencies)
        # write the new content to the temp file
        with open(tmp_path, 'w') as tmp_file:
            tmp_file.write(new_content)
    # save the path in config
    return tmp_path


def _append_dependencies(tests_control, dependencies):
    """Append the list of dependencies to the file content and return"""
    content = ''
    for pkg in deb822.Packages.iter_paragraphs(tests_control):
        if pkg['Tests'] == 'systemtests':
            current_deps = pkg['Depends']
            for dep in dependencies:
                current_deps += '\n         {},'.format(dep)
            pkg['Depends'] = current_deps
        content += pkg.dump() + '\n'
    return content.rstrip()


def _add_copy_keyfile(run_cmd, config_stack):
    """ In case the keyfile is defined, add the parameter used to copy it to
    the device
    """
    keyfile = config_stack.get('ssh_private_key')
    if keyfile and os.path.isfile(keyfile):
        run_cmd.extend(['--copy', '{src}:{dest}'.format(
            src=keyfile, dest=ssh.get_device_keyfile_path(keyfile))])


def _build_basic_adt_run_test_command(config_stack, extra_depends, verbose):
    """ Build and return the first part of the adt-run """
    output_dir = config_stack.get('output_dir')

    run_cmd = ['adt-run']
    if verbose:
        run_cmd.append('-ddd')
    if extra_depends:
        run_cmd.append('--override-control={}'.format(
            _create_temp_control_file(extra_depends))
        )
    run_cmd.extend([
        # Updating and installing can take a while.
        '--timeout-short={}'.format(_ADT_RUN_SHORT_TIMEOUT),
        # Regression tests can take up to 8 hours, no reason to cut short.
        '--timeout-test={}'.format(_ADT_RUN_TEST_TIMEOUT),
        '-B',
        '--unbuilt-tree={}'.format(os.getcwd()),
        '--output-dir',
        output_dir,
        # Even if the config file on the host has a different name, on the
        # testbed it's copied with the default name.
        '--copy',
        '{src}:{dest}'.format(
            src=config_stack.store.path,
            dest=config.get_device_config_file_path()),
    ])

    _add_copy_keyfile(run_cmd, config_stack)
    return run_cmd


def _build_adt_run_test_command(config_stack, extra_depends=None,
                                update_apt=False, network_ready=False,
                                add_ppa=True, install_deps=False,
                                install_tests=False, cache_deps=False,
                                verbose=False):

    device_password = config_stack.get('device_password')
    run_cmd = _build_basic_adt_run_test_command(
        config_stack, extra_depends, verbose)

    ssh_cmd = (
        ['ssh', '-s', 'adb', '--', '-p', device_password] +
        _get_serial_argument(config_stack)
    )

    run_cmd.extend(_get_network_setup_command(config_stack, network_ready))
    run_cmd.extend(_get_update_apt_command(update_apt))
    run_cmd.extend(_get_apparmor_rules_update_command())

    if cache_deps:
        run_cmd.extend(_get_apt_cacher_setup_command())
        return run_cmd + ['---'] + ssh_cmd

    if add_ppa:
        run_cmd.extend(_get_fs_rw_command())
        run_cmd.extend(_get_ppa_and_testability_setup_command())

    if install_deps or install_tests:
        run_cmd.extend(_get_install_setup_command(install_tests))
    else:
        run_cmd.extend(_get_fs_ro_command())

    return run_cmd + ['---'] + ssh_cmd


def _get_update_apt_command(update_apt):
    """Return command to update apt."""
    if update_apt:
        return [
            '--setup-commands',
            # emulate pitti's convenience 'ro-apt-update' script, see
            # http://anonscm.debian.org/cgit/autopkgtest/autopkgtest.git/commit/?id=a51f26f61c376  # NOQA
            'echo "Mounting rw to update"; RETRY=0; '
            'until mount -o remount,rw / || [ $RETRY -gt 10 ]; do sleep 1; '
            'RETRY=$(($RETRY+1)); echo "mount attempt failed, trying '
            'again..."; done; if [ $RETRY -gt 10 ]; then exit 1; fi; '
            'echo "apt-get updating"; apt-get update || '
            '(sleep 15; apt-get update); '
            'echo "Mounting ro after updating"; RETRY=0; '
            'until mount -o remount,ro / || [ $RETRY -gt 60 ]; do '
            'sleep 1; RETRY=$(($RETRY+1)); echo "mount attempt failed, '
            'trying again..."; done; if [ $RETRY -gt 60 ]; then exit 1; fi'
        ]
    return []


def _get_device_network_command(wifi_ssid, wifi_password):
    """Return command to configure wifi network on device.

    :param wifi_ssid: SSID of required wifi network
    :param wifi_password: Password of required wifi network

    """
    cmd_fmt = ('nmcli device wifi connect '
               '\'{ssid}\' password \'{password}\' 2>/dev/null || true')
    return cmd_fmt.format(ssid=wifi_ssid, password=wifi_password)


def _get_fs_rw_command():
    return [
        '--setup-commands',
        'echo "Mounting rw to install packages"; '
        'sync; RETRY=0; until mount -o remount,rw / || [ $RETRY -gt 10 ]; '
        'do sleep 1; RETRY=$(($RETRY+1)); echo "mount attempt failed, trying '
        'again..."; done; if [ $RETRY -gt 10 ]; then exit 1; fi'
    ]


def _get_fs_ro_command():
    return [
        '--setup-commands',
        'echo "Mounting ro after installing packages"; '
        'sync; RETRY=0; until mount -o remount,ro / || [ $RETRY -gt 10 ]; '
        'do sleep 1; RETRY=$(($RETRY+1)); echo "mount attempt failed, trying '
        'again..."; done; if [ $RETRY -gt 10 ]; then exit 1; fi'
    ]


def _get_network_setup_command(config_stack, network_ready):
    """ The network needs to be configured on the device
    This setup-command should be the first to run so that wi-fi is ready
    """
    cmd = []
    if not network_ready:
        network_cmd = _get_device_network_command(
            config_stack.get('wifi_ssid'),
            config_stack.get('wifi_password'))
        cmd = ['--setup-commands', network_cmd]

    return cmd + [
        '--setup-commands', 'echo \'Checking device network connection...\'; '
        'RETRY=0; until wget -q --spider launchpad.net || [ $RETRY -gt 30 ]; '
        'do sleep 1; RETRY=$(($RETRY+1)); echo "wget attempt $RETRY failed, '
        ' trying again..."; done; if [ $RETRY -gt 30 ]; then echo \'Device '
        'has no network connection!\'; exit 1; fi'
    ]


def _get_ppa_and_testability_setup_command():
    return [
        '--setup-commands',
        'if ! ls /etc/apt/sources.list.d/canonical-platform-qa* > /dev/null '
        '2>&1; then '
        'apt-add-repository -y ppa:canonical-platform-qa/selenium; '
        'apt-get --no-list-cleanup update -o Dir::Etc::SourceList=/dev/null; '
        'fi',
        '--setup-commands',
        'if apt-cache policy qttestability-autopilot | '
        'grep -q "Installed: (none)"; then '
        'apt-get install -y --no-install-recommends qttestability-autopilot; '
        'fi'
    ]


def _get_reboot_command():
    """Return command to reboot device."""
    return ['--setup-commands', 'reboot']


def _get_apt_cacher_autostart_command():
    """Write the apt-cacher-ng conf file into /etc/init/apt-cacher-ng.conf
    to re-launch apt-cacher-ng on each subsequent reboot.

    This method assumes that upstart is used as the init system. If this
    changes in future then this method will need updating to accomodate that.
    """
    script = ('start on runlevel [2345]\n'
              'script\n' +
              _get_mount_dir_command(EXT_LOG_DIR, APT_CACHER_LOG_DIR) +
              _get_mount_dir_command(EXT_CACHE_DIR, APT_CACHER_CACHE_DIR) +
              _get_apt_cacher_start_command() +
              '\nend script')
    return ['--setup-commands',
            _get_create_file_command(
                content=script, path=APT_CACHER_INIT_FILE)]


def _get_apt_cacher_install_command():
    """Return command to install apt-cacher-ng on device and configure it to
    use userdata partion for caching and logging."""
    return [
        '--setup-commands',
        _get_install_package_command(APT_CACHER) +
        _get_apt_cacher_stop_command() +
        _get_create_folder_command(
            path=EXT_LOG_DIR, owner=APT_CACHER, permissions=PERMISSION) +
        _get_create_folder_command(
            path=EXT_CACHE_DIR, owner=APT_CACHER, permissions=PERMISSION) +
        _get_create_folder_command(
            path=APT_CACHER_CACHE_DIR, owner=APT_CACHER,
            permissions=PERMISSION) +
        _get_mount_dir_command(EXT_LOG_DIR, APT_CACHER_LOG_DIR) +
        _get_mount_dir_command(EXT_CACHE_DIR, APT_CACHER_CACHE_DIR) +
        _get_create_file_command(content=APT_PROXY, path=APT_PROXY_FILE) +
        _get_apt_cacher_start_command()]


def _get_create_folder_command(path, owner, permissions):
    """Return command to create a folder with specified path, owner and
    permissions."""
    return ('if ! [ -d {path} ]; then '
            'mkdir -p {path}; fi; '
            'chown {owner}:{owner} {path}; '
            'chmod {permissions} {path}; ').format(
                path=path, owner=owner, permissions=permissions)


def _get_install_package_command(package):
    """Return command to install package on device."""
    return ('apt-get -y install {package}; ').format(package=package)


def _get_mount_dir_command(target, source):
    """Return command to create a symlink between source and target
    directories."""
    return 'mount -o bind {target} {source}; '.format(
        target=target, source=source)


def _get_create_file_command(content, path):
    """Return command to create file with specified content and path."""
    return 'echo \'{content}\' | tee {path}; '.format(
        content=content, path=path)


def _get_apt_cacher_start_command():
    """Return command to start apt-cacher-ng process."""
    return ('/usr/sbin/apt-cacher-ng -c /etc/apt-cacher-ng '
            'pidfile=/var/run/apt-cacher-ng/pid '
            'SocketPath=/var/run/apt-cacher-ng/socket '
            'foreground=0; ')


def _get_apt_cacher_stop_command():
    """Return command to stop apt-cacher-ng process."""
    return 'pkill apt-cacher-ng || true; '


def _get_apt_cacher_setup_command():
    """Return command to install and configure apt-cacher-ng on device."""
    return (_get_fs_rw_command() + _get_apt_cacher_install_command() +
            _get_apt_cacher_autostart_command() + _get_reboot_command())


def _get_install_setup_command(install_tests=False):
    """ Get a list with all the dependencies from the tests/control file and
    when the apt-get install does not install any package, the device is set
    as ro and exit, otherwise it is rebooted and will start as ro
    """
    dir_name = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
    if install_tests:
        dependencies = testdesc.get_all_dependencies(dir_name, 'TMPINSTALL')
    else:
        dependencies = testdesc.get_all_dependencies(dir_name, 'TESTPKG')
    install_command = 'apt-get -o dir::cache::archives="/tmp/" -y install'
    clean_command = 'apt-get clean'
    for dep in dependencies:
        install_command += ' {}'.format(dep.split()[0])

    return ['--setup-commands', clean_command,
            '--setup-commands', install_command,
            '--setup-commands', 'sleep 5',
            '--setup-commands', clean_command,
            '--setup-commands', 'rm -f /userdata/.writable_image',
            '--setup-commands', 'reboot']


def _get_apparmor_rules_update_command():
    rules_file = '/var/cache/apparmor/click-ap.rules'
    update_command = 'printf "dbus (receive, send) ' \
                     'bus=session path=/com/canonical/Autopilot/**,\n' \
                     '/tmp/adt-run.** r,\n" > {rules};' \
                     'aa-clickhook --include={rules}'.format(rules=rules_file)
    return [
        '--setup-commands',
        'if [ ! -e {rules} ];then {command};fi'.format(rules=rules_file,
                                                       command=update_command)
    ]


def generate_user_report(config_stack):
    """Create a human readable report from the subunit results."""
    artifacts_directory = results.get_artifacts_directory(config_stack)
    results.convert_subunit_results_to_pyunit(artifacts_directory)
    results.print_results(artifacts_directory)


if __name__ == '__main__':
    sys.exit(main())
