# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import time

from collections import deque


def wait_until(predicate, *args, timeout=10, period=1, stop_event=None,
               validator=None, **kwargs):
    """
    Waits either until the timeout is reached, the predicate evaluation is
    True, or the stop_event is set.

    To evaluate the predicate
     :param: predicate: The method to be evaluated
     :param: *args: The predicate positional parameters
     :param: timeout: The timeout for the polling
     :param: period: The retry period of time
     :param: stop_event: Event object to indicate whether to stop waiting
                         before the timeout period is over. This will be
                         ignored if not set.
     :param: validator: Validation method used to evaluate result of predicate.
                        This will be ignored if not set.
     :param: **kwargs: The predicate optional parameters
     :return: Result of predicate if evaluated as True before the
              timeout is reached, False otherwise.
    """
    end = time.time() + timeout
    while time.time() < end:
        if stop_event and stop_event.is_set():
            return False
        result = predicate(*args, **kwargs)
        if _is_match(result, validator):
            return result
        time.sleep(period)
    return False


def wait_until_stable(predicate, *args, timeout=10, period=1, count=3,
                      stop_event=None, **kwargs):
    """
    Waits either until the timeout is reached, the predicate result returns
    the same value for the specified number of times, or the stop_event is
    set.

     :param: predicate: The method to be evaluated which must return a value
                        to be compared.
     :param: *args: The predicate positional parameters
     :param: timeout: The timeout for the polling
     :param: period: The retry period of time
     :param: count: The number of results to compare which must all be equal
                    for result to be set True.
     :param: stop_event: Event object to indicate whether to stop waiting
                         before the timeout period is over. This will be
                         ignored if not set.
     :param: **kwargs: The predicate optional parameters
     :return: True if the result of predicate is stable within the timeout
              period, False otherwise.
    """
    results = deque(maxlen=count)
    end = time.time() + timeout
    while time.time() < end:
        if stop_event and stop_event.is_set():
            return False
        results.append(predicate(*args, **kwargs))
        if results.count(results[0]) == results.maxlen:
            return True
        time.sleep(period)
    return False


def _is_match(result, validator):
    """
    Return True if the the result is evaluated as valid.

    If a validator method is specified then return result of validation.
    Otherwise return boolean evaluation of result.

    :param result: Result to evaluate
    :param validator: Validation method to execute using result as parameter.
    :return: True if result is evaluated as True, False otherwise.

    """
    if validator:
        return validator(result)
    else:
        return bool(result)


class AttrDict(dict):
    """ Class used to access to the dict keys as parameters """

    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self
