# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import contextlib
import json
import os
import subprocess

import fixtures
from debian import debfile
from gi.repository import Click


class ClickPackageInstalled(fixtures.Fixture):
    """Fixture to install a click package.

    This fixture will make sure that the package is uninstalled on clean up.

    """

    def __init__(self, click_file_path):
        """Initialize an instance of the ClickPackageInstalled fixture.

        :param click_file_path: The path to the click package that will be
            installed when the fixture is used.

        """
        super().__init__()
        self.click_file_path = click_file_path

    def setUp(self):
        super().setUp()
        self.addCleanup(self._ensure_package_uninstalled)
        self._install_package()

    def _ensure_package_uninstalled(self):
        package_name, package_version = self._get_package_name_and_version()
        ensure_package_uninstalled(package_name, package_version)

    def _get_package_name_and_version(self):
        with contextlib.closing(
                debfile.DebFile(filename=self.click_file_path)) as package:
            manifest_file = package.control.get_file(
                'manifest', encoding='UTF-8')
            json_manifest = json.load(manifest_file)
            return json_manifest['name'], json_manifest['version']

    def _install_package(self):
        subprocess.check_call([
            'pkcon', '-p', 'install-local', '--allow-untrusted',
            self.click_file_path
        ])


def ensure_package_uninstalled(package_name, package_version=None):
    """Uninstall a package if it is installed."""
    if is_package_installed(package_name):
        uninstall_package(package_name, package_version)


def is_package_installed(package_name):
    """Return True if the package is installed. False otherwise."""
    registry = _get_click_registry()
    return registry.has_package_name(package_name)


def get_click_app_identifier(click):
    """Return the app identifier or None if not found running."""
    output = subprocess.check_output(['ubuntu-app-triplet', click],
                                     universal_newlines=True)
    return output.strip()


def _get_click_registry():
    """Return click registry for the current user."""
    db = Click.DB()
    db.read()
    return Click.User.for_user(db, name=os.getenv('USER'))


def _get_installed_packages_manifest():
    registry = _get_click_registry()
    return registry.get_manifests()


def uninstall_package(package_name, package_version=None):
    """Uninstall a package."""
    if not package_version:
        package_version = _get_installed_package_version(package_name)
    package_id = '{};{};all;local:click'.format(
        package_name, package_version)
    subprocess.check_call(['pkcon', '-p', 'remove', package_id])


def _get_installed_package_version(package_name):
    manifests = _get_installed_packages_manifest()
    for index in range(manifests.get_length()):
        manifest = manifests.get_element(index).get_object()
        if manifest.get_member('name').get_string() == package_name:
            return manifest.get_member('version').get_string()
