# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

import dialer_app

import autopilot

from ubuntuuitoolkit import UbuntuUIToolkitCustomProxyObjectBase

from ubuntu_system_tests.helpers.autopilot import (
    patched_select_single, validate_dbus_object, wait_select_many)
from ubuntu_system_tests.helpers.dialer_app import unformatted_phone_number

logger = logging.getLogger(__name__)

APP_PATH_ROOT = b'DialerApp'


class DialerApp(UbuntuUIToolkitCustomProxyObjectBase):

    """Autopilot custom proxy object for the dialer app."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'DialerApp',
            applicationName='DialerApp')

    @property
    def main_view(self):
        return patched_select_single(
            root=self, class_object=MainView, class_name='MainView',
            objectName='mainView')


class MainView(dialer_app.MainView):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MainView', objectName='mainView')

    @property
    def dialer_page(self):
        return self.wait_select_single(
            DialerPage, objectName='dialerPage', visible=True)

    @property
    def contacts_edit_page(self):
        return self.wait_select_single(
            DialerContactEditorPage,
            objectName='contactEditorPage',
            visible=True
        )

    def get_call_hold_failure_dialog(self):
        """Return dialog displayed to indicate an error activating call hold.
        :return: Dialog object or None if it is not found.
        """
        try:
            return patched_select_single(
                self, Dialog, class_name='Dialog',
                title='Call holding failure', visible=True)
        except autopilot.exceptions.StateNotFoundError:
            return None


class LiveCall(dialer_app.LiveCall):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'LiveCall', objectName='pageLiveCall')

    def get_stop_watch(self):
        """Return the stop watch object"""
        return self.wait_select_single(
            StopWatch, objectName='stopWatch', visible=True)

    def hangup_call(self):
        """Hangup the active call and wait for state to change."""
        self.click_hangup_button()
        try:
            self.active.wait_for(False)
        except autopilot.exceptions.StateNotFoundError:
            # live call page has closed so call has ended
            pass

    def hold_state(self):
        """Return hold state of current call."""
        return self.onHold

    def _get_hold_icon_name(self, hold_state):
        """Return the expected icon name for hold button in specified state."""
        if hold_state:
            return 'media-playback-start'
        else:
            return 'media-playback-pause'

    def toggle_hold(self):
        """Toggle the hold state of the current call by pressing hold button.
        :return: True if the hold state was toggled successfully,
                 False otherwise.
        """
        current_state = self.onHold
        next_state = not current_state
        hold_button = self._get_call_hold_button()
        hold_button.iconSource.wait_for(
            self._get_hold_icon_name(current_state), timeout=0)
        self.click_call_hold_button()
        try:
            self.onHold.wait_for(next_state, timeout=1)
            hold_button.iconSource.wait_for(
                self._get_hold_icon_name(next_state))
            return True
        except AssertionError:
            return False


class StopWatch(UbuntuUIToolkitCustomProxyObjectBase):

    def elapsed_time(self):
        """Return elapsed time as string"""
        return self.elapsed


class DialerPage(dialer_app.DialerPage):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'DialerPage', objectName='dialerPage')

    def _get_phone_number_input(self):
        """Return PhoneNumberInput object"""
        return self.select_single('PhoneNumberInput', visible=True)

    def get_entered_phone_number(self, unformatted=False):
        """Return the phone number entered into the dialer"""
        number = self._get_phone_number_input().displayText
        if unformatted:
            number = unformatted_phone_number(number)
        return number

    def _get_call_button(self):
        """Over-ride to ensure the button is visible and enabled."""
        return self.wait_select_single(
            objectName='callButton', visible=True, enabled=True)

    def _get_add_new_contact_button(self):
        """Return the 'add to contact' button."""
        return self.wait_select_single(
            'CustomButton',
            icon='contact-new',
            visible=True
        )

    def dial_number_add_new_contact(self, number, formatted_number):
        """Type in a phone number and click the 'add to contact' button."""
        self.dial_number(number, formatted_number)
        self.pointing_device.click_object(self._get_add_new_contact_button())


class DialerContactEditorPage(dialer_app.DialerContactEditorPage):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'DialerContactEditorPage',
            objectName='contactEditorPage')

    def _get_new_contact_number_field(self):
        return self.wait_select_single(
            'TextInputDetail', objectName='phoneNumber_0')

    def _get_new_contact_first_name_field(self):
        return self.wait_select_single(
            'TextInputDetail', objectName='firstName')

    def _get_new_contact_last_name_field(self):
        return self.wait_select_single(
            'TextInputDetail', objectName='lastName')

    def _get_contact_detail_phone_number(self):
        return self.select_single(objectName='base_phoneNumber_0')

    def _wait_for_number_field_to_focus(self):
        """Ensure the phone number field is focused."""
        self._get_contact_detail_phone_number().activeFocus.wait_for(True)

    def _write_contact_number(self, phone_number):
        """Type the contact number into the field."""
        phone_number_field = self._get_new_contact_number_field()
        phone_number_field.write(phone_number)

    def _write_first_name(self, first_name):
        """Type first name of the contact to save."""
        first_name_field = self._get_new_contact_first_name_field()
        first_name_field.write(first_name)

    def _write_last_name(self, last_name):
        """Type last name of the contact to save."""
        first_name_field = self._get_new_contact_last_name_field()
        first_name_field.write(last_name)

    def create_new_contact(self, first_name, last_name, phone_number=None):
        """
        Save a new contact by typing in its details.
        :param first_name: first name for the contact
        :param last_name: last name for the contact
        :param phone_number: phone number for the contact
        """
        if phone_number:
            self._write_contact_number(phone_number)
        else:
            self._wait_for_number_field_to_focus()

        self._write_first_name(first_name)
        self._write_last_name(last_name)
        self.save()


class DialerContactViewPage(dialer_app.DialerContactViewPage):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'DialerContactViewPage',
            objectName='contactViewPage')

    def get_contact_primary_number(self):
        """Return the phone number of the currently opened contact."""
        return self.wait_select_single(objectName='label_phoneNumber_0.0').text

    def get_title(self):
        """Return title of contact view page."""
        return self.select_single('PageHeader').title


class ContactsPage(dialer_app.ContactsPage):
    """Over-ride base helper to use wait_select_many."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'ContactsPage',
            objectName='contactsPage')

    def _get_sorted_contact_delegates(self):
        contact_delegates = wait_select_many(
            self, type_name='ContactDelegate', visible=True)
        return sorted(
            contact_delegates, key=lambda delegate: delegate.globalRect.y)


class Dialog(UbuntuUIToolkitCustomProxyObjectBase):
    """Generic CPO for dialer-app owned Dialog objects."""

    def _get_close_button(self):
        return self.select_single(objectName='closeDialog', visible=True)

    @autopilot.logging.log_action(logger.info)
    def close(self):
        self.pointing_device.click_object(self._get_close_button())
        self.wait_until_destroyed()


class PageWithBottomEdge(dialer_app.PageWithBottomEdge):
    """Over-ride bottom edge helper."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'PageWithBottomEdge')
