# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot import exceptions
from gallery_app.emulators import (
    events_view, photo_viewer, main_screen, picker_screen)
from gallery_app.emulators.gallery_utils import GalleryAppException
from retrying import retry
from ubuntuuitoolkit import UbuntuUIToolkitCustomProxyObjectBase

from ubuntu_system_tests.helpers.autopilot import validate_dbus_object
from ubuntu_system_tests.helpers.autopilot.list import order_by_x_coord
from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import file_system as fs

APP_PATH_ROOT = b'comubuntugallery'


class GalleryApplication(UbuntuUIToolkitCustomProxyObjectBase):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'GalleryApplication')

    @property
    def picker_screen(self):
        return self.select_single(objectName='pickerMainView', visible=True)


class MainScreen(main_screen.MainScreen):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MainScreen', objectName='overview')


class EventsView(events_view.EventsView):

    def is_image_displayed(self, image_name):
        try:
            return self._get_image_in_event_view(image_name) is not None
        except GalleryAppException:
            return False

    def pick_photo(self, file_path):
        """
        Picks the requested photo to be consumed by the requester

        :param file_path: identifier for the photo to pick
        """
        self._select_photo(file_path)
        self._click_pick_button()

    def _select_photo(self, file_path):
        """Selects a photo in EventView by long press."""
        photo = self._get_image_in_event_view(file_path)
        self.pointing_device.click_object(photo, press_duration=1)

    def _get_header_pick_button(self):
        """Returns the validate/pick button on the toolbar."""
        return self.get_qml_view().select_single(
            objectName='pickButton_button', visible=True)

    def _click_pick_button(self):
        """Taps the pick button in the header."""
        button = self._get_header_pick_button()
        self.pointing_device.click_object(button)

    def is_image_in_view(self, file_path):
        """ Indicate if the image is currently being displayed  """
        self.app.wait_select_single('MediaViewer', openCompleted=True)
        current = self.app.select_single('MainScreen').mediaCurrentlyInView
        return current == fs.get_file_path(file_path)

    @retry(stop_max_delay=context.shared.retry_time,
           retry_on_exception=lambda exception:
           isinstance(exception, exceptions.StateNotFoundError))
    def get_images_in_event_view(self, image_path):
        """ Return all the photos stored in the image path.
        :param image_path: the path to match with the images source
        """
        all_photos = []
        photos = self.app.select_many('UCUbuntuShape', visible=True)
        for photo in photos:
            images = photo.select_many('QQuickImage', status=1)
            imgs = [image for image in images if image_path in image.source]
            all_photos.extend(imgs)

        if not all_photos:
            raise exceptions.StateNotFoundError(class_name='QQuickImage',
                                                status=1)
        return order_by_x_coord(all_photos, include_off_screen=True)

    def get_media_list_for_event(self, event_number=0):
        event = self.get_event(event_number)
        return event.select_single('QQuickListView')

    def swipe_to_show_more_on_the_right(self, event_number=0):
        event = self.get_event(event_number)
        list = self.get_media_list_for_event()
        list.swipe_to_show_more_right(containers=[event])

    def swipe_to_show_more_on_the_left(self, event_number=0):
        event = self.get_event(event_number)
        list = self.get_media_list_for_event()
        list.swipe_to_show_more_left(containers=[event])


class PhotoViewer(photo_viewer.PhotoViewer):

    def get_opened_photo(self):
        """Returns the first visible opened photo."""
        return self.app.wait_select_single("SingleMediaViewer",
                                           objectName="openedMedia0",
                                           visible=True)

    def _get_image(self):
        """Return the visible (also opaque) QQuickImage."""
        photo = self.get_opened_photo()
        # companion QQuickImage has opacity 0.0
        return photo.wait_select_single(
            'QQuickImage', visible=True, globalRect=photo.globalRect,
            opacity=1.0)

    def get_image_source(self):
        return self._get_image().source

    @retry(stop_max_delay=context.shared.retry_time,
           retry_on_exception=lambda exception:
           isinstance(exception, exceptions.StateNotFoundError))
    def get_photo_component(self):
        return self.app.wait_select_single('GalleryPhotoComponent')


class PickerScreen(picker_screen.PickerScreen):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'PickerScreen',
            objectName='pickerMainView')
