# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
import logging

from unity8 import process_helpers

from ubuntu_system_tests.helpers.dbus_observer import DBusObserver
from ubuntu_system_tests.helpers.threads import StoppingThread

MEDIA_SCANNER = 'mediascanner-2.0'
MATCH_STRING = ("type='method_call',"
                "path='/com/canonical/MediaScanner2/Extractor',"
                "interface='com.canonical.MediaScanner2.Extractor',"
                "destination='com.canonical.MediaScanner2.Extractor',"
                "eavesdrop='true'")

logger = logging.getLogger(__name__)


def stop_media_scanner():
    """ Stop the media scanner process """
    if process_helpers.is_job_running(MEDIA_SCANNER):
        process_helpers.stop_job(MEDIA_SCANNER)


def start_media_scanner():
    """ Start the media scanner process """
    if not process_helpers.is_job_running(MEDIA_SCANNER):
        process_helpers.start_job(MEDIA_SCANNER)


class MediaScannerObserver(DBusObserver):
    """Class to observe when media scanner is active."""

    def __init__(self, stop, result, count):
        """
        Construct the MediaScannerObserver object.
        :param stop: Event object to signal when to stop waiting.
        :param result: Event object to signal when vibrator is activated.
        :param count: Expected number of events to count before stopping.

        """
        super().__init__(stop, result)
        self.target = count
        self.counter = 0
        self.log_frequency = count // 10
        self.next_log = self.log_frequency
        self.bus = dbus.SessionBus()
        self.bus.add_match_string_non_blocking(MATCH_STRING)
        self.bus.add_message_filter(self.on_event)

    def _cleanup(self):
        """Execute cleanup actions before the observer stops."""
        try:
            self.bus.remove_match_string_non_blocking(MATCH_STRING)
            self.bus.remove_message_filter(self.on_event)
        except ValueError:
            pass

    def on_event(self, bus, message):
        """
        Callback for when dbus event is observed.
        Set the result to True if it is the required method.

        :param bus: received dbus message bus object.
        :param message: received dbus message object.

        """
        if (message.get_interface() ==
                'com.canonical.MediaScanner2.Extractor' and
                message.has_member('ExtractMetadata')):
            self.counter += 1
            if self.counter >= self.next_log:
                logger.info('scanned {} files...'.format(self.counter))
                self.next_log += self.log_frequency
            if self.counter >= self.target:
                logger.info(
                    'All {} expected media files scanned.'.format(
                        self.target))
                self.result.set()
                self.stop()


def _start_mediascanner_observer(stop, result, count):
    """
    Start a mediascanner observer.

    :param stop: Event object to signal when to stop observing.
    :param count: Target number of media files to be scanned.
    :param result: Event object to signal when required number of mediascanner
                   events are observed.

    """
    observer = MediaScannerObserver(stop, result, count)
    observer.start()


def start_mediascanner_observer(result, count):
    """
    Listen for when required number of mediascanner events are observed.

    :param result: Event object to signal when required number of mediascanner
                   events are observed.
    :param count: Target number of media files to be scanned.
    :return: StoppingThread object which should be stopped by caller when
             required.

    """
    launch_thread = StoppingThread(
        target=_start_mediascanner_observer, args=[result, count])
    launch_thread.start()
    return launch_thread
