# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from collections import namedtuple
from math import ceil
import ubuntuuitoolkit

from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers.scopes.apps import (
    launch_application_from_apps_scope
)
from ubuntu_system_tests.helpers.unity8 import (
    close_app, launch_application_from_launcher
)

APP = 'messaging-app'
APP_NAME = 'Messaging'
APP_WIN_ID = 'messaging-app'

# Number of characters sent in a single SMS message
SMS_MESSAGE_SIZE = 153

MEDIA_TYPE_PICTURE = 'picture'
MEDIA_TYPE_VIDEO = 'video'
MEDIA_TYPE_CONTACT = 'contact'

Message = namedtuple('Message', ['body', 'attachments'])
Message.__new__.__defaults__ = (None, [])

Search = namedtuple('Search',
                    ['query', 'expected_thread_count',
                     'expected_thread_number'])
Search.__new__.__defaults__ = (None, None, None)


class Attachment:
    """
    Class to represent an attachment.

    This is a class rather than a namedtuple as it is neccessary to modify
    and save the file path of the attachment once it has been added to a
    message. This is not possible using namedtuples which are immutable.

    """
    def __init__(self, media_type, attach_method, identifier=None,
                 validation_file=None, validation_data=[]):
        """
        Configuration parameters for the attachment.

        Note that the attachment_path property is not set on construction of
        the Attachment object. The attachment_path property is set when the
        attachment has been added to the message and represents the file path
        of the attached media item.

        :param media_type: The type of media to be attached.
        :param attach_method: Method to use to add attachment to message.
        :param identifier: Optional unique id to allow the attachment be
                           selected using the attach_method function.
        :param validation_file: Optional file path for a file to compare
                                with the attachment received by recipient.
                                This is used for validation only.
        :param validation_data: Optional list of data items to search for in
                                the attachment received by recipient.
                                This is used for validation only.
        """
        self.attach_method = attach_method
        self.identifier = identifier
        self.media_type = media_type
        self.validation_file = validation_file
        self.validation_data = validation_data
        self.attachment_path = None


def launch_messaging_app():
    """
    Launch the messaging app from apps scope and return proxy object.

    :return: Proxy object for messaging application.

    """
    launch_application_from_apps_scope(APP_NAME)
    return get_messaging_app_proxy()


def close_messaging_app():
    """ Close the messaging app using task switcher """
    if is_messaging_app_running():
        close_app(APP_WIN_ID)


def is_messaging_app_running():
    """ Indicate if the messaging app is currently running """
    return processes.is_process_running(APP)


def get_messaging_app_proxy(
        base=ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):
    """
    Return messaging app proxy object from existing process.

    :return: Proxy object for messaging application.

    """
    if not base:
        return autopilot.get_proxy_object_for_existing_app(APP,
                                                           base=base)

    from ubuntu_system_tests.helpers.messaging._cpo import MainView
    proxy = autopilot.get_proxy_object_for_existing_app(APP)
    return autopilot.patched_wait_select_single(
        root=proxy, class_object=MainView, class_name='MainView')


def launch_messaging_app_from_launcher():
    """Drag out the launcher and tap the messaging app icon to start it."""
    launch_application_from_launcher(APP)
    return get_messaging_app_proxy()


def calculate_sent_message_count(body, size=SMS_MESSAGE_SIZE):
    """
    Calculate the number of sms messages required to send the message.

    :param body: The message body
    :param size: Size in characters of the required message chunks
    :return: Number of messages required to send message body

    """
    return ceil(len(body) / size)


def split_into_received_messages(body, concatenate, size=SMS_MESSAGE_SIZE):
    """
    Break down the specified message into messages that are received by the
    telephony service.

    If concatenate is True then body will be returned as 1 single
    chunk. If concatenate is False then body will be broken down into
    multiple message chunks of specified size.

    :param body: The message body to split into chunks
    :param concatenate: Indicate whether sms concatentation is supported by the
                        receiver
    :param size: Size in characters of the required message chunks
    :return: List of messages of specified size

    """
    if concatenate:
        return [body]
    return [body[pos:size+pos] for pos in range(0, len(body), size)]


def focus_message_input_field(messaging_app_raw_proxy, pointer):
    """
    Tap on the new message text field and wait for it to take focus.
    :param messaging_app_raw_proxy: Messaging app proxy object without any
                                    custom emulator base.
    :param pointer: the pointer object used to tap on object
    """
    message_field = messaging_app_raw_proxy.wait_select_single(
        objectName='messageTextArea')
    pointer.click_object(message_field)
    message_field.activeFocus.wait_for(True)
    return message_field
