# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import time

import autopilot
import ubuntuuitoolkit as uitk

from autopilot.exceptions import StateNotFoundError
from urllib.parse import urlparse
from messaging_app import emulators as messaging_emulators

from ubuntu_system_tests.helpers.autopilot import (
    patched_select_single, patched_wait_select_single, validate_dbus_object,
    wait_select_many
)
from ubuntu_system_tests.helpers.autopilot.list import order_by_y_coord
from ubuntu_system_tests.helpers.autopilot.object import (
    wait_until_is_not_moving)
from ubuntu_system_tests.helpers.messaging import (
    MEDIA_TYPE_PICTURE, MEDIA_TYPE_VIDEO, MEDIA_TYPE_CONTACT)
from ubuntu_system_tests.helpers.system_settings import SIM_1, SIM_2

logger = logging.getLogger(__name__)

APP_PATH_ROOT = b'MessagingApp'
DEFAULT_SIM_MSG = 'Change all Messaging associations to SIM {sim}?'


class MainView(messaging_emulators.MainView):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MainView')

    def get_main_page(self):
        """Override base class method to use patched select method."""
        return patched_wait_select_single(
            root=self, class_object=MainPage, class_name='MainPage')

    @autopilot.logging.log_action(logger.info)
    def reveal_bottom_edge_page(self):
        """Bring the bottom edge page to top of screen."""
        start_x = (self.globalRect.x +
                   (self.globalRect.width * 0.5))
        start_y = (self.globalRect.y + self.height)
        stop_y = start_y - (self.height * 0.7)
        self.pointing_device.drag(start_x, start_y,
                                  start_x, stop_y)
        self.composingNewMessage.wait_for(True)

    @autopilot.logging.log_action(logger.info)
    def type_contact_name_and_select_num(self, name, number):
        self.type_contact_name(name)
        # Wait for the list to settle
        time.sleep(1)
        # Now select the right number from the list
        self.pointing_device.click_object(
            self.get_mobile_number_label(number)
        )

    def type_contact_name(self, name):
        text_entry = self.get_newmessage_multirecipientinput()
        text_entry.enter_text(name)

    def get_mobile_number_label(self, number):
        """Return first matching label displayed."""
        labels = wait_select_many(
            self, type_name='UCLabel', text='Mobile {}'.format(number))
        return order_by_y_coord(labels)[0]

    def detect_select_sim_prompt(self):
        """Detect if the select SIM dialog is shown."""
        try:
            self.wait_select_single('Dialog')
            return True
        except StateNotFoundError:
            return False

    def get_sim_label(self, sim_number):
        return self.wait_select_single(
            'UCLabel', text='SIM {}'.format(sim_number)
        )

    def get_thread_list(self):
        return self.select_single(objectName='threadList')

    def _get_attachment_button(self):
        """Returns the 'attach media' button."""
        return self.get_messages_page().select_single(
            objectName='attachButton')

    def attach_media(self, media_type):
        """Press the attach button on composing message and select the required
        media type. Return the content picker dialog.
        :param media_type: The type of media required, picture, video or
                           contact.
        :return: Content picker dialog proxy object.
        """
        self.pointing_device.click_object(self._get_attachment_button())
        panel = self._get_attachment_panel()
        wait_until_is_not_moving(panel)
        panel.choose_media_type(media_type)
        return self._get_content_picker()

    def _get_attachment_panel(self):
        """Return proxy object for attachment panel."""
        return self.wait_select_single(AttachmentPanel)

    def _get_content_picker(self):
        return self.wait_select_single(ContentPeerPicker13, visible=True)

    def type_message(self, message):
        """Select and type message in new message text area in messages page

        :parameter message: the message to type
        """
        text_entry = self.get_newmessage_textarea()
        text_entry.write(message)
        if text_entry.text != message:
            self.logger.info('typed: "{}" expected: "{}"'.format(
                text_entry.text, message
            ))

    def _get_thread_delegates(self):
        """Get all thread delegates in main screen."""
        return self.get_thread_list().select_many('ThreadDelegate')

    def get_visible_threads_count(self):
        """Return the number of visible threads."""
        return len([t for t in self._get_thread_delegates() if t.height != 0])

    def _is_search_mode(self):
        """Return bool indicating if the App header is in search mode."""
        return self.get_main_page().state == 'search'

    def _get_search_box(self):
        """Return the message searchbox."""
        """Return the contacts search box."""
        return self.select_single(uitk.TextField, objectName='searchField')

    def search(self, text):
        """Tap the search icon and write the requestion text."""
        if not self._is_search_mode():
            self.click_header_action('searchAction')
        search_box = self._get_search_box()
        search_box.clear()
        search_box.write(text)

    def clear_search(self):
        """Clear the search box."""
        if self._is_search_mode():
            self._get_search_box().clear()

    def is_thread_for_number_visible(self, number):
        """
        Return bool indicating the thread for `number` is visible on screen.
        """
        thread = self.get_thread_from_number(number)
        return thread.height > 0

    def _get_popup_paste_button(self):
        """
        Return the 'paste' button on the new message field' contextual popup.
        """
        paste_button = self.wait_select_single(iconName='edit-paste')
        paste_button.opacity.wait_for(1.0)
        return paste_button

    def paste_text_in_message_field(self):
        """
        Start new message and paste the copied text into the new message field.
        """
        self.start_new_message()
        text_entry = self.get_newmessage_textarea()
        # Double click the empty field for the copy popover to appear
        self.pointing_device.click_object(text_entry)
        self.pointing_device.click_object(text_entry)
        # Click the paste button on the popup
        self.pointing_device.click_object(self._get_popup_paste_button())
        return text_entry

    def _get_sim_confirmation_dialog(self, sim):
        """Return default confirmation dialog for specified SIM."""
        return patched_select_single(
            self, class_object=Dialog, class_name='Dialog',
            text=DEFAULT_SIM_MSG.format(sim=sim))

    def get_sim1_default_confirmation_dialog(self):
        """Return SIM 1 default confirmation dialog."""
        return self._get_sim_confirmation_dialog(SIM_1)

    def get_sim2_default_confirmation_dialog(self):
        """Return SIM 2 default confirmation dialog."""
        return self._get_sim_confirmation_dialog(SIM_2)

    def _get_sim_selector(self):
        """Return selector object which controls the SIM labels."""
        return self.select_single(objectName='headerSectionsItem')

    def get_selected_sim(self):
        """Return label of currently selected SIM either 'SIM 1' or 'SIM 2'."""
        index = self._get_sim_selector().selectedIndex
        button = self.select_single(
            objectName='section_button_label_{}'.format(index))
        return button.text

    @autopilot.logging.log_action(logger.info)
    def select_sim(self, sim):
        """Select the SIM to use using selection tabs.
        :param sim: SIM to use, either '1' or '2'.
        """
        selector = self._get_sim_selector()
        label = selector.select_single(
            text='SIM {}'.format(sim))
        label_index = int(label.objectName[-1])
        self.pointing_device.click_object(label)
        selector.selectedIndex.wait_for(label_index)

    def _get_back_button(self):
        """Return the button to return to main screen."""
        if self.composingNewMessage:
            object_name = 'cancel_button'
        else:
            object_name = 'back_button'
        return self.select_single(
            objectName=object_name, visible=True, enabled=True)

    @autopilot.logging.log_action(logger.info)
    def go_back(self):
        """Press back button on header."""
        self.pointing_device.click_object(self._get_back_button())

    def delete_message(self, text):
        """Delete message and wait until the message is deleted."""
        message = self.get_message(text)
        super().delete_message(text)
        message.wait_until_destroyed()


class MainPage(messaging_emulators.MainPage):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MainPage')


class MultiRecipientInput(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MultiRecipientInput')

    def enter_text(self, text):
        contact_search_input = self.select_single(
            'TextField', objectName='contactSearchInput'
        )
        contact_search_input.write(text)


class MessageDelegateFactory(messaging_emulators.MessageDelegateFactory):
    """Class to get all the messages from the app."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MessageDelegateFactory')

    def get_mms_content_file_path(self):
        """Return MMSBase for MMS message."""
        file_url = self.select_single(MMSBase).get_media_file_url()
        return urlparse(file_url).path

    def get_message_bubble(self):
        return self.wait_select_single(MessageBubble, visible=True)


class MessageBubble(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent the message bubble for a displayed message."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MessageBubble')

    def wait_until_sent(self, timeout):
        sent_time = self.select_single(objectName='messageDate')
        sent_time.visible.wait_for(True, timeout=timeout)


class MMSBase(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to examine MMS content."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MMSBase')

    def get_media_file_url(self):
        """Return the file url of the media content."""
        image = self.get_media_container().select_single('QQuickImage')
        return image.get_properties()['source']

    def get_media_container(self):
        """Return container object for media content."""
        return self.select_single('UCUbuntuShape', visible='True')


class Messages(messaging_emulators.Messages):
    """Class to examine content of messages displayed in editor."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'Messages')

    def get_attachments(self):
        """Return list of displayed attachment objects."""
        attachment_types = [
            'ThumbnailImage', 'ThumbnailVideo', 'ThumbnailContact']
        attachments = []
        for attachment_type in attachment_types:
            for attachment in self.select_many(attachment_type, visible=True):
                if ('filePath' in attachment.get_properties() and
                        len(attachment.filePath) > 0):
                    attachments.append(attachment)
        return order_by_y_coord(attachments)

    def get_attachment_count(self):
        """Return a count of the displayed attachments in message editor."""
        return len(self.get_attachments())

    def get_matching_messages(self, text, sender):
        """Return matching messages for specified sender in thread."""
        return self.select_many(
            'RegularMessageDelegate', textMessage=text, senderId=sender,
            visible=True)

    def is_message_displayed(self, text, phone_number):
        """
        Check if message with phone_number and text is displayed in thread list

        :param text: The body of the message we are expecting
        :param number: The sending phone number
        :return: True if received, otherwise False

        """
        return bool(self.get_matching_messages(text, phone_number))

    def how_many_messages_displayed(self, text, phone_number):
        """
        Count how many messages are displayed with specified text and
        phone number in thread list.

        :param text: The body of the message we are expecting
        :param number: The sending phone number
        :return: Number of matching messages

        """
        return len(self.get_matching_messages(text, phone_number))

    @property
    def title(self):
        """Return title of currently displayed messages."""
        return self.select_single('PageHeader').title


class AttachmentPanel(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to select attachment type from panel."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'AttachmentPanel')

    def _get_picture_button(self):
        return self._get_button(MEDIA_TYPE_PICTURE)

    def _get_video_button(self):
        return self._get_button(MEDIA_TYPE_VIDEO)

    def _get_contact_button(self):
        return self._get_button(MEDIA_TYPE_CONTACT)

    def _get_button(self, media_type):
        button_name = media_type + 'Button'
        return self.select_single(objectName=button_name)

    def choose_media_type(self, media_type):
        """Select specified media type.
        :param media_type: Either picture, video or contact.
        """
        self.pointing_device.click_object(self._get_button(media_type))

    def choose_picture(self):
        """Select picture type attachment."""
        self.pointing_device.click_object(self._get_picture_button())

    def choose_video(self):
        """Select video type attachment."""
        self.pointing_device.click_object(self._get_video_button())

    def choose_contact(self):
        """Select contact type attachment."""
        self.pointing_device.click_object(self._get_contact_button())


class ContentPeerPicker13(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to pick a content source application."""

    def pick_app(self, app_name):
        """
        Taps the requested app's icon from the content picker screen
        :param app_name: name of the app to tap

        """
        app_icon = order_by_y_coord(wait_select_many(
            self, text=app_name))[0].get_parent()
        self.pointing_device.click_object(app_icon)


class Dialog(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Class to represent default SIM selection dialog."""

    @autopilot.logging.log_action(logger.info)
    def confirm(self):
        self.pointing_device.click_object(self._get_yes_button())
        self.wait_until_destroyed()

    @autopilot.logging.log_action(logger.info)
    def deny(self):
        self.pointing_device.click_object(self._get_no_button())
        self.wait_until_destroyed()

    def get_dont_ask_again(self):
        return self._get_checkbox().checked

    def set_dont_ask_again(self, state):
        checkbox = self._get_checkbox()
        if checkbox.checked is not state:
            checkbox.change_state()

    def _get_checkbox(self):
        return self.select_single('CheckBox')

    def _get_yes_button(self):
        return self.select_single('Button', text='Yes')

    def _get_no_button(self):
        return self.select_single('Button', text='No')
