# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import configparser
import fixtures
import os
import shutil
import subprocess

from messaging_app import fixture_setup

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import system_settings

MESSAGING_APP_CONF = 'com.ubuntu.messaging-app/MessagingApp.conf'

SECTION_GENERAL = 'General'
SECTION_DUALSIM = 'DualSim'
KEY_HINT = 'hintNecessary'
KEY_IGNORE_FIRST_TIME_DIALOG = 'mainViewIgnoreFirstTimeDialog'
KEY_MESSAGES_DONT_ASK = 'messagesDontAsk'
FALSE = 'false'
TRUE = 'true'


class MessagingTestEnvironment(fixtures.Fixture):
    """Fixture used to clean up the phone and messaging environment"""

    def setUp(self):
        super(MessagingTestEnvironment, self).setUp()
        self.useFixture(RespawnMessagingApp())
        self.useFixture(UseEmptySmsHistory())
        self.useFixture(fixture_setup.RespawnService())
        self.useFixture(DisableSMSGestureHint())
        self.useFixture(SetDefaultSIM())


class MessagingConfigBaseFixture(fixtures.Fixture):
    """Base fixture used to set messaging config"""

    CONFIG_FILE = os.path.join(fs.DIR_HOME_CONFIG, MESSAGING_APP_CONF)

    def setUp(self):
        super().setUp()
        config_dir = os.path.dirname(self.CONFIG_FILE)
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)

    def get_config_value(self, section, key):
        return self._get_config()[section][key]

    def set_config_value(self, section, key, value):
        config = self._get_config()
        if section not in config:
            config.add_section(section)
        config[section][key] = value
        with open(self.CONFIG_FILE, 'w') as configfile:
            config.write(configfile)

    def _get_config(self):
        config = configparser.ConfigParser(strict=False)
        config.optionxform = str  # preserve case of key names
        config.read(self.CONFIG_FILE)
        return config


class DisableSMSGestureHint(MessagingConfigBaseFixture):
    """Fixture used to disable SMS gesture hint"""

    def setUp(self):
        super().setUp()
        original_value = self.get_hint_value()
        self.addCleanup(self.set_hint_value, original_value)
        self.set_hint_value(FALSE)

    def get_hint_value(self):
        try:
            value = self.get_config_value(SECTION_GENERAL, KEY_HINT)
        except KeyError:
            value = TRUE
        return value

    def set_hint_value(self, value):
        self.set_config_value(SECTION_GENERAL, KEY_HINT, value)


class SetDualSIMPromptConfig(MessagingConfigBaseFixture):
    """Fixture used to set dual-sim prompt configuration."""

    def __init__(self, ignore_first_time=TRUE, ask_messages=FALSE):
        self.ignore_first_time = ignore_first_time
        self.ask_messages = ask_messages

    def setUp(self):
        super().setUp()
        ignore_first_time = self.get_ignore_first_time()
        ask_messages = self.get_ask_messages()
        self.addCleanup(self.set_ignore_first_time, ignore_first_time)
        self.addCleanup(self.set_ask_messages, ask_messages)
        self.set_ignore_first_time(self.ignore_first_time)
        self.set_ask_messages(self.ask_messages)

    def get_ignore_first_time(self):
        try:
            value = self.get_config_value(SECTION_DUALSIM,
                                          KEY_IGNORE_FIRST_TIME_DIALOG)
        except KeyError:
            value = FALSE
        return value

    def get_ask_messages(self):
        try:
            value = self.get_config_value(SECTION_DUALSIM,
                                          KEY_MESSAGES_DONT_ASK)
        except KeyError:
            value = FALSE
        return value

    def set_ignore_first_time(self, value):
        self.set_config_value(
            SECTION_DUALSIM, KEY_IGNORE_FIRST_TIME_DIALOG, value)

    def set_ask_messages(self, value):
        self.set_config_value(SECTION_DUALSIM, KEY_MESSAGES_DONT_ASK, value)


class SetDefaultSIM(fixtures.Fixture):
    """Fixture to set the default sim for messaging."""

    def __init__(self, sim=system_settings.SIM_1):
        self.sim = sim

    def setUp(self):
        super().setUp()
        original_value = system_settings.get_default_sim_for_messages()
        self.addCleanup(
            system_settings.set_default_sim_for_messages, original_value)
        system_settings.set_default_sim_for_messages(self.sim)


class UseEmptySmsHistory(fixture_setup.FillCustomSmsHistory):
    """Fixture used to make the history service run with a clean data db"""
    def setUp(self):
        super().setUp()

    def _prepare_history_data(self):
        pass


class RespawnMessagingApp(fixtures.Fixture):
    """ As the Messaging app is not stopped properly by using the
    ubuntu-app-stop command, this fixture ensures the app will be stopped at
    the end of the test
    """
    app_name = 'messaging-app'

    def setUp(self):
        super().setUp()
        self.addCleanup(self._kill_app_to_respawn)
        self._kill_app_to_respawn()

    def _kill_app_to_respawn(self):
        """ This method is used to kill either a process or an app which is not
        stopped properly by using ubuntu-app-stop"""
        subprocess.call(['pkill', '-f', self.app_name])


class FillCustomSmsHistory(fixtures.Fixture):

    history_db = "history.sqlite"
    prefilled_history = os.path.join(fs.DIR_TEST_DATA_DATABASES, history_db)
    database_path = '/tmp/' + history_db

    def setUp(self):
        super().setUp()
        self.addCleanup(self._clear_test_data)
        self.addCleanup(self._kill_service_to_respawn)
        self._clear_test_data()
        self._prepare_history_data()
        self._kill_service_to_respawn()
        self._start_service_with_custom_data()

    def _prepare_history_data(self):
        if os.path.exists(self.prefilled_history):
            shutil.copy(self.prefilled_history, self.database_path)

    def _clear_test_data(self):
        if os.path.exists(self.database_path):
            os.remove(self.database_path)

    def _kill_service_to_respawn(self):
        subprocess.call(['pkill', 'history-daemon'])

    def _start_service_with_custom_data(self):
        os.environ['HISTORY_SQLITE_DBPATH'] = self.database_path
        with open(os.devnull, 'w') as devnull:
            subprocess.Popen(['history-daemon'], stderr=devnull)
