
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
import shutil

from ubuntu_system_tests.helpers import file_system as fs

logger = logging.getLogger(__name__)

DEFAULT_XDG_RUNTIME_DIR = '/run/user/1000'


class MTPObjBase:
    """ Base class used to represent objects that are copied through mtp"""

    def __init__(self, remote_helper):
        self.remote_helper = remote_helper
        self.local_src_path = ""
        self.local_dst_path = ""
        self.remote_src_path = ""
        self.remote_dst_path = ""

    def validate_remote_copy(self):
        """  Validate the src and dst paths are defined """
        if not self.remote_src_path or not self.remote_dst_path:
            raise RuntimeError("Remote copy cannot be done, remote_src_path "
                               "and remote_mtp_path have to be defined")

    def validate_send_to_host(self):
        """  Validate the src and dst paths are defined """
        if not self.local_src_path or not self.remote_src_path:
            raise RuntimeError("Send to host cannot be done, local_src_path "
                               "and remote_src_path have to be defined")


class MTPFile(MTPObjBase):
    """ This class represents a file which is copied through mtp"""

    def __init__(self, remote_helper):
        super(MTPFile, self).__init__(remote_helper)
        self.sha1 = ""

    @property
    def name(self):
        """  Retrieve the name based of the paths defined """
        return os.path.basename(self.local_src_path) or \
            os.path.basename(self.remote_src_path) or \
            os.path.basename(self.remote_dst_path) or \
            os.path.basename(self.local_dst_path)

    def clean(self):
        """ Clean the file in the different places where the file has been
        copied. This method is used mainly for the clean up of the file  """
        fs.delete_file(self.local_src_path)
        if self.remote_src_path:
            self.remote_helper.remove_file(self.remote_src_path)
        if self.remote_dst_path:
            self.remote_helper.remove_file(self.remote_dst_path)
        fs.delete_file(self.local_dst_path)

    def remote_copy(self, remote_dst_path=None):
        """ Copy the file in the remote host from src to dst
        :param remote_dst_path: the remote destination in case it has not been
        defined previously
        """
        if remote_dst_path:
            self.remote_dst_path = remote_dst_path
        self.validate_remote_copy()
        logger.info("Copying remote file from {src_path} to {mtp_path}".
                    format(src_path=self.remote_src_path,
                           mtp_path=self.remote_dst_path))
        self.remote_helper.copy_file(self.remote_src_path,
                                     self.remote_dst_path)

    def get_remote_dst_sha1(self):
        """ Get the sha1 digest calculated in the remote host for the file
        located in the remote_dst_path
        """
        if not self.remote_dst_path:
            raise RuntimeError("Attribute remote_dst_path has to de defined")
        return self.remote_helper.get_sha1(self.remote_dst_path)

    def send_to_host(self, path=None):
        """ Send the file from the device to the ubuntu host. The
        remote_src_path will be updated in case the path is defined
        :param path: The path in the remote host where the file is copied
        """
        if path:
            self.remote_src_path = os.path.join(path, self.name)
        self.validate_send_to_host()
        logger.info("Sending file to remote host from {local_path} to "
                    "{remote_path}".format(local_path=self.local_src_path,
                                           remote_path=self.remote_src_path))
        self.remote_helper.put(self.local_src_path, self.remote_src_path)

    def compare(self, file_path):
        """ Compare the local file to the file located in the path
        :return:True if the files are equal, false otherwise
        """
        return fs.compare_files(self.local_src_path, file_path)


class MTPDir(MTPObjBase):
    """ This class represents a directory which is copied through mtp"""

    def __init__(self, remote_helper):
        super(MTPDir, self).__init__(remote_helper)

    @property
    def name(self):
        """  Retrieve the name based of the paths defined """
        return os.path.basename(self.local_src_path) or \
            os.path.basename(self.remote_src_path) or \
            os.path.basename(self.remote_dst_path) or \
            os.path.basename(self.local_dst_path)

    def clean(self):
        """ Clean the dir in the different places where the dir has been
        copied. This method is used mainly for the clean up of the dir  """
        fs.remove_dir(self.local_src_path)
        if self.remote_src_path:
            self.remote_helper.remove_dir(self.remote_src_path)
        if self.remote_dst_path:
            self.remote_helper.remove_dir(self.remote_dst_path)
        fs.remove_dir(self.local_dst_path)

    def remote_copy(self, remote_dst_path=None):
        """ Copy the dir in the remote host from src to dst
        :param remote_dst_path: the remote destination in case it has not been
        defined previously
        """
        if remote_dst_path:
            self.remote_dst_path = remote_dst_path
        self.validate_remote_copy()
        logger.info("Copying remote directory from {src_path} to {mtp_path}".
                    format(src_path=self.remote_src_path,
                           mtp_path=self.remote_dst_path))
        self.remote_helper.copy_dir(self.remote_src_path,
                                    self.remote_dst_path)


class MTPObjFactory:
    """ A factory class used to create the mtp entities in the local and
    remote host """

    def __init__(self, remote_helper, clean_up):
        """
        :param remote_helper: The helper to interact with the remote host
        :param clean_up: The method used to clean up the objects created
        """
        self.remote_helper = remote_helper
        self.clean_up = clean_up

    def new_file(self):
        """ Create an MTPFile object and add a clean up for it """
        file = MTPFile(self.remote_helper)
        self.clean_up(file.clean)
        return file

    def create_random_file(self, dir, ext='', sha1=True):
        """ Create a MTPFile object with random name
        :param dir: The dir where the file is created
        :param ext: The extension of the file to be created
        :param sha1: if True, the sha1 is calculated and stores in the file
        :return: The created file with the local src path already set
        """
        file = self.new_file()
        file.local_src_path = fs.create_random_file(dir, ext)
        if sha1:
            file.sha1 = fs.calculate_file_sha1(file.local_src_path)
        return file

    def create_file(self, src_file, temp=True):
        """ Create a MTPFile object based on a real file
        :param src_file: The real file used to create the MTPFile obj
        :param temp: if True, the file is copied to a temp dir. The temp is
        used to avoid the original file is deleted in the clean-up
        :return: The created file object with the local src path already set
        """
        file = self.new_file()
        file.local_src_path = src_file
        if temp:
            file.local_src_path = os.path.join(fs.DIR_TEMP, file.name)
            shutil.copy(src_file, file.local_src_path)
        return file

    def create_remote_file(self, path=None, name=None, ext='', sha1=True,
                           kb=0, mb=0, gb=0):
        """ Create a file in the remote host
        :param path: The path where the file is created, if None, the file will
        be created in a temp dir
        :param name: The name of the file to be created, if None, will be used
        a random name
        :param ext: the extension of the file
        :param sha1: if True, the sha1 is calculated and stores in the file
        :param kb: The kilobytes of the file to be created
        :param mb: The megabytes of the file to be created
        :param gb: The gigabytes of the file to be created
        :return: The created file with random content and size equals to
        kb kilobytes + mb megabytes + gb gigabytes
        """
        file = self.new_file()
        file_path = path or fs.DIR_TEMP
        file_name = name or fs.get_random_string()
        file.remote_src_path = os.path.join(file_path, file_name + ext)
        self.remote_helper.create_random_file(file.remote_src_path, kb=kb,
                                              mb=mb, gb=gb)
        if sha1:
            file.sha1 = self.remote_helper.get_sha1(file.remote_src_path)
        return file

    def new_dir(self):
        """ Create an MTPDir object and add a clean up for it """
        dir = MTPDir(self.remote_helper)
        self.clean_up(dir.clean)
        return dir

    def create_remote_dir(self, path=None, name=None):
        """
        Create a dir in the remote host
        :param path: the name path where the new dir has to be created
        :param name: the name of the dir, in case the name is not defined,
        a random name is used
        """
        dir = self.new_dir()
        dir_path = path or fs.DIR_TEMP
        dir_name = name or fs.get_random_string()
        dir.remote_src_path = os.path.join(dir_path, dir_name)
        self.remote_helper.create_dir(dir.remote_src_path)
        return dir


class MTPHelper:

    def __init__(self, remote_helper):
        self.remote_helper = remote_helper
        self.mtp_dir = self._calculate_mtp_dir()
        self.mtp_media_dir = self._calculate_mtp_media_dir()

    def get_mtp_path(self, *paths):
        """ Retrieve the mtp path following the desired paths
        :param paths: A list of subdir to the destination path
        :return: The mtp path already escaped to the destination path
        """
        return '"' + os.path.join(self.mtp_dir, *paths) + '"'

    def get_mtp_media_path(self, *paths):
        """ Retrieve the mtp path following the desired paths
        :param paths: A list of subdir to the destination path
        :return: The mtp path already escaped to the destination path
        :raises: RuntimeError: When the media directory is not detected
        """
        if not self.mtp_media_dir:
            raise RuntimeError("No mtp media directory detected")
        return '"' + os.path.join(self.mtp_media_dir, *paths) + '"'

    def _calculate_mtp_media_dir(self):
        """ Get the path in the ubuntu host machine which maps to the mtp
        media dir. Use a random file to identify the device internal storage
        destination.
        """
        try:
            docs_media_dir = fs.get_media_folder_dir(fs.DIR_DOCUMENTS)
            file = fs.create_random_file(docs_media_dir, ext='.tmp')
            docs_mtp_dir = self._get_mtp_dir(fs.DIR_DOCUMENTS,
                                             os.path.basename(file))
            os.remove(file)
            return os.path.dirname(docs_mtp_dir)
        except RuntimeError:
            return None

    def _calculate_mtp_dir(self):
        """ Get the path in the ubuntu host machine which maps to the mtp dir.
        Use a random file to identify the device internal storage destination.
        """
        file = fs.create_random_file(fs.DIR_HOME_DOCUMENTS, ext='.tmp')
        docs_mtp_dir = self._get_mtp_dir(fs.DIR_DOCUMENTS,
                                         os.path.basename(file))
        os.remove(file)
        return os.path.dirname(docs_mtp_dir)

    def _get_mtp_dir(self, dir_to_map, filename):
        """ Determine the full_path in the ubuntu host machine which maps the
        file and directory in the device
        :param dir_to_map: The directory to map
        :param filename: The name of the file to be mapped
        :return: the full path that maps the dir_to_map and filename in the
                 device
        """
        cmd_line = "python3 -c \"import glob; import os; " \
                   "print(glob.glob(os.getenv('XDG_RUNTIME_DIR', " \
                   "'{default}') + '/gvfs/*/*/{dir}/{filename}')[0])\"".\
            format(default=DEFAULT_XDG_RUNTIME_DIR,
                   dir=dir_to_map,
                   filename=filename)

        file_path = self.remote_helper.run(cmd_line).strip()
        if file_path:
            return os.path.dirname(file_path)
        return None

    def replicate_file(self, src_file, dst_dir, dst_files):
        """ Copy the src file to the dst directory with the different names
        stored in the dst_files variable
        :param src_file: the source file to replicate
        :param dst_dir: the destination directory to copy the files
        :param dst_files: the target names to use
        """
        for file in dst_files:
            src_file.remote_copy(os.path.join(dst_dir.remote_src_path, file))
