# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import subprocess
import socket
from urllib import request
from ubuntu_system_tests.helpers import(
    ofono,
    wait_until
)
from ubuntu_system_tests.helpers.indicators import(
    network as network_indicator,
)

TEST_HOST = 'http://people.canonical.com/~platform-qa/page1.html'


def check_connection(host=TEST_HOST):
    """Try to connect to the requested host and return True if successful,
    otherwise return False."""
    try:
        request.urlopen(host, timeout=1)
        return True
    except (request.URLError, socket.error):
        return False


def turn_on_wifi():
    """Turn on wifi from command line."""
    subprocess.call(['rfkill', 'unblock', 'wlan'])


def turn_off_wifi():
    """Turn off wifi from command line."""
    subprocess.call(['rfkill', 'block', 'wlan'])


def turn_on_data_connection(sim_slot=0):
    """Turn on data connection from command line."""
    turn_off_data_connection()
    if ofono.is_sim_registered(sim_slot):
        _run_system_command_no_stdout(
            '/usr/share/ofono/scripts/enable-gprs /ril_{}'.format(sim_slot))


def turn_off_data_connection():
    """Turn off data connection from command line."""
    number_of_modems = ofono.get_number_of_modems()
    for i in range(number_of_modems):
        if ofono.is_sim_registered(i):
            _run_system_command_no_stdout(
                '/usr/share/ofono/scripts/disable-gprs /ril_{}'.format(i))


def _is_connected(connection_type):
    """
    Return bool indicating if connection of the requested type is established.
    """
    states = subprocess.check_output(['nmcli', 'device'],
                                     universal_newlines=True).splitlines()
    for line in states:
        if connection_type in line:
            if 'connected' in line and 'disconnected' not in line:
                return True
    return False


def is_mobile_data_enabled():
    """Return bool indicating if data connection is enabled."""
    return _is_connected(connection_type='gsm')


def is_wifi_connected():
    """Return bool indicating if wifi is connected."""
    return _is_connected(connection_type='wifi')


def _run_system_command_no_stdout(cmd):
    """Run the given command with subprocess and divert the stdout to
    /dev/null."""
    dev_null = open(os.devnull, 'w')
    subprocess.call(cmd.split(), stdout=dev_null, stderr=subprocess.STDOUT)


def ensure_internet_connectivity(timeout):
    """Wait for internet availability for the specified time, else
    fail the test."""
    return wait_until(check_connection, timeout=timeout, period=1)


def turn_on_wifi_ensure_connectivity(test_case, timeout):
    """Enable wifi and wait for internet to connect successfully."""
    network_indicator.turn_on_wifi()
    test_case.assertTrue(ensure_internet_connectivity(timeout=timeout))
