# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import functools
import ubuntuuitoolkit as uitk

from autopilot.exceptions import StateNotFoundError

from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers.autopilot import (
    is_element, patched_select_single)

NeedsImplementationError = functools.partial(
    NotImplementedError,
    "This method should be implemented by derived class."
)


class NotificationBase(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    """Base class to be used for all notification objects."""

    def get_button(self, text, **kwargs):
        """Get button with required name and properties."""
        return self.select_single('Button', text=text, visible=True, **kwargs)

    def has_button(self, text, **kwargs):
        """Return True if button with expected text exists."""
        return is_element(self.get_button, text, **kwargs)

    def has_buttons(self, buttons):
        """Return True if all listed buttons are present."""
        for button in buttons:
            if not self.has_button(button):
                return False
        return True

    def get_button_count(self):
        """Return number of buttons displayed."""
        return len(self.select_many('Button', visible=True))

    def validate_buttons(self, buttons):
        """Check that all required buttons are displayed and no more."""
        return (self.has_buttons(buttons) and
                self.get_button_count() == len(buttons))

    def has_icon(self):
        """Return True if an icon is displayed."""
        return len(self.iconSource) > 0

    def dismiss(self):
        """
        Method to dismiss a notification.

        This should be overridden in the derived class.

        """
        raise NeedsImplementationError()


class Notifications(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    """Class to return notification objects of specific types."""

    def _get_notification(self, notification_class, obj_name='notification1',
                          **kwargs):
        """
        Return a currently displayed notification object, or None.

        :param notification_class: The class name of notification required.
        :param obj_name: The name of the required notification object.
        :param kwargs: Any othery keyword parameters to identify required
                       notification.
        :return: Notification object meeting required criteria, or None if
                 no matching objects are found.

        """
        try:
            return patched_select_single(
                self, class_object=notification_class,
                class_name='Notification', objectName=obj_name, **kwargs)
        except StateNotFoundError:
            return None

    def get_notification(self, notification_class, timeout=60, stop_event=None,
                         validator=None, **kwargs):
        """
        Wait for notification of required type to appear and return it.

        This could potentially be run for a long time, waiting for reply
        messages to be received on the device. For this reason it has been
        implemented using a looped select_single() so that it can be
        cancelled easily at any point. If wait_select_single() was used with
        a long timeout, there would be no easy way to stop this call if the
        caller needed to be stopped, as is the case with the notification
        helper observer.

        :param notification_class: CPO class to use.
        :param timeout: How long to wait for the notification.
        :param stop_event: threading.Event object indicating when to stop
                           waiting before the timeout period has expired, or
                           None if stopping before timeout is not required.
        :return: Required notification CPO object, or None if the stop_event
                 is signalled or timeout occurs.

        """
        notification = wait_until(self._get_notification,
                                  notification_class,
                                  timeout=timeout,
                                  period=0.5,
                                  stop_event=stop_event,
                                  validator=validator,
                                  **kwargs)
        return notification or None

    def get_new_message_notification(self, timeout=60, stop_event=None):
        """
        Return CPO for new message notification.

        :param timeout: How long to wait for the notification.
        :param stop_event: threading.Event object indicating whether to stop
                           waiting before the timeout period has expired, or
                           None if stopping before the timeout is not required.
        :return: New message notification CPO object.

        """
        from ubuntu_system_tests.helpers.notifications import new_message
        return self.get_notification(
            notification_class=new_message.Notification,
            timeout=timeout,
            validator=new_message.validate,
            stop_event=stop_event)

    def get_provider_notification(self, timeout=60, stop_event=None):
        """
        Return CPO for network provider balance notification.

        :param timeout: How long to wait for the notification.
        :param stop_event: threading.Event object indicating whether to stop
                           waiting before the timeout period has expired, or
                           None if stopping before the timeout is not required.
        :return: Confirmation notification CPO object.

        """
        from ubuntu_system_tests.helpers.notifications import confirmation
        return self.get_notification(
            notification_class=confirmation.Notification,
            timeout=timeout,
            validator=confirmation.validate,
            stop_event=stop_event)

    def get_download_finished_notification(self, timeout=60, stop_event=None):
        """
        Return CPO for download finished notification.

        :param timeout: How long to wait for the notification.
        :param stop_event: threading.Event object indicating whether to stop
                           waiting before the timeout period has expired, or
                           None if stopping before the timeout is not required.
        :return: Download finished notification CPO object.

        """
        from ubuntu_system_tests.helpers.notifications import download_finished
        return self.get_notification(
            notification_class=download_finished.Notification,
            timeout=timeout,
            validator=download_finished.validate,
            stop_event=stop_event,
            iconSource='image://theme/save',
            summary='Download Complete')

    def get_incoming_call_notification(self, timeout=60, stop_event=None):
        """
        Return CPO for incoming call notification.

        :param timeout: How long to wait for the notification.
        :param stop_event: threading.Event object indicating whether to stop
                           waiting before the timeout period has expired, or
                           None if stopping before the timeout is not required.
        :return: Incoming call notification CPO object.

        """
        from ubuntu_system_tests.helpers.notifications import incoming_call
        return self.get_notification(
            notification_class=incoming_call.Notification,
            timeout=timeout,
            validator=incoming_call.validate,
            stop_event=stop_event,
            secondaryIconSource='image://theme/incoming-call')

    def get_alarm_notification(self, timeout=60, stop_event=None):
        """
        Return CPO for alarm notification.

        :param timeout: How long to wait for the notification.
        :param stop_event: threading.Event object indicating whether to stop
                           waiting before the timeout period has expired, or
                           None if stopping before the timeout is not required.
        :return: Alarm notification CPO object.

        """
        from ubuntu_system_tests.helpers.notifications import alarm
        return self.get_notification(
            notification_class=alarm.Notification,
            timeout=timeout,
            validator=alarm.validate,
            stop_event=stop_event)
