# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import fixtures

from subprocess import CalledProcessError

from ubuntu_system_tests.helpers import power
from ubuntu_system_tests.helpers import sensors
from ubuntu_system_tests.helpers.processes import get_process_id
from ubuntu_system_tests.helpers.testbed import run_command_with_sudo as sudo

STOP_POWERD_COMMAND = '/sbin/initctl stop powerd'
START_POWERD_COMMAND = '/sbin/initctl start powerd {}'


class BrightnessEnvironmentFixture(fixtures.Fixture):

    def __init__(self, auto_brightness='disable',
                 brightness_value=power.GSETTINGS_BRIGHTNESS_MAX):
        self.auto_brightness = auto_brightness
        self.brightness_value = str(brightness_value)

    def setUp(self):
        super().setUp()
        self.useFixture(AutoBrightness(state=self.auto_brightness))
        self.useFixture(Brightness(value=self.brightness_value))


class AutoBrightness(fixtures.Fixture):

    def __init__(self, state):
        self.request = state

    def setUp(self):
        super().setUp()
        old_value = power.get_auto_brightness_current_setting()
        self.addCleanup(power.set_auto_brightness_setting, old_value)

        if self.request == 'enable':
            power.set_auto_brightness_setting('true')
        elif self.request == 'disable':
            power.set_auto_brightness_setting('false')


class Brightness(fixtures.Fixture):
    """Fixture to set custom screen brightness."""

    def __init__(self, value):
        self.value = value

    def setUp(self):
        super().setUp()
        old_value = power.get_brightness_setting()
        power.set_brightness_setting(self.value)
        self.addCleanup(power.set_brightness_setting, old_value)


class EnablePowerdFakeSensors(fixtures.Fixture):
    """Fixture to enable fake sensors for powerd."""

    def setUp(self):
        super().setUp()
        if not self.are_fake_sensors_enabled():
            self.enable_fake_sensors()
        self.addCleanup(self.restart_powerd)

    def stop_powerd(self):
        """Stop existing powerd process."""
        sudo(STOP_POWERD_COMMAND)

    def start_powerd(self, fake_sensors=False):
        """Start a new powerd process.
        :param fake_sensors: If True then enable fake sensors, otherwise use
                             real sensors.
        """
        if fake_sensors:
            cmd = START_POWERD_COMMAND.format(sensors.SENSOR_OVERRIDE)
        else:
            cmd = START_POWERD_COMMAND.format('').strip()
        sudo(cmd)

    def restart_powerd(self, fake_sensors=False):
        """Restart powerd process, stopping existing process if running.
        :param fake_sensors: If True then enable fake sensors, otherwise use
                             real sensors.
        """
        try:
            pid = get_process_id('powerd')
        except CalledProcessError:
            pass
        else:
            self.stop_powerd()
            self.delete_fifo(pid)
        self.start_powerd(fake_sensors)

    def enable_fake_sensors(self):
        """Enable fake sensors on the powerd process."""
        self.restart_powerd(fake_sensors=True)
        power.validate_powerd_fifo_ready()
        power.setup_powerd_sensors()

    def delete_fifo(self, pid):
        """Delete fifo buffer for specified process id."""
        sudo('rm -f /tmp/sensor-fifo-{}'.format(pid))

    def are_fake_sensors_enabled(self):
        """Return True if fake sensors enabled on powerd."""
        try:
            pid = get_process_id('powerd')
        except CalledProcessError:
            return False
        return sensors.are_fake_sensors_enabled_for_process(pid)
