# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import logging

from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers.platform import (
    get_device_name, get_channel_name)
from ubuntu_system_tests.helpers.scopes import data as scope_data
from ubuntu_system_tests.helpers import url_dispatcher

SCOPE_ID_TODAY = 'today'
SCOPE_ID_NEARBY = 'nearby'
SCOPE_ID_APPS = 'apps'
SCOPE_ID_NEWS = 'news'
SCOPE_ID_PHOTOS = 'photos'
SCOPE_ID_MUSIC = 'music'
SCOPE_ID_VIDEO = 'video'

SCOPE_DIR_BASE = os.path.join(
    fs.DIR_HOME_LOCAL_SHARE, 'unity-scopes', 'unconfined')
HIDDEN = 'hints_is_hidden'
TODAY_DIR = 'com.canonical.scopes.dashboard_dashboard'
PHOTOS_DIR = 'com.canonical.scopes.photos_photos'
TODAY_HIDDEN = os.path.join(SCOPE_DIR_BASE, TODAY_DIR, HIDDEN)
PHOTOS_HIDDEN = os.path.join(SCOPE_DIR_BASE, PHOTOS_DIR, HIDDEN)

scopes_list = None

SCOPE_NOT_FOUND_ERR = ('Scope id "{s}" not found for device "{d}" and '
                       'channel "{c}" in scope test data file: {f}.')
NO_SCOPES_DEFINED_ERR = ('No scopes defined for device "{d}" and channel '
                         '"{c}" in scope test data file: {f}.')
DEFAULT_SCOPE_INFO = ('No scopes defined for device "{d}" and channel "{c}". '
                      'Using specified default configuration instead.')

logger = logging.getLogger(__name__)


def go_to_scope(scope_id, scope_class=None, **kwargs):
    """
    Go directly to the specified scope and wait until it is loaded.

    :param scope_id: The desired scope id.
    :param scope_class: The class of CPO required to return. If None then a
                        generic GenericScopeView helper is returned.
    :return: The required scope custom proxy object.

    """
    url_dispatcher.go_to_url(get_scope_data(scope_id).url)
    scope = context.shared.dash.get_current_scope(scope_class, **kwargs)
    scope.wait_for_header_to_stop_moving()
    scope.wait_for_processing_to_complete()
    return scope


def go_to_apps_scope():
    """Navigate directly to apps scope and return apps scope CPO."""
    from ubuntu_system_tests.helpers.scopes.apps._cpo import GenericScopeView
    return go_to_scope(SCOPE_ID_APPS,
                       GenericScopeView,
                       objectName='clickscope')


def go_to_music_scope():
    """Navigate directly to music scope and return music scope CPO."""
    from ubuntu_system_tests.helpers.scopes.music._cpo import GenericScopeView
    return go_to_scope(SCOPE_ID_MUSIC,
                       GenericScopeView,
                       objectName='musicaggregator')


def go_to_photos_scope():
    """Navigate directly to photos scope and return photos scope CPO."""
    from ubuntu_system_tests.helpers.scopes.photos._cpo import GenericScopeView
    return go_to_scope(SCOPE_ID_PHOTOS,
                       GenericScopeView,
                       objectName='com.canonical.scopes.photos_photos')


def go_to_video_scope():
    """Navigate directly to video scope and return video scope CPO."""
    from ubuntu_system_tests.helpers.scopes.video._cpo import GenericScopeView
    return go_to_scope(SCOPE_ID_VIDEO,
                       GenericScopeView,
                       objectName='videoaggregator')


def get_scope_list():
    """Return a list of ScopeData objects for current channel and device."""
    global scopes_list
    if scopes_list:
        return scopes_list

    device = get_device_name()
    channel = get_channel_name()
    required_scopes = scope_data._get_required_scope_list(channel, device)
    if not required_scopes:
        # Data for the specified device and channel name does not exist.
        # Check to see if there is a default channel name specified for device.
        required_scopes = _get_default_scope_list(device)
        if required_scopes:
            # Log that the default scope list is being used
            logger.info(DEFAULT_SCOPE_INFO.format(d=device, c=channel))
        else:
            # No default list of scopes is defined so cannot continue
            raise NameError(NO_SCOPES_DEFINED_ERR.format(
                d=device,
                c=channel,
                f=scope_data._get_scopes_file_path()))
    return scope_data._get_scope_object_list(required_scopes)


def _get_default_scope_list(device):
    """Return scopes list specified from default-device-channels for device."""
    for item in scope_data._get_default_device_channels_list():
        if item['device'] == device:
            return scope_data._get_required_scope_list(item['channel'], device)


def get_scope_data(scope_id):
    """Get the expected data for the specified scope id."""
    for scope in get_scope_list():
        if scope.id == scope_id:
            return scope
    raise RuntimeError(SCOPE_NOT_FOUND_ERR.format(
        s=scope_id, d=get_device_name(), c=get_channel_name(),
        f=scope_data._get_scopes_file_path()))


def is_scope_included(scope_id):
    """Return True when the specified scope_id is included in scopes list."""
    if get_device_name() == 'desktop':
        return True
    return scope_id in [scope.id for scope in get_scope_list()]


def is_today_scope_setup():
    """Return True if Today scope has been setup."""
    return os.path.exists(TODAY_HIDDEN)


def is_photos_scope_setup():
    """Return True if Photos scope has been setup."""
    return os.path.exists(PHOTOS_HIDDEN)
