# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import json
import os
import sys

from ubuntu_system_tests.helpers import file_system
from ubuntu_system_tests.helpers import timedate

SCOPES_FILE = os.path.join(file_system.DIR_TEST_DATA_SCOPES, 'scopes.json')
KEY_PARENT_ID = 'parent_id'
KEY_SECTIONS = 'sections'
KEY_CONDITION = 'condition'
KEY_FORMAT_METHODS = 'format_methods'
CONDITION_DEFAULT = 'none'
SECTION_HEADER = 'header'
SECTION_LABEL = 'label'
SECTION_IMAGE = 'image'

THIS_MODULE = sys.modules[__name__]


class ScopeData(object):

    """Class to store all expected data about a scope."""

    def __init__(self, id, title, url, check_header_count='true',
                 check_order='true', default_condition=CONDITION_DEFAULT,
                 parent_id=None, sections=None, unsetup_sections=None):
        self.id = id
        self.title = title
        self.url = url
        self.check_header_count = check_header_count == 'true'
        self.check_order = check_order == 'true'
        self.default_condition = default_condition
        self.parent_id = parent_id
        self.sections = sections
        self.unsetup_sections = unsetup_sections


def _get_scopes_file_path():
    return os.path.realpath(SCOPES_FILE)


def _read_json_from_file(file_path):
    with open(file_path) as f:
        data_json = json.load(f)
    return data_json


def _get_scopes_json():
    return _read_json_from_file(_get_scopes_file_path())


def _get_scopes_list():
    return _get_scopes_json().get('scopes', None) or []


def _get_channels_list():
    return _get_scopes_json().get('channels', None) or []


def _get_default_device_channels_list():
    return _get_scopes_json().get('default-device-channels', None) or []


def _get_scope_from_id(scopes_list_json, scope_id):
    """Return scope json for scope matching scope_id."""
    for scope_json in scopes_list_json:
        if scope_json['id'] == scope_id:
            return scope_json
    return None


def _get_format_current_day_of_month():
    """Return current day of month 0-31."""
    return timedate.get_current_day_of_month()


def _get_format_current_day_name():
    """Return current day name."""
    return timedate.get_current_day_name()


def _get_format_current_month_name():
    """Return current month name."""
    return timedate.get_current_month_name()


def _get_format_current_week_number():
    """Return current week number."""
    return timedate.get_current_week_number()


def _construct_scope(scopes_list_json, scope_id):
    """
    Construct a ScopeData object using the specified scope_id value.

    1) Look up the the scope json definition from the scope id in the list of
       all scopes defined in scopes_list_json.
    2) If this definition includes a parent id then look up that parent
       scope until no other parent scopes are found.
    3) Construct the ScopeData object using the base parent scope.
    4) Add all section titles from the child scope definitions to the
       created ScopeData object, so all required section titles are present.
    5) Return the constructed ScopeData object.

    """
    scope_json = _get_scope_from_id(scopes_list_json, scope_id)
    if KEY_PARENT_ID in scope_json:
        # Look up the base scope re-cursively
        scope = _construct_scope(scopes_list_json, scope_json[KEY_PARENT_ID])
    else:
        # There is no other base scope defined, so create the ScopeData object
        # here using the json for the current scope.
        return ScopeData(**scope_json)

    # To get here there must have been a parent scope defined.
    # Combine the parent scope and child scope by adding all sections from
    # the child scope to the parent scope
    for section in scope_json[KEY_SECTIONS]:
        scope.sections.append(section)

    return scope


def _get_conditional_section_list_from_scope(scope):
    """
    Using the provided scope object, create a list of expected section
    titles for each condition specified.

    The result will be a dictionary of condtions, each containing a list of
    expected sections for that condition.

    :param scope: The ScopeData object to check.
    :return: A dictionary of conditions with a list of expected sections for
             each condition.

    """
    conditions_dict = {}
    conditions = _get_condition_list_from_scope(scope)
    for condition in conditions:
        # create a list of required sections for each condition
        conditions_dict[condition] = _get_sections_for_condition(
            condition, scope.sections)
    return conditions_dict


def _get_condition_list_from_scope(scope):
    """Return a list of all conditions specified in the scope."""
    # use dictionary to keep track of all conditions and avoid duplicates.
    conditions = {}
    conditions[scope.default_condition] = 1
    # Now check through each section to see if it has a condition specified
    for section in scope.sections:
        if KEY_CONDITION in section:
            conditions[section[KEY_CONDITION]] = 1
    return conditions.keys()


def _get_sections_for_condition(condition, sections):
    """Return a list of sections for each specified condition."""
    section_list = []
    for section in sections:
        if KEY_CONDITION in section:
            # this section has a condition specified, check its value
            if section[KEY_CONDITION] == condition:
                # this is a matching condition, so add to list,
                # otherwise just ignore it
                section_list.append(section)
        else:
            # no condition specified for this section, so its required
            section_list.append(section)
    return section_list


def get_section_names_from_list(section_list):
    """
    Return list of section names from the sections list.

    These could be defined as 'header', 'label' or 'image'.

    """
    labels = []
    for section in section_list:
        labels.append(get_section_name_from_section(section))
    return labels


def get_section_name_from_section(section):
    """
    Return section name from the section object.

    The name could be defined as 'header', 'label' or 'image'.

    """
    if SECTION_LABEL in section:
        key = SECTION_LABEL
    elif SECTION_IMAGE in section:
        key = SECTION_IMAGE
    else:
        key = SECTION_HEADER
    value = section[key]

    if KEY_FORMAT_METHODS in section:
        # custom formatting is required on the value returned
        value = _apply_format_methods_to_string(value,
                                                section[KEY_FORMAT_METHODS])

    return value


def get_section_header_names_from_list(section_list):
    """Return a list of Headers items from the section_list."""
    headers = []
    for section in section_list:
        if SECTION_HEADER in section:
            headers.append(section[SECTION_HEADER])
    return headers


def _apply_format_methods_to_string(string, format_methods):
    """
    Apply the specified format methods to the format string.
    :param string: The string to apply the format methods to
    :param format_methods: A string representation of a list of format method
                           names to apply.
                           e.g. "['format_method_1', 'format_method_2']"
    :return: The formatted string

    """
    format_results = []
    for format_method in format_methods.split(','):
        result = getattr(
            THIS_MODULE, '_get_format_{m}'.format(m=format_method.strip()))()
        format_results.append(result)
    return string.format(*format_results)


def _get_device_scope_list(devices, device_name):
    """Return a list of scopes required by a device."""
    scopes = []
    for device in devices:
        if device['name'] == device_name:
            scopes = device['scopes']
    return scopes


def _get_required_scope_list(channel_name, device_name):
    """Return a list of required scope ids for specified channel and device."""
    channels_list_json = _get_channels_list()
    # Find the matching channel and device combination
    # Then store the list of required scope titles
    required_scopes = []
    for channel in channels_list_json:
        if channel['name'] == channel_name:
            devices = channel.get('devices', None) or []
            required_scopes = _get_device_scope_list(devices, device_name)
    return required_scopes


def _get_scope_object(scope_id):
    """Get a ScopeData object for required scope_id."""
    scopes_list_json = _get_scopes_list()
    for scope_json in scopes_list_json:
        # Loop through every scope defined in json
        if scope_json['id'] == scope_id:
            # The required scope id and the json definition match so
            # convert to ScopeData object
            scope = _construct_scope(scopes_list_json, scope_json['id'])
            # Now get a list of sections for each condition specified in the
            # scope. Store the list of conditional sections in the ScopeData
            # object, over-writing previous sections list
            scope.sections = _get_conditional_section_list_from_scope(scope)
            return scope
    # There was no scope matching the required id
    # raise an exception as this implies the json definition is wrong
    ERR_MSG = ('No scope found with id: {s}\nDefinition file: {f}')
    raise NameError(ERR_MSG.format(
        s=scope_id,
        f=_get_scopes_file_path()))


def _get_scope_object_list(required_scopes):
    """Return a list of ScopeData objects from required list."""
    scope_list = []
    for required_scope in required_scopes:
        scope_list.append(_get_scope_object(required_scope['id']))
    return scope_list


def is_image_expected_in_sections(section_list):
    """Return True if an image is expected in the section_list."""
    for section in section_list:
        if SECTION_IMAGE in section:
            return True
    return False
