# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot import exceptions

from ubuntu_system_tests.helpers.autopilot import (
    is_element, patched_wait_select_single, wait_select_many,
    validate_dbus_object)
from ubuntu_system_tests.helpers.autopilot.list import order_by_y_coord
from ubuntu_system_tests.helpers.scopes.base import (
    GenericScopeView as GenericScopeViewBase)
from ubuntu_system_tests.helpers.ubuntuuitoolkit import pageheader
from ubuntu_system_tests.helpers.unity8 import get_dash, UNITY8_DASH_PATH_ROOT

HEADER_MY_MUSIC = 'dashSectionHeadermymusic'
CATEGORY_MY_MUSIC = 'dashCategorymymusic'
CATEGORY_ARTISTS = 'dashCategoryartists'
CATEGORY_ALBUMS = 'dashCategoryalbums'
HEADER_SOUND_CLOUD = 'dashSectionHeadersoundcloud'
CATEGORY_SOUND_CLOUD = 'dashCategorysoundcloud'
HEADER_7DIGITAL = 'dashSectionHeader7digital'
CATEGORY_7DIGITAL = 'dashCategory7digital'


class GenericScopeView(GenericScopeViewBase):
    """ This class contains generic methods used by the music scope classes """

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_DASH_PATH_ROOT, b'GenericScopeView',
            objectName='musicaggregator')

    def _get_displayed_title_labels(self, object_name, section=None):
        """Return all displayed title labels.
        :param object_name: Object name of required label. Either 'titleLabel'
                            or 'subtitleLabel'.
        :param section: Optional section to examine.
        """
        if section:
            grid = section.select_single('CardGrid')
        else:
            grid = self.select_single('CardGrid')
        return [label.text for label in grid.select_many(
            objectName=object_name)]

    def get_displayed_titles(self, section=None):
        """
        Retrieve all the titles displayed for all the songs.
        The titles displayed could be the artist, album, genre, etc.
        :param section: If not None, the titles will be retrieved from there,
        otherwise will be retrieved from the scope
        """
        return self._get_displayed_title_labels(
            object_name='titleLabel', section=section)

    def get_displayed_subtitles(self, section=None):
        """
        Retrieve all the sub-titles displayed for all the songs.
        The sub-titles displayed could be the artist, album, genre, etc.
        :param section: If not None, the titles will be retrieved from there,
        otherwise will be retrieved from the scope
        """
        return self._get_displayed_title_labels(
            object_name='subtitleLabel', section=section)

    def is_artist(self, artist):
        """ Determines if there is an album with the desired name in the
        current scope
        :param artist: The name of the desired artist
        :return: True is there is an artist with the desired name, False
        otherwise
        """
        try:
            self.get_artist(artist)
            return True
        except exceptions.StateNotFoundError:
            return False

    def get_artist(self, artist):
        """ Retrieve a list of artists with the desired name. """
        artists = wait_select_many(
            root=self, objectName='subtitleLabel', text=artist)
        return order_by_y_coord(artists, include_off_screen=True)[0]

    def select_artist(self, artist):
        """
        Click on the album with the desired name and wait until the new
        screen is loaded
        :return: A category preview object for the new screen displayed
        """
        self.pointing_device.click_object(self.get_artist(artist))
        self.wait_for_processing_to_complete()
        from ubuntu_system_tests.helpers.scopes.music import _artist
        return get_dash().get_current_scope(_artist.GenericScopeView,
                                            objectName='musicaggregator',
                                            visible=True)

    def select_my_music(self):
        """
        Click on the my music category header, wait until the page is loaded
        and return the category preview object
        """
        return self._select_category(HEADER_MY_MUSIC, CATEGORY_ARTISTS)

    def get_my_music_titles(self):
        """
        Retrieve all the titles displayed for all the songs in my music
        :raise StateNotFoundError: When the my music section is not displayed
        """
        return self._get_titles(CATEGORY_MY_MUSIC)

    def get_my_music_subtitles(self):
        """
        Retrieve all the subtitles displayed for all the songs in my music
        :raise StateNotFoundError: When the my music section is not displayed
        """
        return self._get_subtitles(CATEGORY_MY_MUSIC)

    def is_my_music_displayed(self):
        """ Indicate if the my music section is being displayed """
        is_element(self._get_category, CATEGORY_MY_MUSIC)

    def select_sound_cloud(self):
        """
        Click on the sound cloud category header, wait until the page is loaded
        and return the category preview object
        """
        return self._select_category(HEADER_SOUND_CLOUD,
                                     CATEGORY_SOUND_CLOUD)

    def get_sound_cloud_titles(self):
        """Retrieve all titles displayed for all the songs in sound cloud."""
        return self._get_titles(CATEGORY_SOUND_CLOUD)

    def select_7digital(self):
        """
        Click on the 7digital category header, wait until the page is loaded
        and return the category preview object
        """
        return self._select_category(HEADER_7DIGITAL,
                                     CATEGORY_7DIGITAL)

    def get_7digital_titles(self):
        """
        Retrieve all the titles displayed for all the songs in 7digital
        :param count: The number of expected titles
        """
        return self._get_titles(CATEGORY_7DIGITAL)

    def _get_titles(self, category_name):
        """
        Retrieve all the titles displayed for all the songs in my music
        """
        category = self._get_category(category_name)
        return self.get_displayed_titles(section=category)

    def _get_subtitles(self, category_name):
        """
        Retrieve all the subtitles displayed for all the songs in my music
        """
        category = self._get_category(category_name)
        return self.get_displayed_subtitles(section=category)

    def _get_section_header(self, header_name):
        return self.wait_select_single(objectName=header_name)

    def _get_category(self, category_name):
        return get_dash().wait_select_single(
            DashCategoryBase, objectName=category_name)

    def _select_category(self, header_name, category_name):
        section = self._get_section_header(header_name)
        self.pointing_device.click_object(section)
        self.wait_for_processing_to_complete()
        return self._get_category(category_name)


class DashCategoryBase(GenericScopeView):
    """
    This class represents a category of music displayed in the music scope.

    """

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_DASH_PATH_ROOT, b'DashCategoryBase')

    def _get_button_by_title(self, title):
        return self.wait_select_single('UCAbstractButton', title=title)

    def _get_track_list_view(self):
        """Return the track list view that can be swiped up and down."""
        return self.select_single(
            'QQuickGridView', objectName='responsiveGridViewGrid')

    def _click_button_by_title(self, title):
        """ Click on the button with the desired title """
        button = self._get_button_by_title(title)
        self._get_track_list_view().swipe_child_into_view(button)
        self.pointing_device.click_object(button)

    def select_track(self, track):
        """ Click on the track with the desired name """
        from ubuntu_system_tests.helpers.scopes.music import _track
        self._click_button_by_title(track)
        return patched_wait_select_single(
            root=self, class_object=_track.Preview, class_name='Preview',
            objectName='preview0', visible=True)

    def select_artist(self, artist):
        """ Click on the artist with the desired name """
        self._click_button_by_title(artist)
        return get_dash().wait_select_single(objectName=CATEGORY_ALBUMS)


class DashPageHeader(pageheader.DashPageHeader):
    """
    This class represents the header section that is part of the Dash
    and implement the methods specific to the Music Header.
    """

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_DASH_PATH_ROOT, b'DashPageHeader')

    def _get_search_button(self):
        """Return header search button."""
        return self.select_single(objectName='search_button')

    def _get_options_list(self):
        """Return options list displayed as panel underneath header."""
        temp_scope = get_dash().select_single(objectName='dashTempScopeItem')
        panel = temp_scope.select_single(objectName='peExtraPanel')
        return panel.select_single(objectName='navigation0')

    def select_search(self, options):
        """Press search button and select search options from list.
        :param options: List of options to select from list.
        :return: Displayed category.
        """
        self.pointing_device.click_object(self._get_search_button())
        options_list = self._get_options_list()
        flickable = options_list.select_single('QQuickFlickable')
        for option in options:
            list_item = flickable.select_single(text=option)
            flickable.swipe_child_into_view(list_item)
            self.pointing_device.click_object(list_item)
        scope = get_dash().get_current_scope()
        scope.wait_for_processing_to_complete()
        categories = wait_select_many(
            scope, type_name=DashCategoryBase, visible=True, timeout=60)
        return order_by_y_coord(categories)[0]
