# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

import autopilot.logging
import autopilot.exceptions as exceptions

import ubuntuuitoolkit as uitk
import ubuntu_system_settings as uss

from ubuntu_system_tests.helpers.autopilot import (
    validate_dbus_object, is_element)

logger = logging.getLogger(__name__)

APP_PATH_ROOT = b'ubuntu-system-settings'


class SystemSettingsMainWindow(uss.SystemSettingsMainWindow):
    """Autopilot helper for the Main Window."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'MainWindow',
            objectName='systemSettingsMainView')

    def _get_battery_item(self):
        return self.select_single(objectName='entryComponent-battery')

    def _get_brightness_item(self):
        return self.select_single(objectName='entryComponent-brightness')

    def go_to_battery_page(self):
        self.scroll_to_and_click(self._get_battery_item())
        return self.wait_select_single('PageComponent', title='Battery')

    def go_to_brightness_page(self):
        self.scroll_to_and_click(self._get_brightness_item())
        return self.wait_select_single('ItemPage', title='Brightness')


class SimPin(uss.SimPin):
    """Autopilot helper for the SimPin Page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'SimPin', objectName='simPinPage')

    def get_sim_pin_switch(self, sim_number):
        """Return the SIM PIN switch."""
        switches = self.select_many(objectName='simPinSwitch')
        switches = sorted(switches, key=lambda switch: (switch.globalRect.y))
        return switches[sim_number]

    @autopilot.logging.log_action(logger.debug)
    def click_sim_pin_switch(self, sim_number):
        """Click on the SIM PIN switch, return the SIM PIN dialog."""
        sim_pin_switch = self.get_sim_pin_switch(sim_number)
        self.pointing_device.click_object(sim_pin_switch)
        return self.get_root_instance().wait_select_single(
            objectName='lockDialogComponent'
        )


class CellularPage(uss.CellularPage):
    """Autopilot helper for the Cellular settings page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'PageComponent',
            objectName='cellularPage')

    def get_default_sim_for_messages_selector(self, value):
        return self.select_single(objectName='defaultForMessages' + value)


class BatteryPage(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the Battery settings page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'PageComponent', title='Battery')

    def _get_wifi_switch_row(self):
        return self.wait_select_single('Standard', text='Wi-Fi')

    def get_battery_page_wifi_switch(self):
        """Return the wifi switch in battery settings page."""
        wifi_row = self._get_wifi_switch_row()
        return wifi_row.select_single('CheckBox', enabled=True)

    def is_wifi_switch_checked(self):
        return self.get_battery_page_wifi_switch().checked

    def turn_off_wifi(self):
        """Turn off wifi from battery settings page."""
        wifi_switch = self.get_battery_page_wifi_switch()
        wifi_switch.swipe_into_view()
        self.pointing_device.click_object(wifi_switch)
        wifi_switch.checked.wait_for(False)


class AboutPage(uss.AboutPage):
    """Autopilot helper for the About page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'PageComponent',
            objectName='aboutPage')

    def get_wifi_address(self):
        """ Retrieve the wifi address value """
        return self.wait_select_single('SingleValue',
                                       text='Wi-Fi address').value

    def get_bluetooth_address(self):
        """ Retrieve the bluetooth address value """
        return self.wait_select_single('SingleValue',
                                       text='Bluetooth address').value

    def _get_storage(self):
        return self.wait_select_single('SingleValue', objectName='storageItem')

    def get_storage(self):
        """ Retrieve the storage item value """
        return self._get_storage().value

    def _go_to_page(self, itemname, pagename):
        item = self.select_single(objectName=itemname)
        item.swipe_into_view()
        self.pointing_device.click_object(item)
        page = self.get_root_instance().wait_select_single(
            objectName=pagename)
        page.active.wait_for(True)
        return page

    def go_to_os(self):
        """ Go to the OS version page and retrieve the page """
        return self._go_to_page('osItem', 'versionPage')

    def go_to_developer_mode(self):
        """ Go to the developer mode page and retrieve the page """
        return self._go_to_page('devmodeItem', 'devModePage')

    def go_to_storage(self):
        """ Go to the storage page and retrieve the page """
        return self._go_to_page('storageItem', 'storagePage')


class LicensesPage(uss.LicensesPage):
    """Autopilot helper for the Licenses page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'ItemPage',
            objectName='licensesPage')

    def _get_licenses_list(self):
        return self.wait_select_single('QQuickListView')

    def _get_license(self, text):
        return self.select_single('Standard', text=text)

    def get_license(self, text):
        """ Swipe until the object appears in the tree, it is not possible to
        use swipe_child_into_view because the license objects are not part of
        the tree
        """
        list = self._get_licenses_list()
        while not list.atYEnd:
            list.swipe_to_show_more_below()
            if is_element(self._get_license, text):
                return self._get_license(text)

        raise exceptions.StateNotFoundError(class_name='Standard', text=text)


class VersionPage(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the Version page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'Version',
            objectName='versionPage')

    def get_build_number(self):
        return self.select_single('SingleValueStacked',
                                  objectName='versionBuildNumberItem').value

    def get_build_desc(self):
        return self.select_single('SingleValueStacked',
                                  objectName='ubuntuBuildIDItem').value


class DevModePage(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the developer mode page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'ItemPage',
            objectName='devModePage')

    def _get_devmode_checkbox(self):
        return self.select_single('CheckBox',
                                  styleName='SwitchStyle')

    def is_devmode_on(self):
        return self._get_devmode_checkbox().checked

    def switch_devmode(self):
        switcher = self._get_devmode_checkbox()
        checked = switcher.checked
        self.pointing_device.click_object(switcher)
        switcher.checked.wait_for(not checked)


class StoragePage(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """Autopilot helper for the developer mode page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'Storage',
            objectName='storagePage')

    def get_pictures_storage(self):
        return self.select_single('StorageItem',
                                  objectName='picturesItem').value

    def get_videos_storage(self):
        return self.select_single('StorageItem',
                                  objectName='moviesItem').value

    def get_audio_storage(self):
        return self.select_single('StorageItem',
                                  objectName='audioItem').value

    def _get_gallery_app(self):
        return self.select_single('SingleValue', objectName='appItemGallery')

    def get_gallery_app_size_in_disk(self):
        return self._get_gallery_app().value

    def get_gallery_app_icon(self):
        return self._get_gallery_app().icon


class BrightnessPage(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'ItemPage',
            objectName='brightnessPage')

    def get_auto_adjust_checkbox(self):
        """Return the auto adjust checkbox switch."""
        return self.select_single('CheckBox')

    def get_auto_adjust_status(self):
        """Return checked state of auto adjust switch."""
        return self.get_auto_adjust_checkbox().checked

    def toggle_auto_adjust(self):
        """Press the auto adjust switch and wait for state to change."""
        original_state = self.get_auto_adjust_status()
        self.pointing_device.click_object(self.get_auto_adjust_checkbox())
        self.get_auto_adjust_checkbox().checked.wait_for(not original_state)
