# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import pexpect
import subprocess

import fixtures
import os

from subprocess import check_call

from ubuntu_system_tests import config

NM_PATH = '/etc/NetworkManager/system-connections/'
ASKPASS_PATH = '/tmp/askpass.sh'

READ_ONLY_COMMAND = 'mount -o remount, ro /'
READ_WRITE_COMMAND = 'mount -o remount, rw /'


class AskPass(fixtures.Fixture):
    """Allows the user to call commands with sudo -A

    Creates an 'askpass' script which allows the user of the
    fixture to call commands with sudo -A when the fixture is
    loaded and cleans up the askpass script when finished
    """

    def __init__(self, device_password):
        super().__init__()
        self.device_password = device_password

    def setUp(self):
        super().setUp()
        self.useFixture(fixtures.EnvironmentVariable(
            'SUDO_ASKPASS', ASKPASS_PATH))
        with open(ASKPASS_PATH, 'w') as askpass:
            self.addCleanup(os.remove, ASKPASS_PATH)
            askpass.writelines('#!/bin/sh\necho {}'.format(
                self.device_password))
            os.fchmod(askpass.fileno(), 0o0744)


class RemoveNetworks(fixtures.Fixture):
    """Removes all networks from the device

    Removes all networks from the device and restarts
    network services to action that change and on cleanup
    restores the previous state
    """

    def __init__(self):
        super().__init__()

    def setUp(self):
        super().setUp()
        # only backup networks if one already exists
        if self.is_network_present():
            self.tmpdir_fixture = fixtures.TempDir()
            self.useFixture(self.tmpdir_fixture)
            run_command_with_sudo('mv {}/* {}/'.format(
                NM_PATH, self.tmpdir_fixture.path))
            # If we succesfully move the connection files
            # then we need to make sure to reinstate them
            self.addCleanup(self._restore_networks)
            run_command_with_sudo('service network-manager restart')
            check_call(['initctl', 'restart', 'indicator-network'])

    def _restore_networks(self):
        run_command_with_sudo('mv {}/* {}'.format(
            self.tmpdir_fixture.path, NM_PATH))
        run_command_with_sudo('service network-manager restart')
        check_call(['initctl', 'restart', 'indicator-network'])

    def is_network_present(self):
        """Return True if a network is currently present."""
        return len(os.listdir(NM_PATH)) > 0


def run_command_with_sudo(command):
    """Run the provided command with sudo.
    :param command: Command to execute with sudo privileges.
    :return: Output of command."""
    root_command = 'sudo -A {}'.format(command)
    with AskPass(_get_device_password()):
        return subprocess.check_output(root_command, shell=True)


def run_command_with_password_prompt(command, args=[],
                                     password_pattern='assword',
                                     timeout=30):
    """
    Run the provided command, enter the password when prompted and wait until
    the command finishes
    :param command: The command to run
    :param args: List with the arguments for the command
    :param password_pattern: Pattern to check when type the device password.
    The pattern can be either a string or of a list of strings.
    :param timeout: The timeout to wait for the command (-1 is unlimited)
    """
    child = pexpect.spawn(command, args)
    child.expect(password_pattern, timeout=10)
    child.sendline(_get_device_password())
    child.expect(pexpect.EOF, timeout=timeout)


def _get_device_password():
    config_stack = config.get_device_config_stack()
    return config_stack.get('device_password')


def make_system_writable():
    """remount the file system to make it read/write enabled."""
    run_command_with_sudo(READ_WRITE_COMMAND)


def make_system_read_only():
    """remount the file system to make it read only."""
    run_command_with_sudo(READ_ONLY_COMMAND)


def write_to_file_with_sudo(path, command, append=False):
    """
    Write to a file with sudo privileges.
    :param path: Path of file to write to.
    :param command: Command to run that will generate content to write.
    :param append: Whether to write to the file in append mode.
    """
    write_cmd = command + ' | sudo -A tee '
    if append:
        write_cmd += '-a '
    write_cmd += path
    with AskPass(_get_device_password()):
        subprocess.check_call(
            write_cmd, shell=True, stderr=subprocess.STDOUT,
            stdout=subprocess.PIPE)
