# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from autopilot import logging as autopilot_logging
from autopilot.exceptions import StateNotFoundError
from ubuntuuitoolkit import UbuntuUIToolkitCustomProxyObjectBase
from ubuntu_system_tests.helpers.autopilot import validate_dbus_object
from ubuntu_system_tests.helpers.autopilot.list import order_by_x_coord
from ubuntu_system_tests.helpers.unity8 import UNITY8_PATH_ROOT, APP_ID_DASH

logger = logging.getLogger(__name__)


class PhoneStage(UbuntuUIToolkitCustomProxyObjectBase):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_PATH_ROOT, b'PhoneStage', objectName='stage')

    def _get_app_windows(self):
        """Return list of displayed ApplicationWindow objects."""
        return self.select_many('ApplicationWindow')

    def _get_app_window(self, app_name):
        """Return ApplicationWindow object for required app."""
        window_name = 'appWindow_' + app_name
        return self.select_single('ApplicationWindow', objectName=window_name)

    def _get_app_window_object_names(self, app_windows):
        """Return list of object names from each app window in app_windows."""
        return [app_window.objectName for app_window in app_windows]

    def _get_app_window_position(self, app_windows, app_window):
        """Return position of required app_window in displayed app_windows."""
        return self._get_app_window_object_names(app_windows).index(
            app_window.objectName)

    def _get_next_app_window(self, app_window):
        """Return the app_window on the right of the specified app_window.
        If there are no more apps on the right then return None."""
        apps = self.get_all_app_windows(include_off_screen=False)
        next_app = self._get_app_window_position(apps, app_window) + 1
        try:
            return apps[next_app]
        except IndexError:
            return None

    def _get_app_window_touch_point(self, app_window):
        """Return x, y co-ordinate where the required app_window object can
        be selected. This should be slightly inside of the left edge of the
        window, between the margins of the required window and the window to
        the right or the screen edge."""
        x, y, w, h = app_window.globalRect
        mid_y = y + h // 2
        next_app = self._get_next_app_window(app_window)
        if next_app:
            next_x = next_app.globalRect.x
        else:
            next_x = self.globalRect.x + self.globalRect.width
        mid_x = x + ((next_x - x) // 4)
        return mid_x, mid_y

    @autopilot_logging.log_action(logger.info)
    def _press_app(self, app_window):
        """Press on the required app_window object."""
        x, y = self._get_app_window_touch_point(app_window)
        self.pointing_device.move(x, y)
        self.pointing_device.click()

    @autopilot_logging.log_action(logger.info)
    def _swipe_from_app_to_right_edge(self, app_window):
        """Swipe from the required app_window object to right screen edge."""
        start_x, start_y = self._get_app_window_touch_point(app_window)
        end_x = self.x + self.width
        end_y = start_y
        self.pointing_device.drag(start_x, start_y, end_x, end_y, rate=3)

    @autopilot_logging.log_action(logger.info)
    def _swipe_from_app_to_top_edge(self, app_window):
        """Swipe from the required app_window object to top screen edge."""
        start_x, start_y = self._get_app_window_touch_point(app_window)
        end_x = start_x
        end_y = 0
        self.pointing_device.drag(start_x, start_y, end_x, end_y)

    @autopilot_logging.log_action(logger.info)
    def _swipe_from_right(self):
        """Swipe from right edge to reveal the application switcher"""
        width = self.width
        height = self.height
        start_x = width
        start_y = int(height // 2)
        end_x = 0
        end_y = start_y
        self.pointing_device.drag(start_x, start_y, end_x, end_y)

    def swipe_top_app_from_view(self):
        """Swipe the right-most visible app from view by swiping it to the
        right edge."""
        apps = self.get_all_app_windows(include_off_screen=False)
        app_count = len(apps)
        if app_count > 2:
            self._swipe_from_app_to_right_edge(apps[-1])
        else:
            logger.warn('Cannot swipe away top app, only {} apps '
                        'displayed.'.format(app_count))

    def swipe_to_top(self):
        """Swipe to the top of the app window stack."""
        flickable = self.select_single(
            'QQuickFlickable', objectName='spreadView')
        while not flickable.atXEnd:
            self._swipe_from_right()
            flickable.moving.wait_for(False)

    def swipe_app_into_view(self, app_window):
        """Swipe to make an app visible in the task switcher."""
        self.swipe_to_top()
        apps = self.get_app_window_names()
        if app_window.objectName not in apps:
            raise StateNotFoundError
        while not self.is_app_visible(app_window):
            self.swipe_top_app_from_view()

    def is_app_visible(self, app_window):
        """Return True if specified app_window is in the visible section of
        the view. This is limited to the top 3 windows."""
        visible_apps = self.get_app_window_names(include_off_screen=False)[-3:]
        return app_window.objectName in visible_apps

    def get_all_app_windows(self, include_off_screen=True):
        """Return ordered list of all app window objects in task switcher."""
        return order_by_x_coord(
            self._get_app_windows(), include_off_screen=include_off_screen)

    def get_app_window_names(self, include_off_screen=True):
        """Return ordered list of app window object names for each app
        displayed."""
        return [app.objectName for app in self.get_all_app_windows(
            include_off_screen)]

    def close(self):
        """Close the task switcher and return to dash."""
        self.switch_to_app('unity8-dash')

    def close_app(self, app_name, return_to_dash=True):
        """Close the application with specified app_name."""
        app = self._get_app_window(app_name)
        self.swipe_app_into_view(app)
        self._swipe_from_app_to_top_edge(app)
        app.wait_until_destroyed()
        if return_to_dash:
            self.switch_to_app(APP_ID_DASH)

    def switch_to_app(self, app_name):
        """Switch to application with specified app_name."""
        app = self._get_app_window(app_name)
        self.swipe_app_into_view(app)
        self._press_app(app)
