# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from autopilot import (
    exceptions,
    logging as autopilot_logging
)
import ubuntuuitoolkit
from unity8.shell.emulators import main_window

from ubuntu_system_tests.helpers.autopilot import validate_dbus_object
from ubuntu_system_tests.helpers.indicators.message import NotificationsIndicatorPage, NotificationsIndicatorItem  # NOQA
from ubuntu_system_tests.helpers.notifications import Notifications
from ubuntu_system_tests.helpers.unity8 import (
    greeter as greeter_helpers, UNITY8_PATH_ROOT)

logger = logging.getLogger(__name__)


class Unity8(ubuntuuitoolkit.UbuntuUIToolkitCustomProxyObjectBase):

    """Autopilot custom proxy object for the Unity8 shell."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_PATH_ROOT, UNITY8_PATH_ROOT,
            applicationName='unity8')

    @property
    def main_window(self):
        return self.select_single(ShellView)

    @autopilot_logging.log_action(logger.debug)
    def wait_for_greeter_content(self):
        greeter = self.main_window.wait_select_single(objectName='greeter')
        greeter.waiting.wait_for(False)

    @autopilot_logging.log_action(logger.info)
    def unlock(self):
        greeter = self.main_window.get_greeter()
        if greeter.created:
            greeter_helpers.hide_greeter_with_dbus()
            greeter.created.wait_for(False)

    def show_dash_from_launcher(self):
        self.main_window.show_dash_from_launcher()

    def get_number_of_sessions(self):
        application_window = self.select_single('ApplicationWindow')
        return len(application_window.select_many('SessionContainer'))

    def get_height(self):
        return self.main_window.height

    def get_width(self):
        return self.main_window.width

    def ensure_app_loaded(self):
        try:
            indicator = self.select_single('ActivityIndicator')
            indicator.onScreen.wait_for(False)
        except exceptions.StateNotFoundError:
            # This is fine, it's already loaded
            pass


class ShellView(main_window.ShellView):
    """Class to extend Unity8 main window ShellView."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, UNITY8_PATH_ROOT, b'ShellView', title='Unity8')

    def get_notifications_list(self):
        """Return a list of notifications currently being displayed."""
        return self.select_single(
            Notifications, objectName='notificationList')

    def get_notifications_panel(self):
        """Return 'Notifications' panel object."""
        return self._get_indicator_panel_item('indicator-messages')

    def clear_notifications(self):
        """
        Clear all existing notifications from 'Notifications' indicator.

        """
        if self.is_new_notification_displayed():
            self.open_indicator_page('indicator-messages').clear_all()
            self.close_indicator_page()

    def open_notifications_indicator(self):
        """Swipe down to display the notifications indicator."""
        return self.open_indicator_page('indicator-messages')

    def is_new_notification_displayed(self):
        """Return True if new notification message icon is displayed."""
        return self.get_notifications_panel().is_new_message_displayed()

    def get_passphrase_unlock_screen(self):
        """Wait for input method to be in shown state and return lockscreen."""
        from ubuntu_system_tests.helpers.unity8 import lock_screen
        input_method = self.wait_select_single(
            'InputMethod', objectName='inputMethod', visible=True)
        input_method.state.wait_for('shown')
        return self.select_single(lock_screen.PassphraseLockscreen)

    def get_missed_call_snap_decision(self):
        """Return missed call snap decision menu."""
        from ubuntu_system_tests.helpers.snap_decision import missed_call
        return self.select_single(
            missed_call.SnapDecisionMenu, objectName='snapDecision',
            visible=True)

    def get_phone_stage(self):
        from ubuntu_system_tests.helpers.unity8 import phone_stage
        return self.select_single(phone_stage.PhoneStage, objectName='stage')

    def swipe_to_show_app_switcher(self):
        """Swipe from right edge to reveal task switcher."""
        stage = self.get_phone_stage()
        stage.swipe_to_top()
        return stage

    def get_current_focused_app(self):
        """Return Id for current focused app."""
        return self.select_single('Shell').focusedApplicationId

    def _is_indicator_panel_opened(self):
        indicator_page = self.wait_select_single('IndicatorsMenu')
        return indicator_page.fullyOpened

    def close_indicator_page(self):
        """
        Override the method to only try to close the indicator panel if its
        not already.
        """
        if self._is_indicator_panel_opened():
            super().close_indicator_page()

    def is_indicator_item_hidden(self, indicator_name):
        """
        Return bool representing if the requested indicator item is hidden or
        not.
        """
        item = self.select_single('IndicatorItem',
                                  objectName=indicator_name + '-panelItem')
        return item.hidden
