# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import glob
import logging
import subprocess
import os
import time

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.touch_actions import TouchActions

from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import processes
from ubuntu_system_tests.helpers.scopes.apps import (
    launch_application_from_apps_scope
)
from ubuntu_system_tests.helpers import wait_until

DEFAULT_WEBVIEW_INSPECTOR_IP = '127.0.0.1'
DEFAULT_WEBVIEW_INSPECTOR_PORT = 9221

logger = logging.getLogger(__name__)


def launch_webapp(app_name, web_app_package):
    """ Launch the webapp and return the webapp proxy object
    :param app_name: The name in the apps scope
    :param web_app_package: The package name
    :return: Proxy object for webapp
    """
    launch_application_from_apps_scope(app_name)
    context.shared.add_cleanup(ensure_webapp_closed, web_app_package)
    wait_until(is_webapp_runing, web_app_package)
    return get_webapp_proxy(web_app_package)


def is_webapp_runing(web_app):
    """ Indicate if the web app is running """
    try:
        return get_webapp_pid(web_app) != 0
    except:
        return False


def _get_webapp_id(web_app):
    try:
        webapp_id = subprocess.check_output(
            ['ubuntu-app-triplet', web_app],
            universal_newlines=True).strip()
        return webapp_id
    except subprocess.CalledProcessError:
        raise RuntimeError('No such webapp {}'.format(web_app))


def get_webapp_pid(web_app):
    """ Retrieve the web app process id """
    try:
        webapp_id = _get_webapp_id(web_app)
        return int(subprocess.check_output(['ubuntu-app-pid', webapp_id],
                                           universal_newlines=True).strip())
    except subprocess.CalledProcessError:
        raise RuntimeError('No such webapp {} running'.format(webapp_id))


def get_webapp_proxy(web_app_package):
    """ Return webapp proxy object from existing process.
    :param web_app_package: The package name
    :return: Proxy object for webapp.
    """
    from ubuntu_system_tests.helpers.webapp import _cpo  # NOQA

    pid = get_webapp_pid(web_app_package)
    proxy = autopilot.get_proxy_object(pid)
    return proxy.wait_select_single('BrowserWindow')


def ensure_webapp_closed(web_app):
    try:
        webapp_id = _get_webapp_id(web_app)
        processes.ensure_application_closed(webapp_id)
    except subprocess.CalledProcessError:
        pass


def get_webdriver(cleanup_method):
    """ Retrieve the webdriver for the current webapp container """
    driver = setup_webdriver()
    cleanup_method(driver.quit)

    driver.implicitly_wait(30)
    return driver


def setup_webdriver():
    options = _get_webdriver_options()
    chromedriver_exec_path = get_chromedriver_exec_path()

    logger.debug(
        'Connecting to webdriver, using {}.'.format(
            chromedriver_exec_path))
    driver = webdriver.Chrome(
        executable_path=chromedriver_exec_path,
        chrome_options=options)

    return driver


def _get_webdriver_options():
    options = Options()
    options.binary_location = ''
    options.debugger_address = '{ip}:{port}'.format(
        ip=DEFAULT_WEBVIEW_INSPECTOR_IP,
        port=DEFAULT_WEBVIEW_INSPECTOR_PORT)
    return options


def get_chromedriver_exec_path():
    """ Return the chromedriver excutable path. Depends on the base
    installation  directory and the current architecture
    :returns: a string containing the chromedriver executable path
    """
    base_deps_dir = ''
    default_path = 'usr/lib/*/oxide-qt/chromedriver'
    adt_artifacts = os.path.dirname(os.getenv('ADT_ARTIFACTS', default=''))

    # This check is to avoid using a dependencies dir which has not been
    # created when the dependencies are already installed
    if adt_artifacts:
        base_deps_dir = os.path.join(adt_artifacts, 'deps')

    tmp_path = os.path.join(base_deps_dir, default_path)
    exec_paths = glob.glob(tmp_path) + glob.glob(
        os.path.join('/', default_path))

    if not exec_paths:
        raise RuntimeError('Chromedriver executable not found')

    return exec_paths[0]


class BasePage:
    """ Base class to be used for all whe WebApp tests which implement the Page
    Object pattern. This class provides basic touch functions to simplify
    the WebDriver TouchActions API
    """

    def __init__(self, driver):
        self.driver = driver

    def tap(self, elem):
        """ Tap on a desired web element """
        actions = TouchActions(self.driver)
        actions.tap(elem)
        actions.perform()

    def tap_at(self, x, y):
        """
        Tap on the selected position
        :param x: x coord to tap and hold
        :param y: y coord to tap and hold
        """

        self.tap_and_hold(x, y, 0)

    def tap_and_hold(self, x, y, hold_time=5):
        """
        Tap on the selected position and hold the time selected
        :param x: x coord to tap and hold
        :param y: y coord to tap and hold
        :param hold_time: time to hold until release (seconds)
        """

        actions_tap = TouchActions(self.driver)
        actions_tap.tap_and_hold(x, y)
        actions_tap.perform()

        time.sleep(hold_time)

        actions_release = TouchActions(self.driver)
        actions_release.release(x, y)
        actions_release.perform()
