# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from webbrowser_app.emulators import browser
import ubuntuuitoolkit as uitk

from ubuntu_system_tests.helpers.autopilot import (
    click_object, patched_wait_select_single, validate_dbus_object,
    wait_select_many)
from ubuntu_system_tests.helpers.autopilot.list import order_by_y_coord

logger = logging.getLogger(__name__)

APP_PATH_ROOT = b'webbrowser-app'


class Browser(browser.Browser):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'Browser')

    def get_page_loaded(self, url):
        """Wait until the page is fully loaded and returns the text in the
        address bar
        :param url The url to wait
        :return the text displayed in the address bar
        """

        self.wait_until_page_loaded(url)
        return self.address_bar.text

    def get_current_url(self):
        return self.address_bar.text_field.text

    def go_to_url(self, url):
        """Make the browser go to the url passed as parameter
        :param url The url to wait
        :raises RuntimeError when the url could not be correctly written

        The clear method and the retry mechanism are are executes to avoid an
        sporadic error that happen when is being written the url
        """
        self.pointing_device.click_object(self.address_bar.text_field)
        for i in range(3):
            if not self.get_current_url() == url:
                self.address_bar.clear()
                self.address_bar.write(url)
            else:
                self.address_bar.text_field.keyboard.press_and_release('Enter')
                return

        raise RuntimeError("The url couldn't be correctly written in the "
                           "address bar")

    def open_drawer(self):
        """ Open the chrome bar drawer """
        drawer_button = self.chrome.get_drawer_button()
        self.pointing_device.click_object(drawer_button)

    def open_history_view(self):
        """Open the history page where the visited pages are shown"""
        self.open_drawer()
        history_action = self.chrome.get_drawer_action('history')
        self.pointing_device.click_object(history_action)
        history_action.wait_until_destroyed()

    def open_bookmarks_view(self):
        """Open the bookmarks page where bookmarks are shown"""
        self.open_drawer()
        bookmarks_action = self.chrome.get_drawer_action('bookmarks')
        self.pointing_device.click_object(bookmarks_action)
        bookmarks_action.wait_until_destroyed()
        return self.get_bookmarks_view()

    def open_downloads_page(self):
        """Open the downloads page where all downloads are shown"""
        self.open_drawer()
        downloads_action = self.chrome.get_drawer_action('downloads')
        self.pointing_device.click_object(downloads_action)
        downloads_action.wait_until_destroyed()
        return self.wait_select_single(DownloadsPage)

    def open_find_in_page(self):
        """ Open the find in page option """
        self.open_drawer()
        find_action = self.chrome.get_drawer_action('findinpage')
        self.pointing_device.click_object(find_action)
        find_action.wait_until_destroyed()

    def find_in_page(self, text):
        """ Write the text in the find in page textfield """
        self._get_find_in_page_textfield().clear()
        self._get_find_in_page_textfield().write(text)

    def _get_find_in_page_textfield(self):
        return self.wait_select_single(objectName='addressBarTextField',
                                       visible=True)

    def get_find_in_page_counter_text(self):
        return self.select_single(objectName='findInPageCounter').text

    def is_find_in_page_enabled(self):
        return self._get_chrome().findInPageMode

    def is_find_in_page_disable(self):
        return not self.is_find_in_page_enabled()

    def is_find_in_page_cursor_displayed(self):
        """ Indicates if the cursor is being displayed in the find in page
        text bar
        """
        textfield = self._get_find_in_page_textfield()
        return textfield.cursorVisible and textfield.cursorPosition >= 0

    def get_find_in_page_placeholder_text(self):
        return self._get_find_in_page_textfield().placeholderText

    def get_find_in_page_previous_button(self):
        return self.select_single(objectName='findPreviousButton')

    def get_find_in_page_next_button(self):
        return self.select_single(objectName='findNextButton')

    def get_history_urls(self):
        """Return all the pages (urls) that are shown in the history page"""
        return wait_select_many(self, type_name='UrlDelegate', visible=True)

    def get_tab_preview_urls(self):
        """Return all the pages (urls) that are shown in the history page"""
        return self._get_urls_delegates()

    def _get_urls_delegates(self):
        return self.select_many('UrlDelegate')

    def open_new_tab(self):
        """Open a new tab from the history view and return the tab"""
        self.open_history_view()
        new_tab_button = self.select_single("ToolbarAction",
                                            iconName="tab-new")
        self.pointing_device.click_object(new_tab_button)
        return self.get_new_tab_view()

    def get_popover_foreground_dialog(self):
        self.select_single('UCStyledItemBase', objectName='popover_foreground')

    def get_popover_foreground_dialog_item(self, text):
        elems = self.select_many('Empty', visible=True)
        return next(elem for elem in elems if elem.text == text)

    def get_download_options_dialog(self):
        return patched_wait_select_single(
            self, Dialog, class_name='Dialog',
            objectName='downloadOptionsDialog')

    def get_open_with_dialog(self):
        return patched_wait_select_single(
            self, ContentPeerPicker13, class_name='ContentPeerPicker13',
            objectName='contentPeerPicker')

    def open_page_and_accept_location_permission_prompt(self, url):
        """Open the provided url in the browser and accept the location
        access prompt."""
        self.go_to_url(url)
        dialog = self.get_geolocation_dialog()
        accept_button = dialog.get_allow_button()
        self.pointing_device.click_object(accept_button)
        self.wait_page_loaded()

    def wait_page_loaded(self):
        """Wait for the webbrowser to stop loading the current page."""
        webview = self.get_current_webview()
        # loadProgress == 100 ensures that a page has actually loaded
        webview.loadProgress.wait_for(100, timeout=20)
        webview.loading.wait_for(False)

    def _get_popup_copy_button(self):
        """Return the 'copy' button on the address bar's contextual popup."""
        copy_button = self.wait_select_single(iconName='edit-copy')
        copy_button.opacity.wait_for(1.0)
        return copy_button

    def _select_url_text(self):
        """Long press on the address bar to show the contextual popup."""
        self.pointing_device.click_object(self.address_bar.text_field)
        self.address_bar.text_field.activeFocus.wait_for(True)
        self.pointing_device.click_object(
            self.address_bar.text_field, press_duration=1)

    def copy_url_text(self):
        """Click the 'copy' button on the address bar's contextual popup."""
        self._select_url_text()
        copy_button = self._get_popup_copy_button()
        self.pointing_device.click_object(copy_button)

    def get_html_section_dimensions(self):
        """ Retrieve the (x, y. width, height) for the html section, not
        including the chrome bar section. When the keyboard is displayed it is
        not included as part these dimensions.
        """
        (x1, y1, w1, h1) = self.get_current_webview().globalRect
        (x2, y2, w2, h2) = self._get_chrome().globalRect

        return x1, y2 + h2, w1, h1 - h2

    def _get_keyboard_area(self):
        return self.wait_select_single('KeyboardRectangle')

    def is_keyboard_displayed(self):
        return self._get_keyboard_area().state == 'shown'

    def is_keyboard_hidden(self):
        return self._get_keyboard_area().state == 'hidden'

    def _get_bookmark_icon(self):
        """Return the bookmark icon on the address bar."""
        bookmark_toggle = self.address_bar.wait_select_single(
            objectName='bookmarkToggle')
        return bookmark_toggle.wait_select_single('Icon', visible=True)

    def is_current_page_bookmarked(self):
        """Return boolean representing if current loaded page is bookmarked."""
        return self._get_bookmark_icon().name == 'starred'

    def _bookmark_current_page(self):
        self.pointing_device.click_object(self._get_bookmark_icon())
        bookmark_options = self.get_bookmark_options()
        bookmark_options.click_dismiss_button()
        bookmark_options.wait_until_destroyed()

    def bookmark_current_page(self):
        """Bookmark the currently loaded page, if its not already."""
        if not self.is_current_page_bookmarked():
            self._bookmark_current_page()


class BookmarksView(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'BookmarksView')

    def get_bookmarks_urls(self):
        """Return all the pages (urls) that are shown in the bookmarks page"""
        return wait_select_many(self, type_name='UrlDelegate', visible=True)

    def delete_bookmark(self, url):
        bookmark_delegate = self.wait_select_single(UrlDelegate, url=url)
        bookmark_delegate.swipe_to_delete()

    def get_done_button(self):
        """Get the 'Done' button."""
        return self.wait_select_single('Button', objectName='doneButton')

    def done(self):
        """Press 'Done' button."""
        done_button = self.get_done_button()
        self.pointing_device.click_object(done_button)


class NewTabView(browser.NewTabView):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'NewTabView')

    def get_bookmarks_more_button(self):
        """Return the 'More' button.

        NOTE the webbrowser-app emulator's objectName doesn't change,
        but its label does--we query the button's 'text' to be thorough.

        """
        return self.select_single(
            objectName='bookmarks.moreButton', text='More')

    def get_bookmarks_less_button(self):
        """Return the 'Less' button.

        NOTE the webbrowser-app emulator's objectName doesn't change,
        but its label does--we query the button's 'text' to be thorough.

        """
        return self.select_single(
            objectName='bookmarks.moreButton', text='Less')

    def show_more(self):
        """Tap the 'More' button."""
        self.pointing_device.click_object(self.get_bookmarks_more_button())

    def show_less(self):
        """Tap the 'Less' button."""
        self.pointing_device.click_object(self.get_bookmarks_less_button())


class UrlDelegate(uitk.UbuntuUIToolkitCustomProxyObjectBase):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'UrlDelegate')

    def select(self):
        self.pointing_device.click_object(self)

    def _drag_pointing_device_to_delete(self):
        x, y, width, height = self.globalRect
        start_x = x + (width * 0.2)
        stop_x = x + (width * 0.8)
        start_y = stop_y = y + (height // 2)

        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
        self.dragging.wait_for(False)

    def _get_delete_button(self):
        """Return the delete button revealed on slide to right."""
        return self.wait_select_single(
            objectName='actionbutton_leadingAction.delete'
        )

    def swipe_to_delete(self):
        """Delete a bookmark by swipe."""
        self._drag_pointing_device_to_delete()
        self.pointing_device.click_object(self._get_delete_button())
        self.wait_until_destroyed()


class ContentPeerPicker13(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """CPO to represent content app selection dialog."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'ContentPeerPicker13',
            objectName='contentPeerPicker')

    def _get_button(self, text):
        return order_by_y_coord(
            self.select_many(objectName='label', text=text))[0].get_parent()

    def _get_gallery_button(self):
        return self._get_button('Gallery')

    def _get_notes_button(self):
        return self._get_button('Notes')

    def _get_tagger_button(self):
        return self._get_button('Tagger')

    def _get_media_player_button(self):
        return self._get_button('Media Player')

    def pick_gallery(self):
        self.pointing_device.click_object(self._get_gallery_button())
        self.wait_until_destroyed()

    def pick_notes(self):
        self.pointing_device.click_object(self._get_notes_button())
        self.wait_until_destroyed()

    def pick_tagger(self):
        self.pointing_device.click_object(self._get_tagger_button())
        self.wait_until_destroyed()

    def pick_media_player(self):
        self.pointing_device.click_object(self._get_media_player_button())
        self.wait_until_destroyed()


class Dialog(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """CPO to represent download options dialog."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(
            path, state, APP_PATH_ROOT, b'Dialog',
            objectName='downloadOptionsDialog')

    def _get_cancel_button(self):
        return self.select_single(objectName='cancelDownloadButton')

    def _get_choose_application_button(self):
        return self.select_single(objectName='chooseAppButton')

    def _get_download_button(self):
        return self.select_single(objectName='downloadFileButton')

    def cancel(self):
        self.pointing_device.click_object(self._get_cancel_button())
        self.wait_until_destroyed()

    def choose_application(self):
        self.pointing_device.click_object(
            self._get_choose_application_button())
        self.wait_until_destroyed()

    def download(self):
        self.pointing_device.click_object(self._get_download_button())
        self.wait_until_destroyed()


class DownloadsPage(browser.DownloadsPage):
    """CPO to represent downloads page."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(path, state, APP_PATH_ROOT,
                                    b'DownloadsPage')

    @property
    def header(self):
        return self.get_header()

    def get_download_delegate(self, url):
        """Return list delegate by url."""
        return self.select_single(DownloadDelegate, url=url)

    def cancel_download(self, url):
        """Cancel the requested download by url"""
        download_item = self.get_download_delegate(url=url)
        download_item.cancel()

    def delete_history_item(self, url):
        """Delete a downloads history item by associated url"""
        self.header.click_edit_button()
        item = self.get_download_delegate(url=url)
        self.pointing_device.click_object(item)
        self.header.click_delete_button()
        item.wait_until_destroyed()

    def clear_history(self):
        """Clear all downloads history"""
        self.header.click_edit_button()
        self.header.click_select_all_button()
        self.header.click_delete_button()

    def get_visible_downloads_count(self):
        return len(self.select_many(DownloadDelegate))

    def is_download_finished(self, url):
        """Return bool representing if the download completed"""
        download_item = self.get_download_delegate(url=url)
        return not download_item.incomplete

    def go_back_to_main_screen(self):
        """Go back to the main browser screen"""
        self.header.click_back_button()
        self.wait_until_destroyed()


class DownloadDelegate(uitk.UbuntuUIToolkitCustomProxyObjectBase):
    """CPO to represent a single Download delegate."""

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(path, state, APP_PATH_ROOT,
                                    b'DownloadDelegate')

    def _get_cancel_button(self):
        return self.select_single('Button', text='Cancel')

    def cancel(self):
        """Cancel the download"""
        self.pointing_device.click_object(self._get_cancel_button())
        self.wait_until_destroyed()


class BrowserPageHeader(browser.BrowserPageHeader):

    @classmethod
    def validate_dbus_object(cls, path, state):
        return validate_dbus_object(path, state, APP_PATH_ROOT,
                                    b'BrowserPageHeader')

    @click_object
    def click_edit_button(self):
        return self.wait_select_single('UCAbstractButton', iconName='edit',
                                       visible=True)

    @click_object
    def click_delete_button(self):
        return self.wait_select_single('UCAbstractButton', iconName='delete',
                                       visible=True)

    @click_object
    def click_select_all_button(self):
        return self.wait_select_single('UCAbstractButton', iconName='select',
                                       visible=True)

    @click_object
    def click_back_button(self):
        return self.wait_select_single('UCAbstractButton',
                                       objectName='backButton', visible=True)
