# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import fixtures
import json
import os
from tempfile import NamedTemporaryFile
from unittest import mock

import testscenarios

from ubuntu_system_tests import (
    config,
    command_line,
    selftests
)


def check_any_item_in_list_contains_value(list, value):
    return any(item.find(value) for item in list)


def check_any_item_in_list_startswith_value(list, value):
    return any(item.startswith(value) for item in list)


class CommandLineTestCase(selftests.ConfigBaseTestCase):

    def setUp(self):
        super().setUp()
        self.config_stack = config.UbuntuSystemTestsStack()

    def test_setup_network_command_contains_credentials(self):
        self.set_config_values()

        command = command_line._get_device_network_command(
            'test_ssid', 'test_password')

        self.assertTrue(command.startswith(
            'nmcli device wifi connect \'test_ssid\' '
            'password \'test_password\''))

    def test_enable_introspection_with_serial(self):
        serial = 'test_serial'
        self.set_config_values(device_serial=serial)

        command = command_line._build_enable_autopilot_introspection_command(
            self.config_stack)

        self.assertEqual(['phablet-config', '-s', 'test_serial'], command[:3])

    def test_enable_introspection_without_serial(self):
        self.set_config_values(device_serial='')

        command = command_line._build_enable_autopilot_introspection_command(
            self.config_stack)

        self.assertNotIn('-s', command)

    def test_adt_run_must_be_called_with_short_timeout(self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertIn('--timeout-short=600', run_arguments)

    def test_ppa_setup_commands_set_image_readwrite(self):
        self.set_config_values()
        self.assertIn(
            'mount -o remount,rw /',
            ' '.join(command_line._build_adt_run_test_command(
                self.config_stack))
        )

    def test_adt_run_command_set_image_back_to_readonly(self):
        self.set_config_values()
        self.assertIn(
            'mount -o remount,ro /',
            ' '.join(command_line._build_adt_run_test_command(
                self.config_stack))
        )

    def test_ppa_setup_commands_only_update_ppa_details(self):
        aptget_command = str('apt-get --no-list-cleanup update -o'
                             ' Dir::Etc::SourceList=/dev/null')
        self.assertTrue(
            check_any_item_in_list_contains_value(
                command_line._get_ppa_and_testability_setup_command(),
                aptget_command)
        )

    def test_ppa_setup_commands_install_testability_libs(self):
        self.assertTrue(
            check_any_item_in_list_contains_value(
                command_line._get_ppa_and_testability_setup_command(),
                'apt-get install -y --no-install-recommends '
                'qttestability-autopilot')
        )

    def test_ppa_setup_commands_used_in_adt_run(self):
        self.set_config_values()
        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertIn(
            'apt-add-repository -y ppa:canonical-platform-qa/selenium',
            run_arguments
        )

    def test_adt_run_with_serial(self):
        serial = 'test_serial'
        self.set_config_values(device_serial=serial)

        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        _, adb_arguments = command.split('ssh -s adb --')
        self.assertIn('-s test_serial', adb_arguments)

    def test_adt_run_without_serial(self):
        self.set_config_values(device_serial='')

        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        _, adb_arguments = command.split('ssh -s adb --')
        self.assertNotIn('-s test_serial', adb_arguments)

    def test_adt_run_without_apt_update_must_be_called_without_setup_command(
            self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(
            self.config_stack, update_apt=False)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertNotIn('apt-get update', run_arguments)

    def test_adt_run_with_apt_update_must_be_called_with_setup_command(self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(
            self.config_stack, update_apt=True)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertIn('apt-get update', run_arguments)

    def test_adt_run_command_with_network_ready_creates_no_network(self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(
            self.config_stack, network_ready=True)

        self.assertFalse(check_any_item_in_list_startswith_value(
            command, 'nmcli device wifi connect'))

    def test_adt_run_command_with_default_parameters_creates_network(self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(
            self.config_stack)

        self.assertTrue(check_any_item_in_list_startswith_value(
            command, 'nmcli device wifi connect'))

    def test_get_config_stack_without_tests_to_run(self):
        self.set_config_values(device_serial='')

        config_stack = command_line._get_config_stack(
            'ubuntu-system-tests.conf', tests_to_run=[])

        self.assertEqual('', config_stack.get('tests_to_run'))

    def test_get_config_stack_with_tests_to_run(self):
        self.set_config_values(device_serial='')

        config_stack = command_line._get_config_stack(
            'ubuntu-system-tests.conf',
            tests_to_run=['test1', 'test2', 'test3'])

        self.assertEqual(
            'test1 test2 test3', config_stack.get('tests_to_run'))

    def test_get_extra_dependencies(self):
        tests = ('app1.tests.test_module.TestClass.test_x '
                 'app2.tests.test_module.TestClass.test_y '
                 'app3.tests.test_module.TestClass.test_z')
        self.set_config_values(tests_to_run=tests)
        expected_depends = ['dep1',
                            'dep2']
        # Mock _read_dependency_files
        with mock.patch(
            'ubuntu_system_tests.command_line._read_dependency_files'
        ) as mock_read_depends:
            mock_read_depends.return_value = {
                'app2.tests': 'dep1',
                'app3.tests': 'dep2'
            }
            extra_depends = command_line._get_extra_dependencies(
                self.config_stack.get('tests_to_run').split()
            )
        self.assertCountEqual(extra_depends, expected_depends)

    def test_read_dependency_file(self):
        with fixtures.TempDir() as deps_dir:
            deps_file1 = open(os.path.join(deps_dir.path, 'deps1.json'), 'w')
            deps_file2 = open(os.path.join(deps_dir.path, 'deps2.json'), 'w')
            deps_json1 = {
                "test_dependencies": [
                    {"id": "app1.tests", "depends": "dep1"}
                ]
            }
            deps_json2 = {
                "test_dependencies": [
                    {"id": "app2.tests", "depends": "dep2"}
                ]
            }
            deps_file1.write(json.dumps(deps_json1))
            deps_file2.write(json.dumps(deps_json2))
            deps_file1.close()
            deps_file2.close()

            expected = {'app1.tests': 'dep1', 'app2.tests': 'dep2'}
            actual = command_line._read_dependency_files(deps_dir.path)
            self.assertEqual(expected, actual)

    def test_extra_dependencies_for_test(self):
        test = 'app1.tests.test_module.TestClass.test_y'
        dependencies_map = {'app1.tests': 'dep1', 'app2.tests': 'dep2'}
        extra_depends = command_line._extra_dependencies_for_test(
            test, dependencies_map
        )
        self.assertEqual(extra_depends, ['dep1'])

    def test_create_temp_control(self):
        with NamedTemporaryFile() as control:
            control.write("""Tests: systemtests
Depends: dep1,
         dep2,
         dep3,

Tests: someothertests
Depends: other1,
         other2,
         other3,""".encode('utf-8'))
            control.flush()
            temp_control = command_line._create_temp_control_file(
                ['dep4', 'dep5'], control.name
            )
            with open(temp_control) as temp:
                tmp_control_content = temp.read()
                expected_content = """Tests: systemtests
Depends: dep1,
         dep2,
         dep3,
         dep4,
         dep5,

Tests: someothertests
Depends: other1,
         other2,
         other3,"""
                self.assertEqual(tmp_control_content, expected_content)


class RunSystemTestsCommandsTestCase(selftests.ConfigBaseTestCase,
                                     testscenarios.TestWithScenarios):

    scenarios = [
        # The nmcli network setup command is passed as --setup-command option
        # to adt-run. Verify that it should not run as separate command.
        ('without network setup', {
            'keyword_arguments': {'update_apt': False,
                                  'network_ready': True},
            'expected_commands': ['phablet-config',
                                  'adt-run'],
        }),
        ('with network setup', {
            'keyword_arguments': {'update_apt': False,
                                  'network_ready': False},
            'expected_commands': ['phablet-config',
                                  'adt-run'],
        }),
    ]

    def setUp(self):
        super().setUp()
        self.config_stack = config.UbuntuSystemTestsStack()

    @mock.patch('ubuntu_system_tests.command_line.generate_user_report')
    def test_run_system_tests_commands(self, mock_generate_user_report):
        def get_command_from_call(call):
            """Return the name of the command, without the arguments."""
            return call[0][0][0]

        self.set_config_values()

        # Do not execute the subprocesses, just record their calls.
        with mock.patch('subprocess.Popen') as mock_popen:
            enter = mock_popen().__enter__()
            # Make sure that check_output and call return with no errors.
            enter.communicate = mock.Mock(return_value=(0, 'dummy'))
            enter.poll = mock.Mock(return_value=0)
            enter.wait = mock.Mock(return_value=0)

            conf_name = os.path.basename(self.config_stack.store.path)
            command_line.run_system_tests(conf_name, [],
                                          **self.keyword_arguments)

        # The first call doesn't run any command.
        calls = mock_popen.call_args_list[2:]
        commands = [get_command_from_call(call) for call in calls]
        self.assertEqual(self.expected_commands, commands)


class MainPassingCommandLineArgumentsTestCase(testscenarios.TestWithScenarios):

    DEFAULT_CONFIG_FILE = 'ubuntu-system-tests.conf'
    RUN_ALL_TESTS = []
    DO_APT_UPDATE = True
    DONT_APT_UPDATE = False
    NETWORK_ALREADY_SETUP = True
    NETWORK_NOT_ALREADY_SETUP = False
    DO_INSTALL_DEPS = True
    DONT_INSTALL_DEPS = False
    DO_INSTALL_TESTS = True
    DONT_INSTALL_TESTS = False
    VISUALIZATION_ON = True
    VISUALIZATION_OFF = False
    DO_USE_SILENT = True
    DONT_USE_SILENT = False
    DO_USE_CACHE_DEPS = True
    DONT_USE_CACHE_DEPS = False
    DO_USE_VERBOSE_LOGGING = True
    DONT_USE_VERBOSE_LOGGING = False

    scenarios = [
        ('no arguments', {
            'command_line_arguments': [],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                RUN_ALL_TESTS,
                DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('alternative config', {
            'command_line_arguments': ['--config=alternative.conf'],
            'parsed_arguments': (
                'alternative.conf',
                RUN_ALL_TESTS,
                DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with apt-get update', {
            'command_line_arguments': ['--update-apt'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                RUN_ALL_TESTS,
                DO_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('one test suite', {
            'command_line_arguments': ['test_suite'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                ['test_suite'],
                DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('multiple test suites', {
            'command_line_arguments': [
                'test_suite1', 'test_suite2', 'test_suite3'
            ],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                ['test_suite1', 'test_suite2', 'test_suite3'],
                DONT_APT_UPDATE, NETWORK_NOT_ALREADY_SETUP, VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with network already setup', {
            'command_line_arguments': ['--network-ready'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with touch visualisation enabled', {
            'command_line_arguments': ['--touch-visualization'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_ON,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with install dependencies enabled', {
            'command_line_arguments': ['--install-deps'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DO_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with install dependencies enabled', {
            'command_line_arguments': ['--install-tests'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DO_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with silent mode specified', {
            'command_line_arguments': ['--silent'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DO_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with cache deps specified', {
            'command_line_arguments': ['--cache-deps'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DO_USE_CACHE_DEPS,
                DONT_USE_VERBOSE_LOGGING,
            )
        }),
        ('with verbose logging specified', {
            'command_line_arguments': ['--verbose'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE,
                NETWORK_NOT_ALREADY_SETUP,
                VISUALIZATION_OFF,
                DONT_INSTALL_DEPS,
                DONT_INSTALL_TESTS,
                DONT_USE_SILENT,
                DONT_USE_CACHE_DEPS,
                DO_USE_VERBOSE_LOGGING,
            )
        }),
    ]

    @mock.patch('ubuntu_system_tests.command_line.run_system_tests')
    def test_main_must_pass_command_line_arguments_to_run_system_tests(
            self, mock_run_system_tests):
        command_line.main(self.command_line_arguments)

        mock_run_system_tests.assert_called_once_with(
            *self.parsed_arguments)
