# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014-2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import io
import os
from unittest import mock

import fixtures
from uciconfig import options
from autopilot.introspection import ProxyBase

from ubuntu_system_tests import (
    config,
    selftests,
)
from ubuntu_system_tests.helpers.autopilot import override_proxy_timeout


class HelpersTestCase(selftests.ConfigBaseTestCase):

    def get_mock_proxy(self):
        """Returns a rubbish proxy object that doesn't actually do anything."""
        return ProxyBase({'id': [0, 0]}, b'/root', None)

    def test_set_proxy_time_updates_polltime(self):
        new_poll_time = 100
        proxy = self.get_mock_proxy()
        self.assertNotEqual(proxy._poll_time, new_poll_time)

        with override_proxy_timeout(proxy, new_poll_time):
            self.assertEqual(proxy._poll_time, new_poll_time)

    def test_set_proxy_time_resets_polltime(self):
        new_poll_time = 100
        proxy = self.get_mock_proxy()
        self.assertNotEqual(proxy._poll_time, new_poll_time)

        original_timeout = proxy._poll_time

        with override_proxy_timeout(proxy, new_poll_time):
            pass

        self.assertEqual(proxy._poll_time, original_timeout)


class ConfigDirTestCase(selftests.ConfigBaseTestCase):

    def test_default_config_dir_must_be_in_home(self):
        config_dir = os.path.join(os.environ.get('HOME'), '.config')
        self.assertEqual(config_dir, config.get_user_config_dir())

    def test_xdg_config_home_env_var_must_override_config_dir(self):
        xdg_dir = os.path.join(self.uniq_dir, 'xdg')
        os.environ['XDG_CONFIG_HOME'] = xdg_dir
        self.assertEqual(xdg_dir, config.get_user_config_dir())


class RegisterTestsOptionsTestCase(selftests.ConfigBaseTestCase):

    def setUp(self):
        super().setUp()
        patcher = mock.patch(
            'ubuntu_system_tests.config.CONFIG_OPTIONS', new=[])
        patcher.start()
        self.addCleanup(patcher.stop)
        self.test_option = options.Option(
            'test.option', default=options.MANDATORY, help_string='test help')

    def register_test_options(self):
        options.option_registry.register(self.test_option)

    def test_register_external_test_options(self):
        config.CONFIG_OPTIONS = [
            options.Option(
                'externaloption1', default=options.MANDATORY,
                help_string='external test option help')
        ]
        config.register_test_options()

        self.assertEqual(
            {'externaloption1': config.CONFIG_OPTIONS[0]},
            options.option_registry.data)
        self.assertEqual(
            {'externaloption1': 'external test option help'},
            options.option_registry.help)

    def test_register_test_options_without_config_options(self):
        """Test that the registry must be empty when there are no options."""
        config.register_test_options()

        self.assertEqual({}, options.option_registry.data)
        self.assertEqual({}, options.option_registry.help)


class UbuntuSystemTestsStackTestCase(selftests.ConfigBaseTestCase):

    def test_store_location(self):
        config_stack = config.UbuntuSystemTestsStack()

        expected_config_path = os.path.join(
            os.environ.get('HOME'), '.config', 'ubuntu-system-tests.conf')
        self.assertEqual(expected_config_path, config_stack.store.path)

    def test_get_autopilot_config_string(self):
        config_stack = config.UbuntuSystemTestsStack()
        config_stack.store._load_from_string(
            'option1 = value1\noption2 = value2\n')

        autopilot_config_string = config_stack.get_autopilot_config_string()
        self.assertEqual(
            'option1=value1,option2=value2', autopilot_config_string)


def register_test_options(only_default=False):
    """Register test options used for testing in the options registry.

    :param only_default: If True then only register test options that have a
                         default value specified. If False then all options
                         will be registered.
    """
    test_options = []
    if not only_default:
        # Mandatory options.
        mandatory_option1 = options.Option(
            'test.option1', default=options.MANDATORY,
            help_string='Test option 1')
        mandatory_option2 = options.Option(
            'test.option2', default=options.MANDATORY,
            help_string='Test option 2')
        # Optional options without default.
        optional_option3 = options.Option(
            'test.option3', help_string='Test option 3')
        optional_option4 = options.Option(
            'test.option4', help_string='Test option 4')
        test_options.extend([
            mandatory_option1, mandatory_option2, optional_option3,
            optional_option4])
    # Optional options with default.
    default_option5 = options.Option(
        'test.option5', default='default option5 value',
        help_string='Test option 5')
    default_option6 = options.Option(
        'test.option6', default='default option6 value',
        help_string='Test option 6')
    # Options with values already existing in the config file.
    existing_option7 = options.Option('test.option7', help_string='dummy')
    existing_option8 = options.Option('test.option8', help_string='dummy')

    test_options.extend([
        default_option5, default_option6, existing_option7, existing_option8])

    for option in test_options:
        options.option_registry.register(option)


class MockedConfigIO(fixtures.Fixture):

    def __init__(self, *input_option_values):
        super().__init__()
        self.input_option_values = input_option_values

    def setUp(self):
        super().setUp()
        self.stdout = self._patch_stdout()
        stdin = self._patch_stdin()
        self._write_option_values_to_stream(
            stdin, self.input_option_values)

    def _patch_stdout(self):
        stdout_patcher = mock.patch('sys.stdout', new_callable=io.StringIO)
        self.addCleanup(stdout_patcher.stop)
        return stdout_patcher.start()

    def _patch_stdin(self):
        stdin_patcher = mock.patch('sys.stdin', new_callable=io.StringIO)
        self.addCleanup(stdin_patcher.stop)
        return stdin_patcher.start()

    def _write_option_values_to_stream(self, stream, option_values):
        for option_value in option_values:
            stream.write(option_value + '\n')
        stream.seek(0)


class UbuntuSystemTestsStackWithTestOptionsTestCase(
        selftests.ConfigBaseTestCase):

    ALL_EXISTING_OPTIONS = [
        ('test.option7', 'option7 existing value'),
        ('test.option8', 'option8 existing value')
    ]
    MISSING_MANDATORY_OPTION_VALUES = [
        'option1 value',
        'option2 value',
    ]
    MISSING_OPTION_VALUES_WITHOUT_DEFAULT = [
        'option3 value',
        'option4 value',
    ]
    MISSING_OPTION_VALUES_WITH_DEFAULT = [
        'option5 value',
        'option6 value',
    ]
    ALL_MISSING_OPTION_VALUES = (
        MISSING_MANDATORY_OPTION_VALUES +
        MISSING_OPTION_VALUES_WITHOUT_DEFAULT +
        MISSING_OPTION_VALUES_WITH_DEFAULT
    )

    def load_stack_with_existing_options(self, only_default=False):
        register_test_options(only_default)
        options = self.ALL_EXISTING_OPTIONS
        config_stack = config.UbuntuSystemTestsStack()
        config_string = '\n'.join(
            ['{} = {}\n'.format(option, value) for option, value in options]
        )
        config_stack.store._load_from_string(config_string)
        return config_stack

    def test_alternative_store_location(self):
        get_user_config_dir = 'ubuntu_system_tests.config.get_user_config_dir'
        with mock.patch(get_user_config_dir) as mock_config_dir:
            mock_config_dir.return_value = self.uniq_dir
            config_stack = config.UbuntuSystemTestsStack('alternative.conf')

        expected_config_path = os.path.join(
            self.uniq_dir, 'alternative.conf')
        self.assertEqual(expected_config_path, config_stack.store.path)

    def test_get_missing_mandatory_options_from_stdin(self):
        """Test that values for missing mandatory options are in the stack."""
        config_stack = self.load_stack_with_existing_options()
        with MockedConfigIO(*self.ALL_MISSING_OPTION_VALUES):
            config_stack.get_missing_options()

        self.assertEqual('option1 value', config_stack.get('test.option1'))
        self.assertEqual('option2 value', config_stack.get('test.option2'))

    def test_get_missing_optional_options_without_default_from_stdin(self):
        """Test that values for missing optional options are in the stack."""
        config_stack = self.load_stack_with_existing_options()
        with MockedConfigIO(*self.ALL_MISSING_OPTION_VALUES):
            config_stack.get_missing_options()

        self.assertEqual(
            'option3 value', config_stack.get('test.option3'))
        self.assertEqual(
            'option4 value', config_stack.get('test.option4'))

    def test_get_missing_optional_options_with_default_from_stdin(self):
        """Test that values for missing optional options are in the stack."""
        config_stack = self.load_stack_with_existing_options()
        with MockedConfigIO(*self.ALL_MISSING_OPTION_VALUES):
            config_stack.get_missing_options()

        self.assertEqual('option5 value', config_stack.get('test.option5'))
        self.assertEqual('option6 value', config_stack.get('test.option6'))

    def test_get_missing_options_must_not_query_for_existing_options(self):
        config_stack = self.load_stack_with_existing_options()
        with MockedConfigIO(*self.ALL_MISSING_OPTION_VALUES):
            config_stack.get_missing_options()

        self.assertEqual(
            'option7 existing value', config_stack.get('test.option7'))
        self.assertEqual(
            'option8 existing value', config_stack.get('test.option8'))

    def test_get_missing_options_not_entering_options_with_default(self):
        """Test that values from missing options are recorded into the stack.

        Values for options with a default can be left blank.

        """
        config_stack = self.load_stack_with_existing_options()
        option_values = (
            self.MISSING_MANDATORY_OPTION_VALUES +
            self.MISSING_OPTION_VALUES_WITHOUT_DEFAULT +
            # Leave blank to use the default.
            ['', '']
        )
        with MockedConfigIO(*option_values):
            config_stack.get_missing_options()

        self.assertEqual(
            'default option5 value', config_stack.get('test.option5'))
        self.assertEqual(
            'default option6 value', config_stack.get('test.option6'))

    def test_get_missing_options_not_entering_options_without_default(self):
        """Test that values from missing options are recorded into the stack.

        Values for optional options without a default can be left blank.

        """
        config_stack = self.load_stack_with_existing_options()
        option_values = (
            self.MISSING_MANDATORY_OPTION_VALUES +
            # Leave blank, not required.
            ['', ''] +
            self.MISSING_OPTION_VALUES_WITH_DEFAULT
        )
        with MockedConfigIO(*option_values):
            config_stack.get_missing_options()

        self.assertEqual('', config_stack.get('test.option3'))
        self.assertEqual('', config_stack.get('test.option4'))

    def test_save_missing_options_to_config_file(self):
        """Test that values from missing options are saved into the file.

        Options already existing in the file must remain the same.

        """
        config_stack = self.load_stack_with_existing_options()
        with MockedConfigIO(*self.ALL_MISSING_OPTION_VALUES):
            config_stack.get_missing_options()

        with open(config_stack.store.path, 'r') as config_file:
            config_lines = config_file.readlines()

        expected_config_lines = [
            # Already existing options.
            'test.option7 = option7 existing value\n',
            'test.option8 = option8 existing value\n',
            # New options added.
            'test.option1 = option1 value\n', 'test.option2 = option2 value\n',
            'test.option3 = option3 value\n', 'test.option4 = option4 value\n',
            'test.option5 = option5 value\n', 'test.option6 = option6 value\n',
        ]
        self.assertEqual(expected_config_lines, config_lines)

    def test_save_missing_options_to_config_file_in_silent_mode(self):
        """Test that values with missing options are automatically assigned
        their default value and saved into the file in silent mode without
        having to prompt the user.

        Options already existing in the file must remain the same.

        """
        config_stack = self.load_stack_with_existing_options(only_default=True)
        config_stack.get_missing_options(silent=True)

        with open(config_stack.store.path, 'r') as config_file:
            config_lines = config_file.readlines()

        expected_config_lines = [
            # Already existing options.
            'test.option7 = option7 existing value\n',
            'test.option8 = option8 existing value\n',
            # New options added using default
            'test.option5 = default option5 value\n',
            'test.option6 = default option6 value\n',
        ]
        self.assertEqual(expected_config_lines, config_lines)

    def test_get_missing_options_from_stdin_prints_help(self):
        """Test that the help from missing options is printed to stdout.

        Help from the options already existing in the config must not be
        printed.

        """
        config_stack = self.load_stack_with_existing_options()
        with MockedConfigIO(*self.ALL_MISSING_OPTION_VALUES) as mocked_io:
            config_stack.get_missing_options()

        expected_prints = (
            'Test option 1: Test option 2: '
            # Optional options without default print that they can be left
            # blank.
            'Test option 3 (leave blank if not required): '
            'Test option 4 (leave blank if not required): '
            # Optional options with default print leaving blank as an option.
            'Test option 5 '
            '(leave blank to use default value: default option5 value): '
            'Test option 6 '
            '(leave blank to use default value: default option6 value): ')
        self.assertEqual(expected_prints, mocked_io.stdout.getvalue())
