# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import tempfile
import unittest
from unittest import mock

import types

import fixtures
import testscenarios

import ubuntu_system_tests
from ubuntu_system_tests import (
    config,
    external_tests,
    selftests,
)


class ExternalTestsListFile(fixtures.Fixture):

    def __init__(self, tests):
        super().__init__()
        if isinstance(tests, list):
            self.contents = '\n'.join(tests)
        else:
            self.contents = tests

    def setUp(self):
        super().setUp()
        _, self.path = tempfile.mkstemp()
        self.addCleanup(os.remove, self.path)
        with open(self.path, 'w') as test_lists_file:
            test_lists_file.write(self.contents)


class GetExternalIdsTestsTestCase(testscenarios.TestWithScenarios):

    scenarios = [
        ('empty file', {
            'file_contents': '',
            'expected_test_ids': []
        }),
        ('file with spaces', {
            'file_contents': '   \n\n   \n  ',
            'expected_test_ids': []
        }),
        ('one test', {
            'file_contents': 'module.suite.test1',
            'expected_test_ids': ['module.suite.test1']
        }),
        ('multiple tests', {
            'file_contents': (
                'module.suite.test1\n'
                'module.suite.test2\n'
                'module.suite.test3\n'
            ),
            'expected_test_ids': [
                'module.suite.test1',
                'module.suite.test2',
                'module.suite.test3'
            ]
        }),
        ('tests with spaces', {
            'file_contents': (
                '\n'
                '   module.suite.test1\n'
                '\n'
                'module.suite.test2   \n'
                '   module.suite.test3   \n'
                '\n'
            ),
            'expected_test_ids': [
                'module.suite.test1',
                'module.suite.test2',
                'module.suite.test3'
            ]
        })
    ]

    def test_get_external_test_ids(self):
        with ExternalTestsListFile(self.file_contents) as test_lists_file:
            test_ids = external_tests.get_external_test_ids(
                test_lists_file.path)
        self.assertEqual(self.expected_test_ids, test_ids)


class FakeProcessHelpers():

    jobs = {}
    # A dictionary to hold job names and their status.

    def is_job_running(self, job):
        return self.jobs.get(job, False)

    def start_job(self, job):
        self.jobs[job] = True

    def stop_job(self, job):
        self.jobs[job] = False


class FakeGreeterHelpers():

    active = True

    def wait_for_greeter(self):
        assert self.active

    def hide_greeter_with_dbus(self):
        self.active = False


@mock.patch(
    'ubuntu_system_tests.helpers.unity8.greeter',
    new_callable=FakeGreeterHelpers)
@mock.patch(
    'ubuntu_system_tests.helpers.unity8.process_helpers',
    new_callable=FakeProcessHelpers)
class EnsureUnityInRequiredStateTestCase(unittest.TestCase):

    def test_stoped_unity_to_required_state_for_unity8_tests(
            self, fake_process_helpers, fake_greeter_helpers):
        """Test that unity8 remains stoped for unity8 tests."""
        fake_process_helpers.stop_job('unity8')
        external_tests.ensure_unity_in_required_state('unity8.test')

        self.assertFalse(fake_process_helpers.is_job_running('unity8'))

    def test_started_unity_to_required_state_for_unity8_tests(
            self, fake_process_helpers, fake_greeter_helpers):
        """Test that unity8 is stoped for unity8 tests."""
        fake_process_helpers.start_job('unity8')
        external_tests.ensure_unity_in_required_state('unity8.test')

        self.assertFalse(fake_process_helpers.is_job_running('unity8'))

    def test_stoped_unity_to_required_state_for_application_tests(
            self, fake_process_helpers, fake_greeter_helpers):
        """Test that unity8 remains started for application tests."""
        fake_process_helpers.stop_job('unity8')
        external_tests.ensure_unity_in_required_state('notunity8.test')

        self.assertTrue(fake_process_helpers.is_job_running('unity8'))

    def test_started_unity_to_required_state_for_application_tests(
            self, fake_process_helpers, fake_greeter_helpers):
        """Test that unity8 is started for application tests."""
        fake_process_helpers.start_job('unity8')
        external_tests.ensure_unity_in_required_state('notunity8.test')

        self.assertTrue(fake_process_helpers.is_job_running('unity8'))


class GetTestIdsToRunTestCase(selftests.ConfigBaseTestCase):

    def test_get_tests_with_no_internal_tests_module(self):
        config_stack = config.UbuntuSystemTestsStack()
        external_tests = ['external_test_1']
        with ExternalTestsListFile(external_tests) as test_lists_file:
            tests_to_run = ubuntu_system_tests.get_tests_ids_to_run(
                config_stack,
                tests_module='not a module',
                testsuite_list_filepath=test_lists_file.path)

        self.assertEqual(external_tests, tests_to_run)

    def test_get_tests_with_internal_tests_module(self):
        """Test that the internal tests module is always in the list to run."""
        config_stack = config.UbuntuSystemTestsStack()
        external_tests = ['external_test_1']
        tests_module = types.ModuleType('internal_tests_module')

        with ExternalTestsListFile(external_tests) as test_lists_file:
            tests_to_run = ubuntu_system_tests.get_tests_ids_to_run(
                config_stack, tests_module, test_lists_file.path)

        expected_tests = external_tests + ['internal_tests_module']
        self.assertEqual(expected_tests, tests_to_run)

    def tests_get_tests_from_config(self):
        config_stack = config.UbuntuSystemTestsStack()
        config_stack.store._load_from_string('tests_to_run=test1 test2 test3')

        tests_to_run = ubuntu_system_tests.get_tests_ids_to_run(config_stack)

        expected_tests = ['test1', 'test2', 'test3']
        self.assertEqual(expected_tests, tests_to_run)
