#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import getpass
import glob
import os
import re
import socket
import stat
import subprocess

from ubuntu_system_tests.helpers.file_system import (
    get_random_string,
    delete_file
)

SSH_IP = 'ssh_ip'
SSH_PASSPHRASE = 'ssh_passphrase'
SSH_PRIVATE_KEY = 'ssh_private_key'
SSH_USER = 'ssh_user'


def _get_local_ip_address():
    """ Retrieve the local ip address
    :return: The ip obtained from the socket connection
    """
    ip = '127.0.0.1'
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def configure_config_stack(config_stack):
    """ Retrieve a private key that can be used to connect to the host and add
     the public key to the authorized_keys if it is not included yet. If either
     it is not required to configure ssh or the ssh password is defined,
     keypairs are not used and the function return None
     """
    config_stack.set(SSH_IP, _get_local_ip_address())
    config_stack.set(SSH_USER, getpass.getuser())

    # Add the private key and the passphrase when it is needed
    private_key = get_valid_key()
    if not private_key:
        private_key, public_key = generate_keypair(get_random_string(10))
        add_public_key_to_ssh_uthorized_keys(public_key)
        os.remove(public_key)

    config_stack.set(SSH_PASSPHRASE, _get_key_passprhase(private_key))
    config_stack.set(SSH_PRIVATE_KEY, get_device_keyfile_path(private_key))

    config_stack.save()


def _get_key_passprhase(private_key):
    """ Get the passphrase from the name, None if it is not found """
    keyname = os.path.basename(private_key)
    match = re.search(r'test_(.+)_key', keyname)
    if match:
        return match.group(1)
    else:
        return


def _get_ssh_authorized_keys_path():
    return os.path.join(os.path.expanduser('~'), '.ssh', 'authorized_keys')


def is_authorized_key(id):
    """ Indicate if the comment it is part of the ssh authorized_keys file
    :param id: a string used to identify the public key. Currently the
    passphrase is being used for that
    """
    authorized_keys = _get_ssh_authorized_keys_path()
    command_check = 'grep "{comment}" {path}'.format(comment=id,
                                                     path=authorized_keys)
    return subprocess.call(command_check,
                           shell=True,
                           stdout=subprocess.PIPE) == 0


def add_public_key_to_ssh_uthorized_keys(public_key):
    """ Add the public key to the ssh authorized_keys if is not included yet"""
    # In case the public key is not authorized yet, add it to the
    # authorized_keys file
    authorized_keys = _get_ssh_authorized_keys_path()
    command_add = 'cat {public_key} >> {authorized_keys}'.format(
        public_key=public_key, authorized_keys=authorized_keys)
    subprocess.call(command_add, shell=True)


def get_valid_key():
    """ Retrieve a private key which has its public key is already authorized
    to be used through ssh
    """
    all_keys = get_all_defined_keys()
    for key in all_keys:
        passphrase = _get_key_passprhase(key)
        if passphrase and is_authorized_key(passphrase):
            return key
    return


def get_all_defined_keys():
    """ Retrieve the keys already saved in the tmp dir """
    return glob.glob('/tmp/test_*_key')


def generate_keypair(passphrase):
    """ Create a pair of keys with the passphrase as part of the key names """
    keypath = '/tmp/test_{}_key'.format(passphrase)
    command = 'ssh-keygen -t rsa -b 4096 -C "{passphrase}" -P "{passphrase}" -f {keypath} -q'  # NOQA
    command = command.format(passphrase=passphrase,
                             keypath=keypath)
    subprocess.check_call(command, shell=True)
    return keypath, keypath + '.pub'


def get_device_keyfile_path(filepath):
    """Return the path of the config file copied to the device."""
    return os.path.join('/tmp', os.path.basename(filepath))


def prepare_device_keys(config_stack):
    """ Prepare the keyfile to be used by changing the ownership and the
    permissions. These changes are done when other ssh authentication
    mechanisms have not being selected
    """
    passphrase = config_stack.get(SSH_PASSPHRASE)
    key_file = config_stack.get(SSH_PRIVATE_KEY)

    if passphrase and key_file and os.path.isfile(key_file):
        os.chown(key_file, os.getuid(), os.getgid())
        os.chmod(key_file, stat.S_IREAD)


def ensure_device_keyfile_deleted(config_stack):
    """ Delete the private key file in the device when required  """
    key_file = config_stack.get(SSH_PRIVATE_KEY)
    not key_file or delete_file(key_file)
