#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import string
import re
import os.path

import debian.deb822
import debian.debian_support
import debian.debfile

#
# Abstract test representation
#

logger = logging.getLogger(__name__)


class Unsupported(Exception):
    """ Test cannot be run in the testbed """

    def __init__(self, testname, message):
        self.testname = testname
        self.message = message

    def __str__(self):
        return 'Unsupported test %s: %s' % (self.testname, self.message)


class InvalidControl(Exception):
    """ Test has invalid control data """

    def __init__(self, testname, message):
        self.testname = testname
        self.message = message

    def __str__(self):
        return 'InvalidControl test %s: %s' % (self.testname, self.message)


def _get_rfc822_lines(filepath, skip_tag='SKIPLINE'):
    """ Get the file lines and filter out comments, python-debian doesn't
    do that (http://bugs.debian.org/743174) """
    lines = []

    if not os.path.exists(filepath):
        raise RuntimeError('File {} does not exist'.format(filepath))

    with open(filepath) as file:
        for l in file:
            # ignore ^# as that breaks continuation lines and skip line tag
            if l.startswith('#') or skip_tag in l:
                continue

            # filter out comments which don't start on first column
            l = l.split('#', 1)[0]
            lines.append(l)

    return lines


def _parse_rfc822(path, skip_tag=None):
    """ Parse Debian-style RFC822 file. Yield dictionaries with the
    keys/values.
    """
    lines = _get_rfc822_lines(path, skip_tag)

    for p in debian.deb822.Deb822.iter_paragraphs(lines):
        r = {}
        for field, value in p.items():
            # un-escape continuation lines
            v = ''.join(value.split('\n')).replace('  ', ' ')
            field = string.capwords(field)
            r[field] = v
        yield r


def _debian_check_unknown_fields(name, record):
    unknown_keys = set(record.keys()).difference(
        {'Tests', 'Test-command', 'Restrictions', 'Features',
         'Depends', 'Tests-directory', 'Classes'})
    if unknown_keys:
        raise Unsupported(name, 'unknown field %s' % unknown_keys.pop())


def _debian_package_from_source(st):
    if 'Package' not in st:
        # source stanza
        return
    # filter out udebs and similar stuff which aren't "real" debs
    if st.get('Xc-package-type', 'deb') != 'deb' or \
            st.get('Package-type', 'deb') != 'deb':
        return
    arch = st['Architecture']
    if arch in ('all', 'any'):
        return '{} (>= 0~)'.format(st['Package'])
    else:
        return '{} (>= 0~) [{}]'.format(st['Package'], arch)


def _debian_packages_from_source(srcdir):
    packages = []

    for st in _parse_rfc822(os.path.join(srcdir, 'debian', 'control')):
        package = _debian_package_from_source(st)
        if package:
            packages.append(package)

    return packages


def _get_build_depends(st):
    deps = []
    if 'Build-depends' in st:
        for d in st['Build-depends'].split(','):
            dp = d.strip()
            if dp:
                deps.append(dp)
    return deps


def _get_build_depends_indep(st):
    deps = []
    if 'Build-depends-indep' in st:
        for d in st['Build-depends-indep'].split(','):
            dp = d.strip()
            if dp:
                deps.append(dp)
    return deps


def _debian_build_deps_from_source(srcdir):
    deps = []
    for st in _parse_rfc822(os.path.join(srcdir, 'debian', 'control')):
        deps.extend(_get_build_depends(st))
        deps.extend(_get_build_depends_indep(st))

    # @builddeps@ should always imply build-essential
    deps.append('build-essential')
    return deps


dep_re = re.compile(
    r'(?P<package>[a-z0-9+-.]+)(?::native)?\s*'
    r'(\((?P<relation><<|<=|>=|=|>>)\s*(?P<version>[^\)]*)\))?'
    r'(\s*\[[[a-z0-9+-. ]+\])?$')


def _debian_check_dep(testname, dep):
    """ Check a single Debian dependency """

    dep = dep.strip()
    m = dep_re.match(dep)
    if not m:
        raise InvalidControl(testname, "Test Depends field contains an "
                             "invalid dependency `%s'" % dep)
    if m.group("version"):
        try:
            debian.debian_support.NativeVersion(m.group('version'))
        except ValueError:
            raise InvalidControl(testname, "Test Depends field contains "
                                 "dependency `%s' with an "
                                 "invalid version" % dep)
        except AttributeError:
            # too old python-debian, skip the check
            pass


def _debian_check_deps(testname, alt_group_str):
    """ Check a multiple Debian dependencies """
    for dep in alt_group_str.split('|'):
        _debian_check_dep(testname, dep)
    return alt_group_str


def _parse_debian_depend(alt_group_str, srcdir, testname, deps):
    """ Parse sinple dependency from line in a Debian package """
    if not alt_group_str:
        # happens for empty depends or trailing commas
        return
    logger.debug('processing dependency %s' % alt_group_str)
    if alt_group_str == '@':
        deps.extend(_get_debian_packages_from_source(srcdir))
    elif alt_group_str == '@builddeps@':
        deps.extend(_get_debian_build_deps_from_source(srcdir))
    else:
        deps.append(_debian_check_deps(testname, alt_group_str))


def _parse_debian_depends(testname, dep_str, srcdir):
    """ Parse Depends: line in a Debian package

    Split dependencies (comma separated), validate their syntax, and expand @
    and @builddeps@. Return a list of dependencies.

    This may raise an InvalidControl exception if there are invalid
    dependencies.
    """
    deps = []
    for alt_group_str in dep_str.split(','):
        alt_group_str = alt_group_str.strip()
        _parse_debian_depend(alt_group_str.strip(), srcdir, testname, deps)
    return deps


def _get_debian_packages_from_source(srcdir):
    deps = []
    for d in _debian_packages_from_source(srcdir):
        logger.debug('synthesised dependency %s' % d)
        deps.append(d)
    return deps


def _get_debian_build_deps_from_source(srcdir):
    deps = []
    for d in _debian_build_deps_from_source(srcdir):
        logger.debug('synthesised dependency %s' % d)
        deps.append(d)
    return deps


def get_all_dependencies(srcdir, skip_tag=None):
    """ Get a list of dependencies from the tests control file
    :param srcdir: the path to the project dir
    :param skip_tag: if any, dependencies with this tag will be excluded
    :return: A dependencies list from the tests control file
    """

    control_path = os.path.join(srcdir, 'debian', 'tests', 'control')
    dependencies = []

    for record in _parse_rfc822(control_path, skip_tag):
        test_names = record['Tests'].replace(',', ' ').split()
        dependencies.extend(_parse_debian_depends(test_names[0],
                                                  record.get('Depends'),
                                                  srcdir))
    return dependencies
