# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from threading import Event

from ubuntu_system_tests import config
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers.addressbook import (
    fixture_setup as address_book_fixture)
from ubuntu_system_tests.helpers.notifications.observer import (
    launch_notification_observer)
from ubuntu_system_tests.helpers.ofono import fixture_setup as ofono_fixture
from ubuntu_system_tests.helpers.telephony import Telephony
from ubuntu_system_tests.tests.base import BaseUbuntuSystemTestCase

TIMEOUT_5_MIN = 300


class BaseTelephonyTestCase(BaseUbuntuSystemTestCase):
    """Base class to be used for all telephony tests."""

    def setUp(self):
        """Base class setUp method."""
        super().setUp()
        self.useFixture(address_book_fixture.AddressBookTestEnvironment())
        self.shell = self.restart_unity().main_window
        self.config_stack = config.get_device_config_stack()
        credentials = {'AccountSid': self.get_telephony_service_account_id(),
                       'AuthToken': self.get_telephony_service_auth_token()}
        self.telephony = Telephony(credentials)
        self.useFixture(ofono_fixture.UnlockSims())
        self.assert_sim_registered()
        self.notification_watcher = None
        # Used to signal when expected number of notifications has been found
        self.notifier_result = Event()

    def assert_sim_registered(self, sim='0'):
        """Wait until specified SIM is registered on the mobile network."""
        self.assertTrue(ofono.wait_for_sim_registered(sim))

    def assert_any_sim_registered(self):
        """Wait until any SIM is registered on the mobile network."""
        self.assertTrue(ofono.wait_for_any_sim_registered())

    def assert_all_sims_registered(self):
        """Wait until all available SIMs are registered on mobile network."""
        self.assertTrue(ofono.wait_for_all_sims_registered())

    def get_telephony_service_account_id(self):
        """Return account id for telephony service."""
        return self.config_stack.get('_twilio_account_sid')

    def get_telephony_service_auth_token(self):
        """Return auth token for telephony service."""
        return self.config_stack.get('_twilio_auth_token')

    def get_telephony_service_number1(self):
        """Return the number to call and message from."""
        return self.config_stack.get('telephony_service_number1')

    def get_telephony_service_number2(self):
        """Return the number to call and message from."""
        return self.config_stack.get('telephony_service_number2')

    def get_device_phone_number(self):
        """Return the number to send messages and calls to."""
        return self.config_stack.get('device_phone_number')

    def get_device_phone_number2(self):
        """Return the secondary number to send messages and calls to."""
        return self.config_stack.get('device_phone_number2')

    def start_notification_observer(self, expected_count=0, timeout=60):
        """
        Start a new thread to observe and dismiss any notifications displayed
        on screen after a call is made or message is sent.

        This will include USSD notifications sent by network operator to
        display remaining credit balance, or reply messages sent to the device.
        For new message notifications the observer won't interact with them,
        only wait until they are automatically closed. Any USSD messages
        received during this time period will be dismissed by pressing the
        Ok button.

        The notification observer will run in a sepearate thread so that the
        test is not blocked by waiting for USSD messages which may or may not
        appear, depending on network provider used.

        expected_count should specify the number of reply messages expected,
        not including any USSD messages. When this number of reply messages
        is reached, result signal will be set.

        If expected_count is 0, then the observer will dismiss a single USSD
        notification before signalling the result. If the observer reaches
        the specified timeout before a USSD notification is observed then it
        will stop automatically at this time.

        The observer will be stopped automatically during test cleanup actions,
        but the caller can also stop it prior to this.

        Note: If the test needs to wait for the specified number of messages to
        be received it should call wait_for_all_notifications().

        :param expected_count: The number of reply messages expected. This
                               should not include USSD messages.
        :param timeout: How long to run the observer thread for.

        """
        notifications = self.shell.get_notifications_list()
        self.notification_observer = launch_notification_observer(
            result=self.notifier_result,
            notifications=notifications,
            expected_count=expected_count,
            timeout=TIMEOUT_5_MIN)
        self.notification_observer.start()
        self.addCleanup(self.stop_notification_observer)

    def wait_for_all_notifications(self):
        """Wait until all expected notifications are dismissed."""
        self.notifier_result.wait()

    def stop_notification_observer(self):
        """Stop the notification observer."""
        self.notification_observer.stop()
