# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import shutil

from autopilot.matchers import Eventually
from testtools.matchers import Equals

from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers import music
from ubuntu_system_tests.helpers import system_settings
from ubuntu_system_tests.helpers import indicators
from ubuntu_system_tests.helpers.music import fixture_setup as music_fixture

from ubuntu_system_tests.tests import base
from ubuntu_system_tests.tests import data as test_data


SOUND_SETTINGS_PAGE = 'soundPage'


class SoundIndicatorTestCase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.restart_unity()
        self.indicator_page = indicators.open_sound_indicator()

    def test_sound_settings_item_open_correct_page(self):
        """
        1. Drag down the sound indicator
        2. Tap on 'Sound settings...'
        3. Ensure sound settings open
        """
        self.indicator_page.open_sound_settings()
        settings_app = system_settings.get_system_settings_proxy()
        system_settings.ensure_settings_page_visible(settings_app,
                                                     SOUND_SETTINGS_PAGE)


class SoundIndicatorWithMprisControlsTestCase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.restart_unity()
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        self.useFixture(music_fixture.MusicAppWelcomeWizard(show_wizard=True))
        shutil.copy2(test_data.SRC_AUDIO_FILE_2, test_data.LOCAL_AUDIO_FILE_2)
        shutil.copy2(test_data.SRC_AUDIO_FILE_4, test_data.LOCAL_AUDIO_FILE_4)
        self.music_app = music.launch_music_app()
        self.music_app.ensure_walkthrough_page_not_visible()

    def test_play_pause_track_from_indicator(self):
        """
        1. Start music app and play a track
        2. Open sound indicator and press pause button
        3. Ensure music is paused in App
        4. Click play button in the app and ensure track continues playing
        """
        songs_page = self.music_app.get_songs_page()
        now_playing_page = songs_page.play_track(0)
        self.assertThat(now_playing_page.player.isPlaying,
                        Eventually(Equals(True)))

        indicator, sound_indicator = indicators.open_sound_indicator(
            return_base=True)
        sound_indicator.playback_controls.pause_track()
        indicator.close()

        self.assertThat(now_playing_page.player.isPlaying,
                        Eventually(Equals(False)))
        now_playing_page.click_play_button()
        self.assertThat(now_playing_page.player.isPlaying,
                        Eventually(Equals(True)))

    def test_kill_music_player_mpris_controls_disable(self):
        """
        1. Start music app and play a track
        2. Close music app from right edge switcher
        3. Open sound indicator and ensure all MPRIS controls are disabled
        """
        songs_page = self.music_app.get_songs_page()
        now_playing_page = songs_page.play_track(0)
        self.assertThat(now_playing_page.player.isPlaying,
                        Eventually(Equals(True)))

        music.close_music_app()

        indicator, sound_indicator = indicators.open_sound_indicator(
            return_base=True)
        self.assertFalse(
            sound_indicator.playback_controls.is_play_button_enabled())
        self.assertFalse(
            sound_indicator.playback_controls.is_next_button_enabled())
        self.assertFalse(
            sound_indicator.playback_controls.is_previous_button_enabled())

    def test_music_controls_next_previous(self):
        """
        prerequisite: ensure at least two tracks are in the library
        1. Open music app, play a track
        2. Open sound indicator and click "next"
        3. Check in music app if the next song is really playing
        4. Open sound indicator and click "back" twice
        5. Ensure the original track is being played.
        """
        songs_page = self.music_app.get_songs_page()
        now_playing_page = songs_page.play_track(0)
        player = now_playing_page.player
        self.assertThat(player.isPlaying, Eventually(Equals(True)))

        orig_title = player.title
        orig_artist = player.artist

        indicator, sound_indicator = indicators.open_sound_indicator(
            return_base=True)
        sound_indicator.playback_controls.skip_next()
        indicator.close()

        self.assertNotEqual(orig_title, player.title)
        self.assertNotEqual(orig_artist, player.artist)

        indicator, sound_indicator = indicators.open_sound_indicator(
            return_base=True)
        sound_indicator.playback_controls.skip_previous()
        sound_indicator.playback_controls.skip_previous()
        indicator.close()

        self.assertEqual(orig_title, player.title)
        self.assertEqual(orig_artist, player.artist)
