# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import traceback
from testtools import skipUnless

from ubuntu_system_tests.helpers import addressbook
from ubuntu_system_tests.helpers import cache
from ubuntu_system_tests.helpers import calculator
from ubuntu_system_tests.helpers import camera
from ubuntu_system_tests.helpers.camera.fixture_setup import (
    SetCameraAccessRequests
)
from ubuntu_system_tests.helpers import clock
from ubuntu_system_tests.helpers.calendar.fixture_setup import (
    BackupRestoreCalendarFixture)
from ubuntu_system_tests.helpers.clock.fixture_setup import (
    SetClockAccessRequests)
from ubuntu_system_tests.helpers import dialer_app
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers.location import (
    fixture_setup as location_fixture)
from ubuntu_system_tests.helpers.location.coordinates import TIMES_SQUARE
from ubuntu_system_tests.helpers.location.fixture_setup import (
    SetLocationAccessRequests)
from ubuntu_system_tests.helpers import music
from ubuntu_system_tests.helpers import messaging
from ubuntu_system_tests.helpers import system_settings
from ubuntu_system_tests.helpers import timer
from ubuntu_system_tests.helpers.webapp import ebay, here
from ubuntu_system_tests.helpers import webbrowser
from ubuntu_system_tests.helpers.webbrowser.fixture_setup import (
    WebbrowserTestEnvironment
)
from ubuntu_system_tests.tests.perf import (
    PerfTestCase, log_time, load_config_values)

logger = logging.getLogger(__name__)

CLASS = 'app_startup'
HOT = 'hot'
COLD = 'cold'
COLD_RUNS_KEY = 'app_startup_cold_runs'
HOT_RUNS_KEY = 'app_startup_hot_runs'
NO_RUNS = 'No cold runs defined for app startup tests'


class BaseAppStartupTestCase(PerfTestCase):

    @skipUnless(load_config_values(COLD_RUNS_KEY, 0), NO_RUNS)
    def setUp(self):
        super().setUp()
        self.shell = self.restart_unity()
        self.addCleanup(cache.clear_qml_cache)

    def _log_started(self, app, type):
        data = {'name': app, 'time': timer.stop(timer.LAUNCH_ID), 'type': type,
                'class': CLASS}
        log_time(data)

    def _run(self, runs, app_name, run_type, launch_method,
             close_method, loaded_method=None, setup_method=None):
        runs_passed = 0
        for i in range(runs):
            try:
                if setup_method:
                    setup_method()
                launch_method()
                if loaded_method:
                    loaded_method()
                self._log_started(app_name, run_type)
                runs_passed += 1
            except:
                logger.error('Unexpected error: {}'.format(
                    traceback.format_exc()))
            finally:
                close_method()
        return runs_passed

    def _run_test(self, app_name, launch_method, close_method,
                  loaded_method=None):
        cold_runs = int(load_config_values(COLD_RUNS_KEY, 1))
        res_cold = self._run(cold_runs, app_name, COLD,
                             launch_method, close_method,
                             loaded_method=loaded_method,
                             setup_method=cache.clear_qml_cache)
        hot_runs = int(load_config_values(HOT_RUNS_KEY, 1))
        res_hot = self._run(hot_runs, app_name, HOT,
                            launch_method, close_method,
                            loaded_method=loaded_method)

        msg = 'Not all the runs worked properly for app {}. Cold runs ' \
              'passed {}/{}. Hot runs passed {}/{}'
        self.assertTrue(cold_runs == res_cold and hot_runs == res_hot,
                        msg.format(app_name, res_cold, cold_runs, res_hot,
                                   hot_runs))


class AppStartupTestCase(BaseAppStartupTestCase):

    def test_dialer_startup(self):
        self._run_test('dialer', dialer_app.launch_dialer_app,
                       dialer_app.close_dialer_app,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_calculator_startup(self):
        self._run_test('calculator', calculator.launch_calculator_app,
                       calculator.close_calculator_app,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_address_book_startup(self):
        self._run_test('address_book', addressbook.launch_address_book,
                       addressbook.close_address_book,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_messaging_startup(self):
        self._run_test('messaging', messaging.launch_messaging_app,
                       messaging.close_messaging_app,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_ebay_startup(self):
        self._run_test('ebay', ebay.launch_ebay_webapp,
                       ebay.close_ebay_webapp,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_system_settings_startup(self):
        self._run_test('system_settings',
                       system_settings.launch_system_settings,
                       system_settings.close_system_settings,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_gallery_startup(self):
        self._run_test('gallery', gallery.launch_gallery_app_from_launcher,
                       gallery.close_gallery_app,
                       loaded_method=self.shell.ensure_app_loaded)

    def test_music_startup(self):
        self._run_test('music', music.launch_music_app,
                       music.close_music_app,
                       loaded_method=self.shell.ensure_app_loaded)


class CameraAppStartupTestCase(BaseAppStartupTestCase):

    def setUp(self):
        self.useFixture(SetCameraAccessRequests())
        super().setUp()

    def _launch_camera_app(self):
        camera_app = camera.launch_camera_app()
        camera_app.ensure_camera_ready()

    def test_camera_startup(self):
        self._run_test('camera', self._launch_camera_app,
                       camera.close_camera_app)


class WebBrowserAppStartupTestCase(BaseAppStartupTestCase):

    def setUp(self):
        self.useFixture(WebbrowserTestEnvironment())
        super().setUp()

    def test_webbrowser_startup(self):
        self._run_test('webbrowser', webbrowser.launch_webbrowser_app,
                       webbrowser.close_webbrowser_app,
                       loaded_method=self.shell.ensure_app_loaded)


class ClockAppStartupTestCase(BaseAppStartupTestCase):

    def setUp(self):
        self.useFixture(location_fixture.DummyLocation(*TIMES_SQUARE))
        self.useFixture(SetClockAccessRequests())
        self.useFixture(BackupRestoreCalendarFixture())
        super().setUp()

    def test_clock_startup(self):
        self._run_test('clock', clock.launch_clock_app,
                       clock.close_clock_app,
                       loaded_method=self.shell.ensure_app_loaded)


class HereAppStartupTestCase(BaseAppStartupTestCase):

    def setUp(self):
        self.useFixture(SetLocationAccessRequests('com.nokia.heremaps_here'))
        super().setUp()

    def test_here_startup(self):
        self._run_test('here', here.launch_here_webapp,
                       here.close_here_webapp,
                       loaded_method=self.shell.ensure_app_loaded)
