# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR .  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot import exceptions
import logging
import os
from retrying import retry
from testtools import skipUnless

from ubuntu_system_tests.helpers.autopilot import object
from ubuntu_system_tests.helpers import cache
from ubuntu_system_tests.helpers import context
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers.scalability import PicturesGenerator
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers import timer
from ubuntu_system_tests.helpers.scopes.fixture_setup import ScopesHints
from ubuntu_system_tests.helpers import wait_until

from ubuntu_system_tests.tests import data as test_data
from ubuntu_system_tests.tests.perf.scalability import (
    BaseScalabilityTestCase, get_scenarios, TYPE_HOME, TYPE_SDCARD)

logger = logging.getLogger(__name__)

DATA_PROCESSED = 'pictures_processed'
PICS_ID = 'pictures'

MY_PHOTOS = 'my_pictures'
CAROUSEL = 'pictures_carousel'
GALLERY = 'pictures_gallery'
SD_REASON = 'The SD card is not inserted in the device'
PHOTO_REASON = 'The photos scope is not present in the device'
SCENARIOS_ID = 'pictures_scalability_runs'
NO_SCENARIOS = 'There are not scenarios defined for scalability pictures tests'


class PicturesScalabilityTestCase(BaseScalabilityTestCase):

    scenarios = get_scenarios(SCENARIOS_ID)
    update_metadata = False

    @skipUnless(get_scenarios(SCENARIOS_ID), NO_SCENARIOS)
    def setUp(self):
        self.useFixture(ScopesHints())
        super().setUp()
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        self.src_file = test_data.SRC_IMAGE_FILE_3
        cache.clear_all_cache()

    def _go_to_photo_preview(self, scope, pic):
        """ Go to the preview screen """
        object.wait_until_is_not_moving(pic)
        scope.select_picture_from_carousel(pic)
        return scope.get_current_photo_preview()

    @retry(stop_max_attempt_number=3,
           retry_on_exception=lambda exception:
           isinstance(exception, exceptions.StateNotFoundError))
    def _click_open_button(self, preview):
        preview.click_open_button()
        return wait_until(gallery.is_gallery_app_running, timeout=5)

    def _run_test(self, test_method):
        self.data_gen = PicturesGenerator(self.src_file, self.dst_dir,
                                          self.load,
                                          update_metadata=self.update_metadata)
        self._run(test_method, DATA_PROCESSED)

    # Carousel #

    def _all_carousel_pics_displayed(self, scope):
        pics = scope.get_pictures_from_carousel(self.dst_dir)
        return len(pics) >= 6

    def _select_picture_from_carousel_by_index(self, scope, index):
        # Consider that the picture 1 is already selected
        if wait_until(self._all_carousel_pics_displayed, scope,
                      timeout=context.shared.retry_time / 1000):
            pic = scope.get_pictures_from_carousel(self.dst_dir)[index]
            scope.select_picture_from_carousel(pic)
            return pic
        else:
            raise RuntimeError('Not all pics displayed in carousel')

    def _test_photo_scope(self):
        # Go to photos scope
        timer.start(PICS_ID)
        scope = scopes.go_to_photos_scope()

        # Check the local pic is in the carousel
        pic = self._select_picture_from_carousel_by_index(scope, 2)

        preview = self._go_to_photo_preview(scope, pic)
        self._click_open_button(preview)

        # The gallery opens
        photo_viewer = gallery.get_gallery_photo_viewer()

        self.assertIn(os.path.basename(pic.source).split('?')[0],
                      photo_viewer.get_photo_component().source)
        self._log_completed(CAROUSEL, PICS_ID)

        gallery.close_gallery_app()

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES), SD_REASON)
    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), PHOTO_REASON)
    def test_photo_scope_sdcard(self):
        self.type = TYPE_SDCARD
        self.dst_dir = fs.get_media_folder_dir(fs.DIR_PICTURES)
        self._run_test(self._test_photo_scope)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), PHOTO_REASON)
    def test_photo_scope_home(self):
        self.type = TYPE_HOME
        self.dst_dir = fs.DIR_HOME_PICTURES
        self._run_test(self._test_photo_scope)

    # My photos #

    def _test_my_photos(self):
        # Go to photos scope
        timer.start(PICS_ID)
        scope = scopes.go_to_photos_scope()
        my_photos = scope.go_to_my_photos()

        cat = my_photos.get_first_category()
        pics = cat.get_pictures_list()
        self.assertLess(0, len(pics))
        self._log_completed(MY_PHOTOS, PICS_ID)

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES), SD_REASON)
    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), PHOTO_REASON)
    def test_my_photos_sdcard(self):
        self.type = TYPE_SDCARD
        self.dst_dir = fs.get_media_folder_dir(fs.DIR_PICTURES)
        self._run_test(self._test_my_photos)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), PHOTO_REASON)
    def test_my_photos_home(self):
        self.type = TYPE_HOME
        self.dst_dir = fs.DIR_HOME_PICTURES
        self._run_test(self._test_my_photos)

    # Gallery #

    def _test_pictures_on_gallery(self):
        """ Verify a picture stored in the device can be viewed in the events
        view
        """
        swipes = 10

        timer.start(PICS_ID)
        gallery_app = gallery.launch_gallery_app_from_launcher()

        self.shell.ensure_app_loaded()
        pics = gallery_app.get_images_in_event_view(self.dst_dir)

        for i in range(swipes):
            gallery_app.swipe_to_show_more_on_the_right()

        self.assertLess(0, len(pics))
        self._log_completed(GALLERY, PICS_ID)

        gallery.close_gallery_app()

    @skipUnless(fs.is_media_folder_dir(fs.DIR_PICTURES), SD_REASON)
    def test_view_gallery_pictures_on_sdcard(self):
        self.type = TYPE_SDCARD
        self.dst_dir = fs.get_media_folder_dir(fs.DIR_PICTURES)
        self._run_test(self._test_pictures_on_gallery)

    def test_view_gallery_pictures_on_home(self):
        self.type = TYPE_HOME
        self.dst_dir = fs.DIR_HOME_PICTURES
        self._run_test(self._test_pictures_on_gallery)
