# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import shutil

from testtools import skipUnless

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers.location import (
    fixture_setup as location_fixture)
from ubuntu_system_tests.helpers.location.coordinates import TIMES_SQUARE
from ubuntu_system_tests.helpers.platform import (
    get_device_name, get_channel_name)
from ubuntu_system_tests.helpers.scopes.data import (
    is_image_expected_in_sections, get_section_names_from_list,
    get_section_header_names_from_list)
from ubuntu_system_tests.helpers.scopes.fixture_setup import ScopesHints

from ubuntu_system_tests.tests import data as test_data
from ubuntu_system_tests.tests.scopes import BaseScopeTestCase, SKIP_REASON

KEY_COMPARE_METHOD = 'compare_method'

FAIL_MSG_HEADERS = ('The number of headers in scope does not match expected '
                    'for scope id: {s} under condition: {c}.\n'
                    'Expected headers:\n{e}\nObserved headers:\n{o}')

FAIL_MSG_LABELS = ('Label: \'{l}\' not found for scope id: \'{s}\' under '
                   'condition: \'{c}\'.\n'
                   'Expected labels:\n{e}\n'
                   'Observed labels:\n{o}')

FAIL_MSG_ORDER = ('Label: \'{l}\' not in expected position for scope id: '
                  '\'{s}\' under condition: \'{c}\'.\n'
                  'Expected order:\n{e}\n'
                  'Observed order:\n{o}')

logger = logging.getLogger(__name__)


class DefaultScopeTest(BaseScopeTestCase):
    """Base class for testing default scope content."""

    def setUp(self):
        super().setUp()
        logger.info('Running test using channel: {c} and device: {d}'.format(
            c=get_channel_name(), d=get_device_name()))


class ScopeHeadingTest(DefaultScopeTest):
    """Tests for the content headings of default scopes."""

    def setUp(self):
        self.useFixture(location_fixture.DummyLocation(*TIMES_SQUARE))
        super().setUp()

    def _apply_condition_local_video(self):
        """Method to apply 'local_video' condition."""
        shutil.copy2(test_data.SRC_VIDEO_FILE_1, test_data.LOCAL_VIDEO_FILE_1)

    def _apply_condition_no_local_video(self):
        """Method to apply 'no_local_video' condition."""
        fs.clean_dir(fs.DIR_HOME_VIDEOS)

    def _apply_condition_local_photo(self):
        """Method to apply 'local_photo' condition."""
        shutil.copy2(test_data.SRC_IMAGE_FILE_1, test_data.LOCAL_IMAGE_FILE)

    def _apply_condition_no_local_photo(self):
        """Method to apply 'no_local_photo' condition."""
        fs.clean_dir(fs.DIR_HOME_PICTURES)

    def _apply_condition_local_music(self):
        """Method to apply 'local_music' condition."""
        shutil.copy2(test_data.SRC_AUDIO_FILE_1, test_data.LOCAL_AUDIO_FILE_1)

    def _apply_condition_no_local_music(self):
        """Method to apply 'no_local_music' condition."""
        fs.clean_dir(fs.DIR_HOME_MUSIC)

    def _apply_condition_none(self):
        """Method to apply 'none' condition."""
        pass

    def _apply_condition_scope_setup(self, restore=True):
        """Method to apply 'scope_setup' condition."""
        self.useFixture(ScopesHints(hidden=True, restore=restore))

    def _apply_condition_scope_unsetup(self, restore=True):
        """Method to apply 'scope_unsetup' condition."""
        self.useFixture(ScopesHints(hidden=False, restore=restore))

    def _apply_condition(self, condition):
        """Call the method which applies the specified condition.

        :param condition: The name of the condition method to apply.

        """
        getattr(self, '_apply_condition_{c}'.format(c=condition))()
        self.dash.get_current_scope().wait_for_processing_to_complete()

    def _find_matching_items_in_list(self, list, required_item):
        """Return a list of indexes for matching items in the list."""
        return [i for i, x in enumerate(list) if x == required_item]

    def _validate_label_in_list(self, expected_label, observed_labels,
                                scope_id, condition, expected_labels):
        """
        Validate that expected_label appears in observed_labels list.

        :return: list of index values where expected_label matches label
                 in observed_labels list.

        """
        indexs = self._find_matching_items_in_list(observed_labels,
                                                   expected_label)
        self.assertGreater(len(indexs), 0,
                           FAIL_MSG_LABELS.format(l=expected_label,
                                                  s=scope_id,
                                                  c=condition,
                                                  e=expected_labels,
                                                  o=observed_labels))
        return indexs

    def _validate_label_in_list_with_custom_validation(self, validation_method,
                                                       expected_label,
                                                       observed_labels,
                                                       scope_id, condition,
                                                       expected_labels):
        """
        Validate that a label in observed_labels list can be matched using
        the specified validation method.
        :param validation_method: startswith, endswith or contains
        :return: list of indexes of matching label in observed_labels list

        """
        indexs = []
        index = 0
        for observed_label in observed_labels:
            validator = getattr(self, '_validate_string_{v}'.format(
                v=validation_method))
            if validator(observed_label, expected_label):
                indexs.append(index)
            index += 1
        self.assertTrue(len(indexs) > 0,
                        FAIL_MSG_LABELS.format(l=expected_label,
                                               s=scope_id,
                                               c=condition,
                                               e=expected_labels,
                                               o=observed_labels))
        return indexs

    def _validate_string_startswith(self, string, expected):
        """Custom validation method to check if string starts with expected."""
        return string.startswith(expected)

    def _validate_string_endswith(self, string, expected):
        """Custom validation method to check if string ends with expected."""
        return string.endswith(expected)

    def _validate_string_contains(self, string, expected):
        """Custom validation method to check if string contains expected."""
        return string.find(expected) >= 0

    def _validate_labels(self, observed_labels,
                         expected_sections, scope_id, condition, check_order):
        """
        Assert that the expected labels appear in the observed labels
        in the order specified.

        The number of labels observed could be greater than the expected
        labels because of the content being displayed. So it is
        not possible to perform an exact match between observed and expected
        labels.

        """
        previous_index = -1
        expected_labels = get_section_names_from_list(
            expected_sections)
        for expected_section, expected_label in zip(expected_sections,
                                                    expected_labels):
            if KEY_COMPARE_METHOD in expected_section:
                index_list = \
                    self._validate_label_in_list_with_custom_validation(
                        expected_section[KEY_COMPARE_METHOD],
                        expected_label, observed_labels, scope_id, condition,
                        expected_labels)
            else:
                index_list = self._validate_label_in_list(
                    expected_label, observed_labels, scope_id, condition,
                    expected_labels)
            # To keep the required order, expect that at least 1 item in
            # index_list will be greater than previous index.
            # Find this item from the list of matching indexs.
            current_index = self._get_next_index(index_list, previous_index)
            if check_order:
                self.assertTrue(current_index > previous_index,
                                FAIL_MSG_ORDER.format(l=expected_label,
                                                      s=scope_id,
                                                      c=condition,
                                                      e=expected_labels,
                                                      o=observed_labels))
            previous_index = current_index

    def _get_next_index(self, index_list, previous_index):
        """
        Choose the next index from index_list based on the value of
        previous_index.

        In order to keep the required order, the next index should be greater
        than the previous index.
        :param index_list: A list of index items
        :param previous_index: The value of the previous index used
        :return: The index value selected from index_list, or -1 if not found

        """
        for index in index_list:
            if index > previous_index:
                return index
        return -1

    def _validate_headers_if_required(self, expected_scope, expected_sections,
                                      observed_headers, condition):
        """
        Check the header count matches expected, if it is required for
        this scope.

        """
        if expected_scope.check_header_count:
            # check that there are the expected number of header items
            expected_headers = get_section_header_names_from_list(
                expected_sections)
            self.assertEqual(
                len(expected_headers), len(observed_headers),
                FAIL_MSG_HEADERS.format(s=expected_scope.id,
                                        c=condition,
                                        e=expected_headers,
                                        o=observed_headers))

    def _validate_scope_data(self, scope_id, expected_scope, expected_sections,
                             condition):
        """
        Extract displayed data from the scope to validate its content.

        :param scope_id: The id of the scope to check.
        :param expected_scope: Expected data for the scope.
        :param expected_sections: The sections expected to be displayed.
        :param condition: The name of the condition being applied to the scope.

        """
        logger.info('Testing scope: {s}, under condition: {c}.'.format(
            s=expected_scope.title, c=condition))
        scope = scopes.go_to_scope(scope_id)
        scope.wait_for_processing_to_complete()
        # check title is as expected
        self.assertEqual(expected_scope.title, scope.get_title())
        # get lists of observed and expected labels
        observed_headers, observed_labels = scope.get_all_content(
            is_image_expected_in_sections(expected_sections))
        # wait until scope is ready before performing validation, this will
        # ensure a valid failure screenshot is captured if test fails.
        scope.wait_for_processing_to_complete()
        # check that labels appear in order expected
        self._validate_labels(observed_labels,
                              expected_sections,
                              expected_scope.title,
                              condition,
                              expected_scope.check_order)
        # some scopes disable the header count check, so only check
        # the number of headers if requried
        self._validate_headers_if_required(
            expected_scope, expected_sections, observed_headers, condition)

    def _test_scope(self, scope_id):
        """
        Verify all the headers and titles are correct for the specified
        scope under required conditions.
        :param scope_id: The id of the scope to test.

        """
        # get all the expected data for required scope
        expected_scope = scopes.get_scope_data(scope_id)

        # Test scope in un-setup state if required
        if expected_scope.unsetup_sections:
            self._apply_condition_scope_unsetup()
            self._validate_scope_data(
                scope_id, expected_scope, expected_scope.unsetup_sections,
                'unsetup')
            # No need to restore previous state, as this is being done already
            self._apply_condition_scope_setup(restore=False)

        # Test scope under each condition specified in expected data
        for condition in expected_scope.sections:
            self._apply_condition(condition)
            self._validate_scope_data(
                scope_id, expected_scope, expected_scope.sections[condition],
                condition)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_TODAY), SKIP_REASON)
    def test_today_scope(self):
        """ Verify that for the today scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_TODAY)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_NEARBY), SKIP_REASON)
    def test_nearby_scope(self):
        """ Verify that for the nearby scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_NEARBY)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_APPS), SKIP_REASON)
    def test_apps_scope(self):
        """ Verify that for the apps scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_APPS)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_NEWS), SKIP_REASON)
    def test_news_scope(self):
        """ Verify that for the news scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_NEWS)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_PHOTOS), SKIP_REASON)
    def test_photos_scope(self):
        """ Verify that for the photos scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_PHOTOS)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_MUSIC), SKIP_REASON)
    def test_music_scope(self):
        """ Verify that for the music scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_MUSIC)

    @skipUnless(scopes.is_scope_included(scopes.SCOPE_ID_VIDEO), SKIP_REASON)
    def test_video_scope(self):
        """ Verify that for the video scope the titles the subsections are
        correct for the predefined conditions.
        """
        self._test_scope(scopes.SCOPE_ID_VIDEO)


class ScopeSetupTest(DefaultScopeTest):
    """Tests for setting up a scope."""

    def setUp(self):
        self.useFixture(location_fixture.DummyLocation(*TIMES_SQUARE))
        self.useFixture(ScopesHints(hidden=False))
        super().setUp()

    def _test_scope_setup(self, scope_id, scope_setup_validator):
        """Validate that pressing skip setup button changes scope state.
        :param scope_id: Scope id to test.
        :param scope_setup_validator: Method that returns setup state of the
                                      tested scope.
        """
        scope = scopes.go_to_scope(scope_id)
        self.assertFalse(scope_setup_validator())
        self.assertTrue(scope.has_skip_button())
        scope.skip_setup()
        scope.wait_for_processing_to_complete()
        self.assertFalse(scope.has_skip_button())
        self.assertTrue(scope_setup_validator())

    def test_setup_today_scope(self):
        """Press setup button on the Today scope and check state updates."""
        self._test_scope_setup(
            scopes.SCOPE_ID_TODAY, scopes.is_today_scope_setup)

    def test_setup_photos_scope(self):
        """Press setup button on the Photos scope and check state updates."""
        self._test_scope_setup(
            scopes.SCOPE_ID_PHOTOS, scopes.is_photos_scope_setup)


class ScopeNavigationTest(BaseScopeTestCase):
    """Tests for navigation of the default scopes."""

    def test_scopes_navigation(self):
        """ Verify the navigation through the scopes and their order
        navigating from the left to the right and then from the right to the
        left. The total number of scopes is also validated.
        """
        scope_counter = 0
        expected_scopes = scopes.get_scope_list()
        expected_scope_count = len(expected_scopes)

        # repeat for each scope from left to right
        for expected_scope in expected_scopes:
            scope = self.dash.get_current_scope()
            self.assertEqual(expected_scope.title, scope.get_title())

            scope_counter += 1
            if expected_scope_count > scope_counter:
                self.dash.scroll_to_right_scope()

        self.assertEqual(expected_scope_count, self.dash.get_scope_count(),
                         'The observed scope count does not match expected.')

        # repeat for each scope from right to left avoiding the last one
        for expected_scope in reversed(expected_scopes[:-1]):
            self.dash.scroll_to_left_scope()

            scope = self.dash.get_current_scope()
            self.assertEqual(expected_scope.title, scope.get_title())
