# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers import platform
from ubuntu_system_tests.helpers import system_settings

from ubuntu_system_tests.tests import base


class AboutThisPhoneTestCase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.shell = self.restart_unity().main_window

        ss_app = system_settings.launch_system_settings()
        self.about_page = ss_app.go_to_about_page()

    def test_check_device_name(self):
        """ Check the device name is correct """
        manufacturer = platform.get_device_manufacturer()
        model = platform.get_device_model()

        self.assertEqual('{} {}'.format(manufacturer, model),
                         self.about_page.get_device_name())

    def test_check_serial_number(self):
        """ Check the serial number displayed is correct """
        serial = platform.get_serial_number()
        self.assertEqual(serial, self.about_page.get_serial())

    def test_check_imei(self):
        """ Check the imei displayed is correct """
        imei = self.about_page.get_imei()
        self.assertIn(imei, ofono.get_imei_list())

    def test_check_wifi_address(self):
        """ Check the wifi address displayed is correct """
        address = platform.get_mac_address()
        self.assertEqual(address.upper(), self.about_page.get_wifi_address())

    def test_check_bluetooth_address(self):
        """ Check the bluetooth address displayed is correct """
        address = platform.get_bluetooth_address()
        self.assertEqual(address.upper(),
                         self.about_page.get_bluetooth_address())

    def _storage_to_kb(self, storage_line):
        to_kb = 1
        space, unit = storage_line.split()[0:2]
        if 'MB' == unit:
            to_kb *= 1024
        if 'GB' == unit:
            to_kb *= 1024 * 1024
        return float(space) * to_kb

    def _bytes_to_kb(self, bytes):
        return bytes / 1024

    def _check_storage_in_range(self, expected, obtained):
        """ Check if the obtained value is close to the expected one
        because the displayed sizes are not exactly the same to the one get
        from the os
        """
        threshold = 0.1 * obtained
        self.assertGreater(expected, obtained - threshold)
        self.assertLess(expected, obtained + threshold)

    def _check_storage_dirs(self, storage_page):
        self._check_storage_in_range(
            fs.get_disk_usage(fs.DIR_HOME_PICTURES),
            int(storage_page.get_pictures_storage()))
        self._check_storage_in_range(
            fs.get_disk_usage(fs.DIR_HOME_VIDEOS),
            int(storage_page.get_videos_storage()))
        self._check_storage_in_range(
            fs.get_disk_usage(fs.DIR_HOME_MUSIC),
            int(storage_page.get_audio_storage()))

    def _check_gallery_app_in_storage(self, storage_page):
        install_dir = os.path.join(fs.DIR_PREINSTALLED_CLICK_APPS,
                                   gallery.APP_CLICK)
        gallery_app_size = self._bytes_to_kb(
            fs.get_disk_usage(install_dir))
        self._check_storage_in_range(gallery_app_size, self._storage_to_kb(
            storage_page.get_gallery_app_size_in_disk()))

        icon = storage_page.get_gallery_app_icon()
        self.assertTrue(icon.endswith(gallery.APP_ICON_NAME))
        self.assertIn(install_dir, icon)
        self.assertTrue(os.path.exists(fs.remove_file_path(icon)))

    def test_check_storage(self):
        """ Check the storage shows the free storage in the device """
        user_data_free_space = fs.get_user_data_free_space()
        total_free_space = fs.get_total_free_space()
        about_free_scpace = self._storage_to_kb(self.about_page.get_storage())

        self.assertGreaterEqual(about_free_scpace, user_data_free_space)
        self.assertLessEqual(about_free_scpace, total_free_space)

        storage_page = self.about_page.go_to_storage()
        self._check_storage_dirs(storage_page)
        self._check_gallery_app_in_storage(storage_page)

    def test_check_licenses(self):
        """ Check the AppArmor license is included in the Software licenses """
        licenses_page = self.about_page.go_to_software_licenses()
        license = licenses_page.get_license('apparmor')
        self.assertIsNotNone(license)

    def test_check_software(self):
        """ Check the OS and last update information is correct """
        installed_os = self.about_page.get_os_information()

        build_number = platform.get_build_number()
        os_version = platform.get_os_and_version()

        self.assertIn(build_number, installed_os)
        self.assertIn(os_version, installed_os)

        installed = platform.get_os_last_update_date()
        updated_date = self.about_page.get_last_updated_date()
        self.assertEqual(installed.strftime("%-m/%-d/%y"), updated_date)

        version_page = self.about_page.go_to_os()
        self.assertIn(build_number, version_page.get_build_number())
        self.assertIn(os_version, version_page.get_build_desc())

    def test_check_developer_mode(self):
        """ Check the developer mode can be set """
        devmode_page = self.about_page.go_to_developer_mode()
        self.assertTrue(devmode_page.is_devmode_on())
