# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ubuntu_system_tests.helpers import display
from ubuntu_system_tests.helpers import indicators
from ubuntu_system_tests.helpers import system_settings
from ubuntu_system_tests.helpers.power.fixture_setup import (
    BrightnessEnvironmentFixture, EnablePowerdFakeSensors)
from ubuntu_system_tests.helpers import sensors
from ubuntu_system_tests.tests import base


class AutoBrightnessTestCase(base.BaseUbuntuSystemTestCase):

    def setUp(self):
        super().setUp()
        self.useFixture(BrightnessEnvironmentFixture())
        self.useFixture(EnablePowerdFakeSensors())
        self.restart_unity()

    def _test_automatic_screen_brightness(self, settings_page):
        """
        Test the automatic screen brightness setting on the specified
        settings page.
        1. Enable automatic screen brightness setting
        2. Set light conditions dark and ensure screen dims
        3. Set light conditions bright and ensure screen brightens
        4. Disable automatic screen brighness setting
        5. Set light conditions dark and ensure brightness doesn't change

        :param settings_page: The page that contains auto-brightness settings.
        """
        self.assertFalse(settings_page.get_auto_adjust_status())
        brightness_default = display.get_stable_brightness_value()

        settings_page.toggle_auto_adjust()
        self.assertTrue(settings_page.get_auto_adjust_status())
        sensors.set_light_level_dark()
        brightness_auto_dark = display.get_stable_brightness_value()
        self.assertLess(brightness_auto_dark, brightness_default)

        sensors.set_light_level_bright()
        brightness_auto_bright = display.get_stable_brightness_value()
        self.assertGreater(brightness_auto_bright, brightness_auto_dark)

        settings_page.toggle_auto_adjust()
        self.assertFalse(settings_page.get_auto_adjust_status())
        brightness_manual_bright = display.get_stable_brightness_value()
        sensors.set_light_level_dark()
        brightness_manual_dark = display.get_stable_brightness_value()
        self.assertEqual(brightness_manual_bright, brightness_manual_dark)

    def test_battery_indicator_automatic_screen_brightness(self):
        """
        1. Drag down the battery indicator
        2. Enable automatic screen brightness setting
        3. Set light conditions dark and ensure screen dims
        4. Set light conditions bright and ensure screen brightens
        5. Disable automatic screen brighness setting
        6. Set light conditions dark and ensure brightness doesn't change
        """
        battery_idicator = indicators.open_battery_indicator()
        self._test_automatic_screen_brightness(battery_idicator)

    def test_system_settings_automatic_screen_brightness(self):
        """
        1. Open system settings and go to brightness page
        2. Enable automatic screen brightness setting
        3. Set light conditions dark and ensure screen dims
        4. Set light conditions bright and ensure screen brightens
        5. Disable automatic screen brighness setting
        6. Set light conditions dark and ensure brightness doesn't change
        """
        settings = system_settings.launch_system_settings()
        brightness_page = settings.go_to_brightness_page()
        self._test_automatic_screen_brightness(brightness_page)
