# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015, 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import shutil
import time
from datetime import datetime
from threading import Event

from autopilot.exceptions import StateNotFoundError
from autopilot.matchers import Eventually
from retrying import retry
from testtools import skipUnless
from testtools.matchers import Equals

from ubuntu_system_tests.helpers import addressbook
from ubuntu_system_tests.helpers import dialer_app as dialer
from ubuntu_system_tests.helpers import display
from ubuntu_system_tests.helpers import media
from ubuntu_system_tests.helpers import mediaplayer
from ubuntu_system_tests.helpers import network
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers import scopes
from ubuntu_system_tests.helpers import sensors
from ubuntu_system_tests.helpers import webbrowser
from ubuntu_system_tests.helpers.addressbook import(
    Contact,
    fixture_setup as address_book_fixture
)
from ubuntu_system_tests.helpers.data import load_test_metadata
from ubuntu_system_tests.helpers.dialer_app import fixture_setup as dialer_fixture_setup  # NOQA
from ubuntu_system_tests.helpers import indicators
from ubuntu_system_tests.helpers.indicators import(
    network as network_indicator
)
from ubuntu_system_tests.helpers.messaging.fixture_setup import (
    MessagingTestEnvironment)
from ubuntu_system_tests.helpers.music import launch_music_app
from ubuntu_system_tests.helpers.music.fixture_setup import (
    MusicAppWelcomeWizard)
from ubuntu_system_tests.helpers.ofono import fixture_setup as ofono_fixture_setup  # NOQA
from ubuntu_system_tests.helpers.power.fixture_setup import (
    EnablePowerdFakeSensors)
from ubuntu_system_tests.helpers.system_settings.fixture_setup import SilentMode  # NOQA
from ubuntu_system_tests.helpers.timedate import get_current_utc_time
from ubuntu_system_tests.helpers.vibration import start_vibration_observer
from ubuntu_system_tests.tests.base import wait_until
from ubuntu_system_tests.tests.base_telephony import BaseTelephonyTestCase
from ubuntu_system_tests.tests.data import (
    LOCAL_AUDIO_FILE_5, LOCAL_VIDEO_FILE_3, SRC_AUDIO_FILE_5, SRC_VIDEO_FILE_3)

CONTACT_NAME_1 = 'Fulano de Tal'

CALL_URL_SHORT = 'http://people.canonical.com/~platform-qa/call_short.xml'
CALL_URL_30_SEC = 'http://people.canonical.com/~platform-qa/call_30_sec.xml'
CALL_URL_INDEFINITE = 'http://people.canonical.com/~platform-qa/' \
                      'call_indefinite.xml'
CALL_OUTGOING_API = 'outbound-api'
CALL_OUTGOING_DIAL = 'outbound-dial'
CALL_INCOMING = 'inbound'
CALL_NO_ANSWER = 'no-answer'
CALL_COMPLETED = 'completed'
CALL_INPROGRESS = 'in-progress'
CALL_RINGING = 'ringing'
CALLING_FROM = 'Calling from '
MISSED_CALL = 'missed call'
CALL_RETRY_COUNT = 5

KEY_DEVICE_PASSWORD = 'device_password'

ERR_CALL_LOG_COUNT = ('Error: Expected 1 call to be present in call log '
                      'from: "{from_}" to: "{to}". {count} calls were '
                      'reported.')

REJECT_MSG_OPTIONS = 'Message & decline'
REJECT_MSG_BUSY = 'I\'m busy at the moment. I\'ll call later.'
REJECT_MSG_LATE = 'I\'m running late, on my way now.'
REJECT_MSG_CALL_BACK = 'Please call me back later.'
REJECT_MSGS = [REJECT_MSG_OPTIONS, REJECT_MSG_BUSY, REJECT_MSG_LATE,
               REJECT_MSG_CALL_BACK]

URL_UBUNTU = 'http://www.ubuntu.com/'

test_data = load_test_metadata()


class CallsTestCase(BaseTelephonyTestCase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        # enable fake proximity sensors prior to restarting unity8
        self.useFixture(EnablePowerdFakeSensors())
        super().setUp()
        self.useFixture(dialer_fixture_setup.DialerTestEnvironment())
        self.useFixture(SilentMode(False))
        self.contact_1 = Contact(name=CONTACT_NAME_1,
                                 number=self.get_telephony_service_number1())
        self.useFixture(
            address_book_fixture.AddTemporaryContacts([self.contact_1]))

    def _duration_to_datetime(self, duration):
        """
        Convert provided duration string into datetime object.

        :param duration: Duration string in MM:SS format
        :return: datetime object of specified duration

        """
        return datetime.strptime(duration, '%M:%S')

    def _is_greater_than(self, value, target):
        """Return true if value is greater than target."""
        return value > target

    @retry(stop_max_attempt_number=CALL_RETRY_COUNT)
    def _retry_connect_call(self, from_, to, dialer_app):
        """
        Re-dial phone number and validate call connected to telephony service.

        Terminate existing call and re-dial the previous phone number.
        :param from_: Calling phone number.
        :param to: Number to be called.
        :param dialer_app: Dialer app proxy object.

        """
        if dialer_app.hasCalls:
            # Terminate existing call
            dialer_app.live_call_page.click_hangup_button()
        dialer = dialer_app.dialer_page
        dialer.visible.wait_for(True)
        if not dialer.get_entered_phone_number():
            # press call button once to enter previously called number
            dialer.click_call_button()
        displayed_number = dialer.get_entered_phone_number(unformatted=True)
        self.assertEqual(displayed_number, to)
        # call again
        dialer.click_call_button()
        self.assertTrue(
            wait_until(self.telephony.is_call_active, from_, to),
            'Telephony service logs show call was unable to connect from '
            'device phone number: {} to telephony service number: '
            '{}.'.format(from_, to))

    def _validate_call_started(self, call_page, caller_id, second_count=2):
        """
        Validate that a call has started successfully. Check that the caller id
        is displayed, the hang-up button is enabled and that the call counter
        is counting.

        :param call_page: The LiveCall page object to validate
        :param caller_id: The expected name of the caller
        :param second_count: The number of timer second events to check. This
                             is the number of times the second counter should
                             be updated.

        """
        self.assertThat(
            call_page.caller, Eventually(Equals(caller_id)))

        call_page._get_hangup_button().enabled.wait_for(True)

        stop_watch = call_page.get_stop_watch()
        for count in range(0, second_count):
            start = self._duration_to_datetime(stop_watch.elapsed_time())
            wait_until(
                self._is_greater_than,
                timeout=3,
                period=1,
                value=self._duration_to_datetime(stop_watch.elapsed_time()),
                target=start)

    def _validate_call_ended(self, call_page, timeout=60):
        """
        Validate that a call ends successfully.

        :param call_page: The LiveCall page object to validate
        :param timeout: How long to wait for the call to end

        """
        try:
            call_page.visible.wait_for(False, timeout=timeout)
        except StateNotFoundError:
            # This just means the call page has already closed
            pass

    @retry(stop_max_delay=30000)
    def _validate_telephony_call(self, call_id, from_, to, direction,
                                 status=CALL_COMPLETED):
        """
        Validate the call logs on the telephony service.

        :param call_id: Id value for the call
        :param from_: Phone number used by telephony service to make call
        :param to: Phone number of device receiving the call
        :param direction: Direction of the call
        :param status: Status of the call

        """
        call = self.telephony.get_call(call_id)
        self.assertEqual(call['status'], status)
        self.assertEqual(call['direction'], direction)
        self.assertEqual(call['to'], to)
        self.assertEqual(call['from'], from_)

    def _validate_call_status(self, call_id, from_, to, direction, status):
        """
        Return True if call has status parameters as specified, False
        otherwise.

        :param call_id: Id value for the call
        :param from_: Phone number used by telephony service to make call
        :param to: Phone number of device receiving the call
        :param direction: Direction of the call
        :param status: Status of the call
        """
        try:
            self._validate_telephony_call(
                call_id, from_, to, direction, status)
            return True
        except AssertionError:
            return False

    def _call_contact_from_address_book(self, required_contact,
                                        wait_for_answer=True):
        """
        Helper to launch the address book app and call a contact.

        :param required_contact: Contact object holding name and number to call
        :param wait_for_answer: Whether to wait for the call to be answered by
                                remote end before returning.
        :return: A tuple containing the dialer-app live call page proxy object
                 and the ID of the call on the telephony service.

        """
        self.start_notification_observer()
        start_time = get_current_utc_time()
        address_book = addressbook.launch_address_book()
        contacts_page = address_book.get_contact_list_page()
        contact = contacts_page.open_contact_by_name(required_contact.name)
        contact.call(required_contact.number)
        dialer_app = dialer.get_dialer_app_proxy()
        dialer_app.dialer_page.click_call_button()
        self.addCleanup(ofono.end_call)
        if not wait_until(self.telephony.is_call_active,
                          from_=self.get_device_phone_number(),
                          to=required_contact.number):
            self._retry_connect_call(
                from_=self.get_device_phone_number(),
                to=required_contact.number,
                dialer_app=dialer_app)
        call_log = self.telephony.get_calls(
            from_=self.get_device_phone_number(),
            to=required_contact.number,
            time_from=start_time)
        self.assertEqual(1, len(call_log))
        call_sid = call_log[0]['sid']
        if wait_for_answer:
            self.assertTrue(wait_until(self.telephony.is_call_in_progress,
                                       from_=self.get_device_phone_number(),
                                       to=required_contact.number,
                                       timeout=30),
                            'Phone call was not answered by telephony service')
        return dialer_app, call_sid

    def _clear_notifications(self):
        """Clear any existing notification messages displayed in indicator."""
        self.shell.clear_notifications()
        self.assertFalse(self.shell.is_new_notification_displayed())

    def _get_incoming_call_notification(self, timeout=30):
        """Wait for incoming call notification to appear and return it."""
        notifications_list = self.shell.get_notifications_list()
        notification = notifications_list.get_incoming_call_notification(
            timeout)
        self.assertIsNotNone(
            notification,
            'Incoming call notification did not show or the call never arrived'
        )
        return notification

    def _validate_call_notification(self, call_notifier,
                                    contact, expected_sim='SIM 1'):
        """
        Validate that the call notifier displays correctly and return the
        caller's phone number as displayed in the notification.
        :param call_notifier: Call notifier object to validate
        :param contact: Expected contact details
        :return: Phone number as displayed by the notification

        """
        self.assertThat(call_notifier.get_caller_name(),
                        Eventually(Equals(contact.name)))
        self.assertEqual(call_notifier.get_reject_options(), REJECT_MSGS)
        caller_msg = call_notifier.get_caller_message()
        if ofono.number_of_sims() > 1:
            expected_sim_message = 'On [{sim}]'.format(
                sim=expected_sim
            )
            (sim_message, caller_number) = caller_msg.split('\n')
            self.assertEquals(sim_message, expected_sim_message)
            return caller_number
        else:
            self.assertTrue(caller_msg.startswith(CALLING_FROM))
            caller_number = caller_msg[len(CALLING_FROM):]
            return caller_number

    def _get_missed_call_notification(self, contact):
        """
        Validate that there is a missed call notification from the
        specified contact displayed in the messages indicator and return it.
        :param contact: The contact who made the missed call.
        :return: Message object for the missed call.

        """
        self.assertTrue(self.shell.is_new_notification_displayed())
        notifications = self.shell.open_notifications_indicator()
        notification = notifications.get_missed_call(caller=contact.name)
        self.assertIn(MISSED_CALL, notification.body)
        return notification

    def _create_observers(self):
        """Create observers used to watch for sound and vibrate events."""
        self.sound_played = Event()
        self.mediaplayer_observer = mediaplayer.start_media_player_observer(
            self.sound_played)
        self.addCleanup(self.mediaplayer_observer.stop)
        self.vibra_used = Event()
        self.vibra_observer = start_vibration_observer(self.vibra_used)
        self.addCleanup(self.vibra_observer.stop)

    def _reset_observers(self):
        """Reset the sound and vibration observer results."""
        self.sound_played.clear()
        self.vibra_used.clear()

    def _receive_missed_call_and_wait_for_hangup(self):
        """Receive call and wait for remote end to hangup without answering."""
        self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=5,
            call_url=CALL_URL_SHORT)
        self._get_incoming_call_notification().wait_until_destroyed()

    def _assert_proximity_near(self):
        """Set proximity sensor to near state and check display turns off."""
        sensors.set_proximity_near()
        self.assertTrue(wait_until(
            display.is_display_off, timeout=5),
            'Display did not turn off with near proximity during call.')

    def _assert_proximity_far(self):
        """Set proximity sensor to far state and check display turns on."""
        sensors.set_proximity_far()
        self.assertTrue(
            wait_until(display.is_display_on, timeout=5),
            'Display did not turn back on with far proximity during call.')

    def _assert_proximity_sensor(self):
        """Set proximity sensor to near and far states and check that the
        display turns off and on appropriately.
        """
        self._assert_proximity_near()
        self._assert_proximity_far()

    def _assert_proximity_sensor_inactive(self):
        """Set proximity sensor to near and check that the display does not
        turn off.
        """
        self.assertTrue(display.is_display_on())
        sensors.set_proximity_near()
        self.addCleanup(sensors.set_proximity_far)
        self.assertFalse(wait_until(
            display.is_display_off, timeout=5),
            'Display turned off with near proximity when not expected.')

    def test_receive_incoming_call_from_contact(self):
        """
        Test to recive an incoming call from a contact.
        1. Receive incoming call from a contact
        2. Swipe to answer the call
        3. Validate that the call information is correct
        4. Wait for call to be terminated from remote end

        """
        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=60,
            call_url=CALL_URL_30_SEC)

        call_notifier = self._get_incoming_call_notification()
        self._validate_call_notification(call_notifier, self.contact_1)

        # Let the phone ring briefly
        time.sleep(1)
        call_notifier.start_call()

        call_page = dialer.get_dialer_app_proxy().live_call_page
        self._validate_call_started(call_page, self.contact_1.name)
        self.assertTrue(display.is_display_on())
        self._assert_proximity_sensor()
        self._validate_call_ended(call_page)
        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            direction=CALL_OUTGOING_API)

    def test_return_missed_call_using_indicator(self):
        """
        Test to reply to a missed call from a contact.
        1. Receive incoming call from a contact.
        2. Do not answer call and wait for caller to hang up.
        3. Check that the messages notifier is updated.
        4. Swipe down messages notifier and press call icon to return call.
        5. Ensure that dialer app loads with the contact's name and number
           ready to be called.

        """
        self._clear_notifications()
        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=5,
            call_url=CALL_URL_SHORT)

        call_notifier = self._get_incoming_call_notification()
        caller_number = self._validate_call_notification(
            call_notifier, self.contact_1)
        call_notifier.wait_until_destroyed()

        self._get_missed_call_notification(self.contact_1).call()
        dialer_page = dialer.get_dialer_app_proxy().dialer_page
        self.assertThat(dialer_page.get_entered_phone_number(),
                        Eventually(Equals(caller_number)))

        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            direction=CALL_OUTGOING_API,
            status=CALL_NO_ANSWER)

    def test_return_missed_call_device_locked(self):
        """
        Test to reply to a missed call from a contact when device is locked.
        1. Receive incoming call from a contact.
        2. Do not answer call and wait for caller to hang up.
        3. Check that the messages notifier is updated.
        4. Swipe down messages notifier and press the contact avatar to
           display snap decision menu.
        5. Press 'Call back' button to return call.
        6. Unlock the greeter by entering passphrase.
        7. Check that the dialer-app is loaded showing the correct phone
           number already inserted.

        """
        self._clear_notifications()
        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=5,
            call_url=CALL_URL_SHORT)

        call_notifier = self._get_incoming_call_notification()
        caller_number = self._validate_call_notification(
            call_notifier, self.contact_1)
        call_notifier.wait_until_destroyed()

        self._get_missed_call_notification(self.contact_1).press_avatar()
        self.shell.get_missed_call_snap_decision().call_back()
        self.shell.get_passphrase_unlock_screen().unlock(
            self.config_stack.get(KEY_DEVICE_PASSWORD))

        dialer_page = dialer.get_dialer_app_proxy().dialer_page
        self.assertThat(dialer_page.get_entered_phone_number(),
                        Eventually(Equals(caller_number)))

        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            direction=CALL_OUTGOING_API,
            status=CALL_NO_ANSWER)

    def test_call_contact_using_address_book(self):
        """
        Test to call a contact from the address book application.
        1. Open address book app and open contact page
        2. Press call button on required phone number
        3. In dialer-app press call button
        4. After call has started hang-up the call by pressing hang up button
        5. Check logs from telephony service to ensure they are correct

        """
        start_time = get_current_utc_time()
        dialer_app, call_sid = self._call_contact_from_address_book(
            self.contact_1)
        call_page = dialer_app.live_call_page
        self._validate_call_started(call_page, self.contact_1.name)
        self.assertTrue(display.is_display_on())
        self._assert_proximity_sensor()

        call_page.click_hangup_button()
        self._validate_call_ended(call_page)
        end_time = get_current_utc_time()

        call_log = self.telephony.get_calls(
            from_=self.get_device_phone_number(),
            to=self.contact_1.number,
            time_from=start_time,
            time_to=end_time)

        self.assertEqual(len(call_log), 1, ERR_CALL_LOG_COUNT.format(
            from_=start_time, to=end_time, count=len(call_log)))

        self._validate_telephony_call(
            call_id=call_log[-1]['sid'],
            from_=self.get_device_phone_number(),
            to=self.contact_1.number,
            direction=CALL_INCOMING)

    def _validate_call_on_hold(self, sim=None):
        self.assertTrue(wait_until(ofono.is_call_on_hold, sim=sim))

    def _validate_call_not_on_hold(self, sim=None):
        self.assertTrue(
            wait_until(lambda: ofono.is_call_on_hold(sim=sim) is False))

    def test_call_on_hold(self):
        """
        Test to call a contact from the address book and put on hold.
        If the network doesn't support call hold feature then the test result
        will be reported as skipped.
        1. Open address book app and open contact page.
        2. Press call button on required phone number.
        3. In dialer-app press call button.
        4. After call has started press the hold button.
        5. Check that the call is put on hold.
        6. Press the hold button again and check call is active.
        7. Hang-up the call by pressing hang up button.
        8. Check logs from telephony service to ensure they are correct.

        """
        dialer_app, call_sid = self._call_contact_from_address_book(
            self.contact_1)
        call_page = dialer_app.live_call_page
        self._validate_call_started(call_page, self.contact_1.name)
        self._validate_call_not_on_hold()

        on_hold = call_page.toggle_hold()
        failure_dialog = dialer_app.get_call_hold_failure_dialog()
        if not on_hold:
            self._validate_call_not_on_hold()
            failure_dialog.close()
            self._end_call_from_live_screen(dialer_app)
            self.skipTest('Call holding not supported by network operator.')

        self._validate_call_on_hold()
        self.assertIsNone(failure_dialog)

        self.assertTrue(call_page.toggle_hold())
        self._validate_call_not_on_hold()

        self._end_call_from_live_screen(dialer_app)

    def test_incoming_call_silent_mode(self):
        """
        Test to receive an incoming call whilst silent mode is activated.
        1. Open sound indicator.
        2. Turn silent mode on and close indicator.
        3. Receive incoming phone call.
        4. Ensure no sound is played but the phone still vibrates.
        5. Open sound indicator.
        6. Turn off silent mode and close the indicator.
        7. Receive incoming phone call.
        8. Ensure ringtone is played and the phone vibrates.

        """
        self._create_observers()
        indicator, indicator_page = indicators.open_sound_indicator(
            return_base=True)

        self.assertFalse(indicator_page.get_silent_mode_switcher().checked)
        indicator_page.enable_silent_mode()
        self.assertTrue(indicator_page.get_silent_mode_switcher().checked)
        indicator.close()

        self._reset_observers()
        self._receive_missed_call_and_wait_for_hangup()
        self.assertFalse(self.sound_played.is_set())
        self.assertTrue(self.vibra_used.is_set())

        indicator_page = indicator.open()
        self.addCleanup(indicator.ensure_closed)
        indicator_page.disable_silent_mode()
        self.assertFalse(indicator_page.get_silent_mode_switcher().checked)
        indicator.close()

        self._reset_observers()
        self._receive_missed_call_and_wait_for_hangup()
        self.assertTrue(self.sound_played.is_set())
        self.assertTrue(self.vibra_used.is_set())

    def _end_call_from_live_screen(self, dialer):
        if dialer.live_call_page.active:
            dialer.live_call_page.hangup_call()

    def _receive_call(self, phone_number, expected_sim='SIM 1'):
        call = self.telephony.make_phonecall(
            to=phone_number,
            from_=self.contact_1.number,
            hangup_timeout=10,
            call_url=CALL_URL_INDEFINITE)
        self.addCleanup(self.telephony.hangup_call, call['sid'])

        call_notifier = self._get_incoming_call_notification()
        self._validate_call_notification(
            call_notifier, self.contact_1, expected_sim=expected_sim
        )
        # Let the phone ring briefly
        time.sleep(1)
        call_notifier.start_call()
        self.addCleanup(ofono.end_call)

        call_page = dialer.get_dialer_app_proxy().live_call_page
        self._validate_call_started(call_page, self.contact_1.name)

        return call

    def _launch_browser_and_browse_page(self, page_url):
        browser = webbrowser.launch_webbrowser_from_launcher()
        browser.go_to_url(page_url)
        browser.wait_until_page_loaded(page_url)

    def _validate_call_in_progress_and_terminate(self, call, phone_number):
        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=phone_number,
            direction=CALL_OUTGOING_API,
            status=CALL_INPROGRESS)

        # End call from the UI once we check everything.
        dialer_app = dialer.launch_dialer_app_from_launcher()
        self._end_call_from_live_screen(dialer_app)

        self._validate_telephony_call(
            call['sid'],
            from_=self.contact_1.number,
            to=phone_number,
            direction=CALL_OUTGOING_API)

    def _receive_call_browse_internet(self, phone_number,
                                      page_url=URL_UBUNTU,
                                      expected_sim='SIM 1'):
        """Receive a call on specified phone number and browse internet."""
        call = self._receive_call(phone_number, expected_sim=expected_sim)
        self._launch_browser_and_browse_page(page_url)
        self._validate_call_in_progress_and_terminate(call, phone_number)

    def test_receive_call_browse_internet(self):
        """
        1. Create an incoming call, swipe notification to start call
        2. Ensure dialer app starts with active call
        3. Drag launcher from left edge and start webbrowser app
        4. Open a website and ensure it opens
        5. Return to the dialer app and end the call
        """
        self._receive_call_browse_internet(self.get_device_phone_number())

    @skipUnless(ofono.number_of_sims() > 1, 'More than 1 SIM is required.')
    def test_receive_call_sim2_browse_internet(self):
        """
        1. Create an incoming call on sim 2, swipe notification to start call
        2. Ensure dialer app starts with active call
        3. Drag launcher from left edge and start webbrowser app
        4. Open a website and ensure it opens
        5. Return to the dialer app and end the call
        """
        self._receive_call_browse_internet(
            self.get_device_phone_number2(), expected_sim='SIM 2'
        )

    def _turn_off_wifi_enable_mobile_data(self):
        """Turn off wifi and enable data connection."""
        network_indicator.turn_off_wifi()
        self.addCleanup(self._turn_on_wifi_ensure_internet_connectivity, 30)
        if not network.is_mobile_data_enabled():
            network.turn_on_data_connection()
            self.addCleanup(network.turn_off_data_connection)
        self.assertTrue(network.ensure_internet_connectivity(timeout=30))

    def _turn_on_wifi_ensure_internet_connectivity(self, timeout):
        """Enable wifi and wait for internet availability."""
        network_indicator.turn_on_wifi()
        self.assertTrue(network.ensure_internet_connectivity(timeout=timeout))

    @skipUnless(ofono.is_sim_registered(0), 'No SIM in slot 1')
    def test_browse_internet_on_data_during_call(self):
        """
        Prerequisite: Disable wifi and enable data connection
        1. Create an incoming call, swipe notification to start call
        2. Ensure dialer app starts with active call
        3. Drag launcher from left edge and start webbrowser app
        4. Open a website and ensure it opens
        5. Return to the dialer app and end the call
        """
        self.useFixture(ofono_fixture_setup.ModemForData(0))
        phone_number = self.get_device_phone_number()
        call = self._receive_call(phone_number)
        # Disable wifi and enable data connection to browse a website,
        # we didn't disable wifi at the start of the test because, in rare
        # cases, due to low network signals in the lab, data connection may
        # not be stable enough to request a call.
        self._turn_off_wifi_enable_mobile_data()
        simple_page = 'http://people.canonical.com/~platform-qa/page1.html'
        self._launch_browser_and_browse_page(simple_page)
        self._validate_call_in_progress_and_terminate(call, phone_number)

    def test_reject_incoming_call(self):
        """
        Test to reject an incoming call.
        1. Call the device under test
        2. Phone rings and displays the caller dialogue
        3. Swipe to reject the call
        4. The call is rejected
        """
        call = self.telephony.make_phonecall(
            from_=self.contact_1.number,
            to=self.get_device_phone_number(),
            hangup_timeout=10,
            call_url=CALL_URL_SHORT)
        self.addCleanup(self.telephony.cancel_call, call['sid'])
        self.assertTrue(
            wait_until(
                self._validate_call_status,
                call['sid'],
                from_=self.contact_1.number,
                to=self.get_device_phone_number(),
                direction=CALL_OUTGOING_API,
                status=CALL_RINGING
            )
        )
        call_notifier = self._get_incoming_call_notification()
        self.assertTrue(ofono.is_call_ringing())
        self._validate_call_notification(call_notifier, self.contact_1)
        call_notifier.end_call()
        self.assertTrue(wait_until(lambda: not ofono.is_call_ringing()))
        self.assertFalse(ofono.is_call_active())

    def test_reject_incoming_call_with_message(self):
        """
        Reject an incoming call by sending message to caller.
        1. Receive an incoming call.
        2. Choose to reject the call by pressing a reject message option.
        3. Check that the call is ended and that the caller received the
           sent reject message.
        """
        self.useFixture(MessagingTestEnvironment())
        start_time = datetime.today()
        device_phone_number = self.get_device_phone_number()

        call = self.telephony.make_phonecall(
            to=device_phone_number,
            from_=self.contact_1.number,
            hangup_timeout=20,
            call_url=CALL_URL_SHORT)

        call_notifier = self._get_incoming_call_notification()
        self._validate_call_notification(call_notifier, self.contact_1)
        # start notification observer to wait for ack reply message and clear
        # any ussd balance notification messages.
        # expected_count is 3 because observer is called twice when the call
        # notifier closes, and once when the reply notification message closes
        self.start_notification_observer(expected_count=3)
        call_notifier.reject_call_with_message(REJECT_MSG_BUSY)
        # call should be cancelled immediately to prevent call-backs
        self.telephony.cancel_call(call['sid'])
        self.assertFalse(ofono.is_call_active())
        self.wait_for_all_notifications()

        msgs = self.telephony.get_sent_sms(
            self.contact_1.number,
            device_phone_number,
            start_time,
            REJECT_MSG_BUSY)
        self.assertEqual(len(msgs), 1)

    def test_outgoing_call_rejected(self):
        """
        Make an outgoing call which is rejected by the receiver.
        1. Make an outgoing call using address book.
        2. Set proximity sensor near to turn off display.
        3. From remote end, reject the call without answering.
        4. Ensure call is terminated successfully and display turns back on.
        """
        dialer_app, call_sid = self._call_contact_from_address_book(
            self.contact_1, wait_for_answer=False)
        call_page = dialer_app.live_call_page
        self.assertTrue(display.is_display_on())
        self._assert_proximity_near()
        self.telephony.hangup_call(call_sid)
        self._validate_call_ended(call_page)
        self.assertTrue(
            wait_until(display.is_display_on, timeout=15),
            'Display did not turn back on after call remotely terminated.')
        self._assert_proximity_far()
        self._validate_telephony_call(
            call_sid, self.get_device_phone_number(),
            self.contact_1.number, CALL_INCOMING, CALL_COMPLETED)

    def test_receive_call_during_video_playback(self):
        """
        Receive an incoming call during video playback.
        1. Copy a video to the device.
        2. Navigate to video scope and locate video.
        3. Play local video.
        4. Receive incoming call.
        5. Ensure video is paused when incoming call notification appears.
        6. Accept the call and check proximity sensor is active.
        7. Terminate the call.
        8. Return to video player and check video is paused at correct point.
        9. Play video and ensure proximity sensor is no longer active.
        """
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        shutil.copy2(SRC_VIDEO_FILE_3, LOCAL_VIDEO_FILE_3)

        scope = scopes.go_to_video_scope()
        my_video = scope.get_my_videos()[0]
        preview = scope.select_video(my_video)
        self.assertEqual('video3', preview.get_title())
        preview.play_video()

        player = mediaplayer.get_media_player_proxy().player
        self.assertTrue(player.playing)

        self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=10,
            call_url=CALL_URL_30_SEC)
        call_notifier = self._get_incoming_call_notification()
        self.assertTrue(player.paused)
        stop_time = player.remaining_time

        call_notifier.start_call()
        call_page = dialer.get_dialer_app_proxy().live_call_page
        self._validate_call_started(call_page, self.contact_1.name)
        self.assertTrue(display.is_display_on())
        self._assert_proximity_sensor()
        call_page.hangup_call()

        switcher = self.shell.swipe_to_show_app_switcher()
        switcher.switch_to_app(mediaplayer.APP)
        self.assertTrue(wait_until(
            lambda: self.shell.get_current_focused_app() == mediaplayer.APP))

        self.assertTrue(player.paused)
        self.assertEqual(player.remaining_time, stop_time)
        player.play()
        self.assertTrue(player.playing)

        self._assert_proximity_sensor_inactive()

    def test_receive_call_during_music_playback(self):
        """
        Receive an incoming call during music playback.
        1. Copy music to the device.
        2. Launch the Music app and switch to songs view.
        3. Select the song to start playing.
        4. Receive incoming call.
        5. Ensure the music is paused as soon as the notification appears.
        6. Accept the call and ensure proximity sensor is functional.
        7. Terminate the call from remote end.
        8. Ensure music playback resumes when the call is terminated.
        """
        self.useFixture(media.BackupRestoreLocalMediaFixture())
        self.useFixture(MusicAppWelcomeWizard(show_wizard=False))
        shutil.copy2(SRC_AUDIO_FILE_5, LOCAL_AUDIO_FILE_5)

        music_app = launch_music_app()
        songs = music_app.get_songs_page()
        now_playing = songs.play_track(0)
        self.assertEqual(
            now_playing.get_artist(), test_data.audio.noise60.artist)
        self.assertEqual(
            now_playing.get_title(), test_data.audio.noise60.title)
        player = now_playing.player
        self.assertTrue(player.isPlaying)

        call = self.telephony.make_phonecall(
            to=self.get_device_phone_number(),
            from_=self.contact_1.number,
            hangup_timeout=10,
            call_url=CALL_URL_30_SEC)
        call_notifier = self._get_incoming_call_notification()
        self.assertFalse(player.isPlaying)
        stop_time = player.position

        call_notifier.start_call()
        call_page = dialer.get_dialer_app_proxy().live_call_page
        self._validate_call_started(call_page, self.contact_1.name)
        self.assertTrue(display.is_display_on())
        self._assert_proximity_sensor()

        self.telephony.hangup_call(call['sid'])
        self._validate_call_ended(call_page)
        self.assertTrue(player.isPlaying)
        self.assertGreaterEqual(player.position, stop_time)

        self._assert_proximity_sensor_inactive()
