# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import time

from autopilot.matchers import Eventually

from testtools import skipUnless
from testtools.matchers import Equals

from ubuntu_system_tests.helpers.backup_restore_fixture import (
    BackupRestoreFixture)
from ubuntu_system_tests.helpers import camera
from ubuntu_system_tests.helpers.camera.fixture_setup import (
    SetCameraAccessRequests)
from ubuntu_system_tests.helpers import device
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import images

from ubuntu_system_tests.helpers.location import (
    fixture_setup as location_fixture)
from ubuntu_system_tests.helpers.location.coordinates import EIFFEL_TOWER
from ubuntu_system_tests.helpers import mediaplayer
from ubuntu_system_tests.helpers.platform import is_camera_supported
from ubuntu_system_tests.helpers.unity8.greeter import show_greeter_with_dbus
from ubuntu_system_tests.helpers import videos
from ubuntu_system_tests.helpers import wait_until

from ubuntu_system_tests.tests import base

logger = logging.getLogger(__name__)

RGBA_EXT = '.rgba'
PNG_EXT = '.png'
SCREENSHOT_PREFIX = 'screenshot_'


class BaseCameraTestCase(base.BaseUbuntuSystemTestCase):

    @skipUnless(is_camera_supported(), 'Camera-app required')
    def setUp(self):
        """Backup existing images and settings and launch camera-app."""
        super().setUp()

        self.unity_proxy = self.restart_unity()
        self.unity_proxy.unlock()
        self.useFixture(BackupRestoreFixture(fs.DIR_HOME_CAMERA_CONFIG))
        self.useFixture(BackupRestoreFixture(fs.DIR_HOME_CAMERA_PICTURES))
        self.useFixture(BackupRestoreFixture(fs.DIR_HOME_CAMERA_VIDEOS))
        self.useFixture(BackupRestoreFixture(fs.DIR_HOME_CAMERA_THUMBNAILS))
        self.camera_app = camera.launch_camera_app()

    def _check_image_not_blank(self, image):
        """Check if the taken image is blank or very limited in colours."""
        colour_count = images.get_colour_count(image)
        self.assertGreater(colour_count, 2000)

    def _check_flash_status(self, image, expected_status):
        """Check that the flash was used in the photo."""
        flash_status = images.get_flash_status_from_exif_data(image)
        self.assertThat(flash_status, Equals(expected_status))

    def _get_front_camera_picture_size(self):
        """ Retrieve the size (width, height) of the front camera pictures """
        metadata = device.get_current_device_metadata()
        return metadata.front_camera.width, metadata.front_camera.height

    def _get_rear_camera_picture_size(self):
        """ Retrieve the size (width, height) of the rear camera pictures """
        metadata = device.get_current_device_metadata()
        return metadata.rear_camera.width, metadata.rear_camera.height

    def _check_picture_size(self, file_path, front):
        """ Check the size of the picture against the predefined values for
        the device
        :param file_path: tha path to the picture
        :param front: True if the picture was taken with front camera
        """
        (width, height) = images.get_image_size(images.get_image(file_path))
        if front:
            (expected_width,
             expected_height) = self._get_front_camera_picture_size()
        else:
            (expected_width,
             expected_height) = self._get_rear_camera_picture_size()

        self.assertEqual(expected_width, width, "Wrong picture width")
        self.assertEqual(expected_height, height, "Wrong picture height")

    def _test_take_picture(self, config=None):
        """ Take a picture and check that the photo is not black and that the
        flash was used in case flash_enabled is True.
        :param config: A CameraConfig object with the desired configuration
        :return: The path to the picture
        """

        # Apply the desired configuration for the camera
        camera.config_camera(self.camera_app, config)

        # Take picture and save the file path
        picture_number = len(fs.files_in_dir(fs.DIR_HOME_CAMERA_PICTURES)) + 1
        photoroll_hint = self.camera_app.take_photo(picture_number == 1)
        photo_name = camera.get_last_picture_saved(picture_number)

        # When the photoroll hist is triggered, go to the gallery and go back
        if photoroll_hint:
            gallery = photoroll_hint.swipe_to_dismiss()
            photo = gallery.get_photo(photo_name)
            self.assertThat(photo.visible, Eventually(Equals(True)))
            self.assertThat(photo.enabled, Eventually(Equals(True)))
            self.assertFalse(photo.isVideo)
            gallery.go_back()

        # Validate the picture taken is ok
        self.camera_app.ensure_camera_ready()
        self._check_image_not_blank(photo_name)
        return photo_name

    def get_player_screenshots(self, media_player, screenshots):
        """ Take an screenshot if the remaining time > 0
        :param media_player: the mediaplayer object
        :param screenshots: list to add screenshots
        """
        timeline = media_player.get_timeline()
        if timeline.remainingTime != '00:00:00':
            screenshots.append(self.get_screenshot_image())
        else:
            return True

    def _test_take_video_with_camera(self, config):
        """
        Record a video with the preset camera. Swipe to the photo roll gallery
        view and ensure video is displayed.
        :param config: A CameraConfig object with the desired configuration
        """
        record_time = 10
        screenshot_interval = 1

        # Apply the desired configuration for the camera
        camera.config_camera(self.camera_app, config)

        # Record the video
        self.camera_app.record_video(record_time)

        # Swipe to photo roll and start the playback. During the playback take
        # screenshots of the full screen video
        self.camera_app.swipe_to_gallery(self)
        self.camera_app.start_playback()

        # Verify the source file in the player is the one recently recorded
        media_player = mediaplayer.get_media_player_proxy().player
        self.assertEqual(fs.FILE_PREFIX + camera.get_recorded_video(),
                         media_player.source)

        # Take screenshots during video playback
        screenshots = []
        wait_until(self.get_player_screenshots, media_player, screenshots,
                   timeout=record_time, period=screenshot_interval)

        # Verify at least 3 screenshot have been taken
        self.assertTrue(len(screenshots) > 2)

        # Verify the screenshot have change during the video playback
        for i in range(len(screenshots) - 1):
            im1 = screenshots[i]
            im2 = screenshots[i + 1]
            self.assertFalse(images.images_almost_equal(im1, im2),
                             "There were not detected differences between "
                             "consecutive screenshots")

    def _test_ensure_configured_picture_quality(self, config=None):
        """
        Take three photos with the different encoding qualities available.
        Validate that better quality images are more heavy but the size is the
        same.
        Check that the photos are not black and that the flash was not used.
        :param config: the initial config to apply
        """

        # Apply the desired configuration for the camera
        camera.config_camera(self.camera_app, config)

        # Take pictures with different qualities
        picture_fine = self._test_take_picture(
            camera.get_config(quality='Fine Quality'))
        picture_normal = self._test_take_picture(
            camera.get_config(quality='Normal Quality'))
        picture_basic = self._test_take_picture(
            camera.get_config(quality='Basic Quality'))

        # Get Pictures parameters
        pixels_fine = images.get_image_number_of_pixels(
            images.get_image(picture_fine))
        size_fine = os.path.getsize(picture_fine)

        pixels_normal = images.get_image_number_of_pixels(
            images.get_image(picture_normal))
        size_normal = os.path.getsize(picture_normal)

        pixels_basic = images.get_image_number_of_pixels(
            images.get_image(picture_basic))
        size_basic = os.path.getsize(picture_basic)

        # Assert the number of pixels is the same for all the pictures
        self.assertEqual(pixels_fine, pixels_normal)
        self.assertEqual(pixels_normal, pixels_basic)

        # Assert the size of the picture is more than 20% bigger for better
        # quality images
        quality_improvement = 1.2
        self.assertTrue(size_fine > size_normal * quality_improvement)
        self.assertTrue(size_normal > size_basic * quality_improvement)


class CameraDefaultTestCase(BaseCameraTestCase):

    def setUp(self):
        self.useFixture(SetCameraAccessRequests())
        super().setUp()


class CameraTestCase(CameraDefaultTestCase):

    def setUp(self):
        super().setUp()

    def test_take_picture_with_rear_camera(self):
        """
        Take a photo with the rear camera and flash enabled.
        Swipe to the photo roll gallery view and ensure photo is displayed.
        Check that the photo is not black and that the flash was used.
        """
        pic = self._test_take_picture(camera.get_config(flash='On',
                                                        mode='camera'))
        self._check_flash_status(pic, True)

    def test_take_picture_with_front_camera(self):
        """
        Take a photo with the front camera. Swipe to the photo roll gallery
        view and ensure photo is displayed. Check that the photo is not black.
        """
        self._test_take_picture(camera.get_config(swap_camera=True,
                                                  mode='camera'))

    def test_take_picture_with_flash_off(self):
        """
        Take a photo with the rear camera and flash disabled.
        Swipe to the photo roll gallery view and ensure photo is displayed.
        Check that the photo is not black and that the flash was not used.
        """
        pic = self._test_take_picture(camera.get_config(flash='Off',
                                                        mode='camera'))
        self._check_flash_status(pic, False)

    def test_take_picture_with_hdr_on(self):
        """
        Take a photo with the rear camera and HDR enabled.
        Swipe to the photo roll gallery view and ensure photo is displayed.
        Check that the photo is not black and that the HDR was used.
        """
        self._test_take_picture(camera.get_config(hdr='On',
                                                  mode='camera'))

    def test_take_picture_with_hdr_off(self):
        """
        Take a photo with the rear camera and HDR disabled.
        Swipe to the photo roll gallery view and ensure photo is displayed.
        Check that the photo is not black and that the HDR was not used.
        """
        self._test_take_picture(camera.get_config(hdr='Off',
                                                  mode='camera'))

    def test_ensure_configured_picture_quality_front_camera(self):
        """
        Take three photos with the front camera using the different quality
        encodings. Validate that better quality images are more heavy.
        """
        self._test_ensure_configured_picture_quality(
            camera.get_config(swap_camera=True,
                              mode='camera'))

    def test_ensure_configured_picture_quality_rear_camera(self):
        """
        Take three photos with the rear camera using the different quality
        encodings. Validate that better quality images are more heavy.
        """
        self._test_ensure_configured_picture_quality()

    def test_lock_unlock_camera(self):
        """
        Lock and unlock 10 times the phone while the camera preview is shown
        and verify the camera doesn't freeze by comparing two screenshots.
        Both screenshots should be different because the camera is making
        autofocus while the screenshots are taken.
        """
        for i in range(10):
            # Lock & unlock the phone
            show_greeter_with_dbus()
            self.unity_proxy.unlock()

            # Get two screenshots once the phone is unlocked and the camera
            # preview is shown, waiting a time between both
            im1 = self.get_screenshot_image()
            time.sleep(i * 0.5)
            im2 = self.get_screenshot_image()

            # As the camera makes autofocus after the screen is unlocked, both
            # screenshots should be different
            self.assertFalse(images.images_almost_equal(im1, im2))

    def test_check_rear_picture_size(self):
        """
        Take a picture with the rear camera and check the size is the expected
        for the current device.
        """
        picture = self._test_take_picture()

        # Note: This assertion will fail due to the issue raised on
        # https://bugs.launchpad.net/camera-app/+bug/1468341
        self._check_picture_size(picture, False)

    def test_check_front_picture_size(self):
        """
        Take a picture with the rear camera and check the size is the expected
        for the current device.
        """
        picture = self._test_take_picture(
            camera.get_config(swap_camera=True))

        # Note: This assertion will fail due to the issue raised on
        # https://bugs.launchpad.net/camera-app/+bug/1468341
        self._check_picture_size(picture, True)

    def test_take_video_with_rear_camera(self):
        """
        Record a video with the rear camera. Swipe to the photo roll gallery
        view and ensure video is displayed.
        """
        self._test_take_video_with_camera(
            camera.get_config(mode='video'))

    def test_take_video_with_front_camera(self):
        """
        Record a video with the front camera. Swipe to the photo roll gallery
        view and ensure video is displayed.
        """
        self._test_take_video_with_camera(camera.get_config(mode='video',
                                                            swap_camera=True))

    def test_tap_to_focus_during_photo_preview(self):
        """
        Open the camera and tap to focus, ensure images are different.
        """
        self.camera_app.ensure_camera_ready()
        im1 = self.get_screenshot_image()
        self.camera_app.make_random_focus()
        im2 = self.get_screenshot_image()
        self.assertFalse(images.images_almost_equal(im1, im2))


class CameraLocationTestCase(CameraDefaultTestCase):

    def setUp(self):
        self.useFixture(location_fixture.DummyLocation(*EIFFEL_TOWER))
        super().setUp()

    def test_take_picture_with_geotagging(self):
        """ Validate the location metadata for the pictures taken with the
        camera app is correct when the geotagging is enabled.
        """
        pic = self._test_take_picture(camera.get_config(mode='camera',
                                                        gps='On'))
        (lat, long) = images.get_gps_location_from_exif_data(pic)

        self.assertEqual(round(float(EIFFEL_TOWER[0]), 2),
                         round(float(lat), 2))
        self.assertEqual(round(float(EIFFEL_TOWER[1]), 2),
                         round(float(long), 2))


class CameraDisabledTestCase(BaseCameraTestCase):

    def setUp(self):
        self.useFixture(SetCameraAccessRequests(camera=False))
        super().setUp()

    def test_deny_prompt_camera_disabled(self):
        """
        Disable the camera service (currently done by modifying the trust
        database as not possible to interact with the prompts) and ensure
        that the main take photo/shoot video buttons are disabled.
        """
        self.assertFalse(self.camera_app.get_exposure_button().enabled)
        self.camera_app.switch_record_control()
        self.assertFalse(self.camera_app.get_exposure_button().enabled)


class AudioDisabledTestCase(BaseCameraTestCase):

    def setUp(self):
        self.useFixture(SetCameraAccessRequests(audio=False))
        super().setUp()

    def test_deny_prompt_audio_disabled(self):
        """
        Record a video with the rear camera denying the access to the audio
        service (currently done by modifying the trust database as not
        possible to interact with the prompts). Validate the video recorded
        is ok and that in the exif data there are not any audio channels saved
        """

        # Deny the access request to the audio service, record video and
        # validate it is ok
        self._test_take_video_with_camera(
            camera.get_config(mode='video'))

        # Check the number of channels in the video file is 0
        video_path = camera.get_last_video_saved()
        channels = videos.get_audio_channels_from_exif_data(video_path)
        self.assertEqual(0, channels, "Invalid number of audio channels in "
                                      "the video recorded")
