# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import shutil

from autopilot.matchers import Eventually
from testtools import skipUnless
from testtools.matchers import Equals

from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers import web
from ubuntu_system_tests.helpers.addressbook import(
    fixture_setup as address_book_fixture)
from ubuntu_system_tests.helpers.camera import fixture_setup as camera_fixture
from ubuntu_system_tests.helpers.media import BackupRestoreLocalMediaFixture
from ubuntu_system_tests.helpers.messaging import (
    Message, Attachment, MEDIA_TYPE_PICTURE, MEDIA_TYPE_VIDEO,
    MEDIA_TYPE_CONTACT)
from ubuntu_system_tests.helpers.uid import b64_uuid

from ubuntu_system_tests.tests.base_messaging import MessagingBase
from ubuntu_system_tests.tests import data as test_data


SAMPLE_PICTURE = os.path.basename(test_data.CAMERA_IMAGE_FILE_3)
SAMPLE_VIDEO = os.path.basename(test_data.CAMERA_VIDEO_FILE_1)
TIMEOUT_5_MIN = 300


class MMSTestCase(MessagingBase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        super().setUp()
        self.useFixture(BackupRestoreLocalMediaFixture())
        self.useFixture(camera_fixture.SetCameraAccessRequests())
        self._prepare_test_data()

    def _prepare_test_data(self):
        """Copy media files used during mms tests."""
        fs.create_directory_if_not_exists(fs.DIR_HOME_CAMERA_PICTURES)
        fs.create_directory_if_not_exists(fs.DIR_HOME_CAMERA_VIDEOS)
        shutil.copy2(test_data.SRC_IMAGE_FILE_3, test_data.CAMERA_IMAGE_FILE_3)
        shutil.copy2(test_data.SRC_VIDEO_FILE_1, test_data.CAMERA_VIDEO_FILE_1)

    def _test_send_mms_message(self, recipients, attachments):
        """Send an MMS to recipients with specified attachments and validate
        the message is received correctly.
        :param recipients: List of recipients to send the message to.
        :param attachments: List of attachment objects.
        """
        self.launch_messaging()
        message = Message(b64_uuid(), attachments)
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_receive_mms_from_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from
        1. From another device send an MMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
           - An image should be contained within the message
        """
        message_body = b64_uuid()
        media_url = 'http://people.canonical.com/~platform-qa/mms-ubuntu.jpg'
        mms_media_file = web.download_file(media_url)
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.get_telephony_service_number1(),
            message_body,
            media_url=media_url,
            timeout=TIMEOUT_5_MIN)

        self.assertThat(notification.summary,
                        Eventually(Equals('Message from {contact}'.format(
                                          contact=self.contact_1.name))))
        self.assertThat(notification.body, Eventually(Equals(message_body)))
        notification.tap_on()

        self.get_messaging_app()
        self.assertThat(self.main_view.get_header().title,
                        Eventually(Equals(self.contact_1.name)))

        mms_message = self.main_view.get_message(message_body)
        photo_path = mms_message.get_mms_content_file_path()
        self.assertTrue(fs.compare_files(mms_media_file, photo_path))

    def test_send_mms_with_photo_from_gallery(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending to and a photo in the gallery
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5  Tap the "attach" button on the button left of the screen
            - Content hub opens with list of Apps we can select content from
        6. Tap 'Gallery' button
            - Gallery App opens
        7  On the main camera screen select a photo
           - Messaging App is brought to front with the newly selected
             image being shown besides the message text field
        8. Press the Send button
            - The message is added to the conversation and is shown in green
              containing the newly selected photo's thumbnail
        9. Confirm the MMS was received on the other device
            - The message was received with the correct text and photo
        """
        attachment = Attachment(
            attach_method=self._add_attachment_from_gallery,
            identifier=SAMPLE_PICTURE, media_type=MEDIA_TYPE_PICTURE,
            validation_file=test_data.CAMERA_IMAGE_FILE_3)
        self._test_send_mms_message([self.contact_1], [attachment])

    def test_send_mms_with_photo_from_camera(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5  Tap the "attach" button on the button left of the screen
            - Content hub opens with list of Apps we can select content from
        6. Tap 'Camera' button
            - Camera App opens
        7  On the main camera screen tap the exposure button
           - New photo is taken and the exposure button is changed to
             "affirmative" icon
        8  Tap the affirmative icon
           - Messaging App is brought to front with the newly selected
             image being shown besides the message text field
        9. Press the Send button
            - The message is added to the conversation and is shown in green
              containing the newly selected photo's thumbnail
        10. Confirm the MMS was received on the other device
            - The message was received with the correct text and photo
        """
        attachment = Attachment(
            attach_method=self._add_attachment_from_camera,
            media_type=MEDIA_TYPE_PICTURE)
        self._test_send_mms_message([self.contact_1], [attachment])

    def test_send_mms_with_photo_from_camera_photoroll(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending to and a photo in the camera roll
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5  Tap the "attach" button on the button left of the screen
            - Content hub opens with list of Apps we can select content from
        6. Tap 'Camera' button
            - Camera App opens
        7  On the main camera screen swipe left
           - Photo roll opens with the a photo visible
        8  Select the sample photo
           - Messaging App is brought to front with the newly selected
             image being shown besides the message text field
        9. Press the Send button
            - The message is added to the conversation and is shown in green
              containing the newly selected photo's thumbnail
        10. Confirm the MMS was received on the other device
            - The message was received with the correct text and photo
        """
        attachment = Attachment(
            attach_method=self._add_attachment_from_camera_photoroll,
            media_type=MEDIA_TYPE_PICTURE)
        self._test_send_mms_message([self.contact_1], [attachment])

    def test_send_mms_with_video_from_gallery(self):
        """Send an MMS message with attached video from gallery.
        1. Launch messaging app.
        2. Compose new message and attach video file from gallery.
        3. Send message.
        4. Ensure that the attachments were received and they match the
           original video that was attached to message."""
        attachment = Attachment(
            attach_method=self._add_attachment_from_gallery,
            identifier=SAMPLE_VIDEO, media_type=MEDIA_TYPE_VIDEO,
            validation_file=test_data.CAMERA_VIDEO_FILE_1)
        self._test_send_mms_message([self.contact_1], [attachment])

    def test_send_mms_with_contact_from_address_book(self):
        """Send an MMS message with attached contact from address book.
        1. Launch messaging app.
        2. Compose new message and attach contact from address book.
        3. Send message.
        4. Ensure that the attachments were received and they match the
           original contact that was attached to message."""
        self.useFixture(
            address_book_fixture.AddTemporaryContacts([self.contact_2]))
        attachment = Attachment(
            attach_method=self._add_attachment_from_address_book,
            identifier=self.contact_2.name, media_type=MEDIA_TYPE_CONTACT,
            validation_data=[self.contact_2.name, self.contact_2.number])
        self._test_send_mms_message([self.contact_1], [attachment])
