# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from autopilot.matchers import Eventually
from testtools import skipUnless
from testtools.matchers import Equals

from ubuntu_system_tests.tests.base_messaging import MessagingBase
from ubuntu_system_tests.helpers.addressbook import(
    Contact,
    fixture_setup as address_book_fixture
)
from ubuntu_system_tests.helpers import messaging
from ubuntu_system_tests.helpers.messaging import(
    Message,
    Search,
    fixture_setup as messaging_fixture
)
from ubuntu_system_tests.helpers import network
from ubuntu_system_tests.helpers.network import(
    fixture_setup as network_fixture
)
from ubuntu_system_tests.helpers import ofono
from ubuntu_system_tests.helpers.indicators.message import NotificationsIndicatorPage, NotificationsIndicatorItem  # NOQA
from ubuntu_system_tests.helpers.system_settings import SIM_1, SIM_ASK
from ubuntu_system_tests.helpers.uid import b64_uuid
from ubuntu_system_tests.helpers import webbrowser

CONTACT_1 = Contact(name='Imran Khan', number='08151')
CONTACT_2 = Contact(name='John Connor', number='08153')
CONTACT_3 = Contact(name='Bugs Bunny', number='08155')

DUMMY_SEARCH_QUERY = 'lolololX'

SEARCH_1 = Search(query=CONTACT_1.name, expected_thread_count=1,
                  expected_thread_number=CONTACT_1.number)
SEARCH_2 = Search(query=CONTACT_3.name.split()[1], expected_thread_count=1,
                  expected_thread_number=CONTACT_3.number)
SEARCH_3 = Search(query=DUMMY_SEARCH_QUERY, expected_thread_count=0,
                  expected_thread_number=None)

TIMEOUT_5_MIN = 300
# the number of sms threads in history.sqlite
DEFAULT_SMS_THREAD_COUNT = 5


class SMSTestCase(MessagingBase):

    @skipUnless(ofono.is_sim_inserted(), 'No SIM inserted')
    def setUp(self):
        super().setUp()

    def test_receive_sms_from_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        message_body = b64_uuid()
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_1.number,
            message_body,
            timeout=TIMEOUT_5_MIN)

        self._validate_notification_summary_and_body(
            notification, self.contact_1.name, message_body
        )
        notification.tap_on()
        self.get_messaging_app()
        msgs = self.main_view.get_main_page().open_thread(
            self.contact_1.number)
        self.assertThat(msgs.title,
                        Eventually(Equals(self.contact_1.name)))

    def _validate_notification_summary_and_body(self, notification,
                                                name_or_number, body,
                                                expected_sim='SIM 1'):
        """
        Validate the expected summary and body of the message against the
        content of the notification
        """
        expected_summary = name_or_number
        if ofono.number_of_sims() > 1:
            expected_summary += ' - [{sim}]'.format(sim=expected_sim)
        self.assertThat(
            notification.summary, Eventually(Equals(expected_summary))
        )
        self.assertThat(notification.body, Eventually(Equals(body)))

    def _request_incoming_sms_and_open_message_indicator(self, message):
        """
        Request an incoming SMS on DUT, wait for the notification to
        hide and open the message indicator.
        """
        self.shell.clear_notifications()
        self.assertFalse(self.shell.is_new_notification_displayed())

        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_1.number,
            message.body,
            timeout=TIMEOUT_5_MIN)
        self._validate_notification_summary_and_body(
            notification, self.contact_1.name, message.body
        )
        self.assertEqual(message.body, notification.body)
        notification.wait_until_destroyed()

        self.assertTrue(self.shell.is_new_notification_displayed())
        message_indicator = self.shell.open_notifications_indicator()
        self.addCleanup(self.shell.close_indicator_page)
        return message_indicator

    def test_receive_sms_device_locked(self):
        """
        Receive an SMS from a contact when the device is still locked.
        1. Ensure there are no existing notifications
        2. Receive sms from contact whilst greeter is locked
        3. Check notification is displayed
        4. Swipe down notifications indicator to display new message
        5. Clear all messages

        """
        message = Message(body=b64_uuid())
        msg_indicator = self._request_incoming_sms_and_open_message_indicator(
            message)
        message_item = msg_indicator.get_message(sender=self.contact_1.name,
                                                 body=message.body)
        msg_indicator.clear_all()
        message_item.wait_until_destroyed()
        self.assertThat(lambda: self.shell.is_new_notification_displayed(),
                        Eventually(Equals(False)))

    def _reply_sms_from_indicator_and_validate_replies(self, message,
                                                       recipients):
        msg_indicator = self._request_incoming_sms_and_open_message_indicator(
            message)
        message_item = msg_indicator.get_message(sender=recipients[0].name,
                                                 body=message.body)
        notifications_list = self.shell.get_notifications_list()
        self.start_notification_observer(expected_count=len(recipients),
                                         timeout=TIMEOUT_5_MIN)
        msg_indicator.reply_sms(message.body, message_item)
        notification = notifications_list.get_new_message_notification(
            TIMEOUT_5_MIN)
        self._validate_notification_summary_and_body(
            notification, self.contact_1.name, self.contact_1.expected_reply
        )
        notification.wait_until_destroyed()
        self.validate_outgoing_message_received(recipients, message)

    def test_reply_sms_from_indicator_device_locked(self):
        """
        Prerequisite: device screen locked with at least one contact in
        address book.
        1. Send an incoming sms to DUT
        2. Wait for message notification to hide
        3. Drag down the message indicator and tap the newly received message
        4. Type in the reply and send the message
        5. Ensure the reply is received on the other end
        """
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self._reply_sms_from_indicator_and_validate_replies(message,
                                                            recipients)

    def test_reply_sms_from_indicator(self):
        """
        At least one contact in address book
        1. Send an incoming sms to DUT
        2. Wait for message notification to hide
        3. Drag down the message indicator and tap the newly received message
        4. Type in the reply and send the message
        5. Ensure the reply is received on the other end
        """
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self._reply_sms_from_indicator_and_validate_replies(message,
                                                            recipients)

    def test_receive_sms_from_unknown_number(self):
        """
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             number and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        message = Message(body=b64_uuid())
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_2.number,
            message.body,
            timeout=TIMEOUT_5_MIN)

        self._validate_notification_summary_and_body(
            notification, self.contact_2.number, message.body
        )
        notification.tap_on()

        self.get_messaging_app()
        msgs = self.main_view.get_main_page().open_thread(
            self.contact_2.number)
        self.assertThat(msgs.title,
                        Eventually(Equals(self.contact_2.number)))

    def test_receive_sms_with_no_data_connection(self):
        """
        Prerequisite: A contact in the address book with the number you
        are sending from. Wifi and data connections disabled.
        1. From another device send an SMS to the phone number of a SIM
           card that is in the SUT
           - A notification appears containing the senders
             name and the message content.
        2. Tap on the message notification before it disappears
           - The messaging application is opened showing all
             messages received by that sender. The message
             just sent should appear at the bottom.
        """
        # Only turn off data connection as we need wifi to request
        # incoming SMS.
        self.useFixture(network_fixture.TurnOffDataConnection())
        notifications_list = self.shell.get_notifications_list()
        message = Message(body=b64_uuid())
        self.request_incoming_message(self.get_device_phone_number(),
                                      self.contact_1.number,
                                      message.body)
        # Turn off wifi before the incoming SMS
        self.useFixture(network_fixture.TurnOffWifi())
        notification = notifications_list.get_new_message_notification(
            TIMEOUT_5_MIN)
        self._validate_notification_summary_and_body(
            notification, self.contact_1.name, message.body
        )
        notification.tap_on()

        self.get_messaging_app()
        msgs = self.main_view.get_main_page().open_thread(
            self.contact_1.number)
        self.assertThat(msgs.title,
                        Eventually(Equals(self.contact_1.name)))

    def test_receive_sms_with_url_and_open_in_browser(self):
        """
        1. Request an incoming SMS containing a URL to DUT
        2. Tap on the message notification to open the messaging app
        3. Click on the message text in the messaging app
        4. Ensure the webbrowser opens and the correct url is loaded
        """
        domain = 'www.ubuntu.com'
        url = 'http://{}'.format(domain)
        message = Message(body=domain)
        notification = self.receive_message_and_get_notification(
            self.get_device_phone_number(),
            self.contact_1.number,
            message.body,
            timeout=TIMEOUT_5_MIN)
        self._validate_notification_summary_and_body(
            notification, self.contact_1.name, message.body
        )
        notification.tap_on()

        self.get_messaging_app()
        msgs = self.main_view.get_main_page().open_thread(
            self.contact_1.number)
        self.assertThat(msgs.title,
                        Eventually(Equals(self.contact_1.name)))
        self.click_message_url(domain)
        webbrowser_app = webbrowser.get_webbrowser_app_proxy()
        webbrowser_app.wait_until_page_loaded('{}/'.format(url))

    def verify_thread(self, messaging_app, phone_number, message):
        """Check that the thread with given number and message exists"""
        # verify our number
        message_thread = messaging_app.get_thread_from_number(phone_number)
        self.assertEquals(message_thread.phoneNumber, phone_number)
        # verify our text
        self.assertEquals(message_thread.textMessage, message)

    def test_receive_sms_clear_notification(self):
        """
        1. Receive an incoming SMS
        2. Drag down the notifications indicator and press 'clear all'
        3. Open messaging app and ensure the newly received message exists
        """
        message = Message(body=b64_uuid())
        msg_indicator = self._request_incoming_sms_and_open_message_indicator(
            message)
        message_item = msg_indicator.get_message(sender=self.contact_1.name,
                                                 body=message.body)
        msg_indicator.clear_all()
        message_item.wait_until_destroyed()
        self.assertThat(lambda: self.shell.is_new_notification_displayed(),
                        Eventually(Equals(False)))
        messaging_app = messaging.launch_messaging_app_from_launcher()
        self.verify_thread(messaging_app, self.contact_1.number, message.body)

    @skipUnless(ofono.number_of_sims() > 1, 'More than 1 SIM is required.')
    def test_send_sms_choose_sim(self):
        """
        Prerequisite: 2 SIMs inserted.

        1. Open system settings and set 'Ask me each time' for messages
        2. Open Messaging app.
        3. Choose SIM to send message with
        4. Compose SMS message to contact
        5. Send SMS and verify it is sent using the correct SIM

        """
        self.useFixture(messaging_fixture.SetDualSIMPromptConfig())
        self.set_default_sim_for_messages_with_uss(SIM_ASK)
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(
            recipients, message,
            sim_dialog=self.main_view.get_sim1_default_confirmation_dialog,
            confirm_default_sim=True)

    @skipUnless(ofono.number_of_sims() > 1, 'More than 1 SIM is required.')
    def test_send_sms_using_default_sim(self):
        """
        Prerequisite: 2 SIMs inserted.

        1. Open system settings and set 'Ask me each time' for messages
        2. Open Messaging app.
        3. Choose SIM to send message with
        4. Compose SMS message to contact
        5. Send SMS and verify it is sent using the correct SIM

        """
        self.useFixture(messaging_fixture.SetDualSIMPromptConfig())
        self.set_default_sim_for_messages_with_uss(SIM_1)

        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(
            recipients, message, expected_default_sim=SIM_1)

    def test_send_sms_to_contact(self):
        """
        Prerequisite: A contact in the address book with the number you
                     are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5. Press the Send button
            - The message is added to the conversation and is shown in green
        6. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_to_multiple_contacts(self):
        """
        Prerequisite: Two contacts in the address book with the numbers you
                     are sending to
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the name of the first contact to send to in the To.. field
            - The contacts name is stored in the To... field
        4. Type the name of the second contact to send to in the To.. field
            - The contacts name is stored in the To... field
        5. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        6. Press the Send button
            - The message is added to the conversation and is shown in green
        7. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.useFixture(
            address_book_fixture.AddTemporaryContacts([self.contact_2]))
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1, self.contact_2]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_to_unknown_number(self):
        """
        1. Open the messaging application
            - The messaging application is opened to the main screen
        2. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        3. Type the number of the contact to send to in the To.. field
            - The contacts number is stored in the To... field
        4. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        5. Press the Send button
            - The message is added to the conversation and is shown in green
        6. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.launch_messaging()
        message = Message(body=b64_uuid())
        self.send_message_to_number_and_validate_reply(self.contact_2, message)

    def turn_on_wifi_and_wait_for_internet(self, timeout):
        """Turn on wifi and ping a server to ensure internet connectivity."""
        network.turn_on_wifi()
        success = network.ensure_internet_connectivity(timeout)
        if not success:
            self.fail("Internet not working after {} seconds".format(timeout))

    def test_send_long_sms_to_contact(self):
        """
        1. Open Messaging app.
        2. Compose long SMS message to contact
        3. Send SMS and verify it is sent

        """
        self.launch_messaging()
        message = Message(body=b64_uuid(length=180))
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(recipients, message)

    def test_send_sms_with_no_data_connection(self):
        """
        Prerequisite: A contact in the address book with the number you
                     are sending to.
        1. Open the network indicator by dragging from top bar
            - The network indicator opens with the list of active SIMs
              and wifi status.
        2. Tap the Wifi switch if its already ON
            - Wifi turns off
        3. Tap the 'Cellular settings...' item
            - System settings opens with the cellular page
        4. Tap the cellular data switch if its ON
            - Data connection gets disabled
        5. Open the messaging application
            - The messaging application is opened to the main screen
        6. Swipe up to create a new message
            - The keyboard appears with the message composition controls
              above it (add photo button, 'Write a message...' field)
              and a Send button. The To.. field has focus.
        7. Type the name of the contact to send to in the To.. field
            - The contacts name is stored in the To... field
        8. Type the message text in the Write a message..
           field - e.g. 'Hello World'
            - The message text is entered into the Write a message.. field
        9. Press the Send button
            - The message is added to the conversation and is shown in green
        10. Confirm the message was received on the other device
            - The message was received with the correct text
        """
        self.useFixture(network_fixture.TurnOffInternet())
        self.launch_messaging()
        message = Message(body=b64_uuid())
        recipients = [self.contact_1]
        self.send_message_to_contacts_and_validate_replies(
            recipients, message, query_telephony_service=False)
        self.turn_on_wifi_and_wait_for_internet(timeout=30)
        self.validate_outgoing_message_received(recipients, message)

    def _assert_all_contacts_are_visible(self, contacts):
        self.assertThat(lambda: self.main_view.get_visible_threads_count(),
                        Eventually(Equals(DEFAULT_SMS_THREAD_COUNT)))
        for contact in contacts:
            self.assertTrue(
                self.main_view.is_thread_for_number_visible(contact.number))

    def test_search_sms_by_contact_name(self):
        """
        The test is using a dummy database from the messaging_app test suite,
        it is used for having a pre-filled sms history.
        1. Open messaging app
        2. Search for messages by Name
        3. Ensure message filtering is working
        """
        contacts = [CONTACT_1, CONTACT_2, CONTACT_3]
        searches = [SEARCH_1, SEARCH_2, SEARCH_3]
        self.useFixture(address_book_fixture.AddTemporaryContacts(contacts))
        self.useFixture(messaging_fixture.FillCustomSmsHistory())
        self.launch_messaging()

        # verify that we got the messages from our database
        self._assert_all_contacts_are_visible(contacts)

        for search in searches:
            self.main_view.search(search.query)
            self.assertThat(lambda: self.main_view.get_visible_threads_count(),
                            Eventually(Equals(search.expected_thread_count)))
            if int(search.expected_thread_count) > 0:
                self.assertTrue(self.main_view.is_thread_for_number_visible(
                    search.expected_thread_number))

        self.main_view.clear_search()
        self._assert_all_contacts_are_visible(contacts)
