# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu System Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from collections import namedtuple
import logging
import os.path

from autopilot import exceptions
from autopilot.matchers import Eventually
from testtools.matchers import Equals

from ubuntu_keyboard.emulators import keyboard

from ubuntu_system_tests.helpers import autopilot
from ubuntu_system_tests.helpers.data import load_test_metadata
from ubuntu_system_tests.helpers import file_system as fs
from ubuntu_system_tests.helpers import gallery
from ubuntu_system_tests.helpers import images
from ubuntu_system_tests.helpers import mediaplayer
from ubuntu_system_tests.helpers import messaging
from ubuntu_system_tests.helpers.messaging import (
    fixture_setup as messaging_fixtures
)
from ubuntu_system_tests.helpers.ofono import number_of_sims
from ubuntu_system_tests.helpers import wait_until
from ubuntu_system_tests.helpers.indicators import TRANSFER_INDICATOR
from ubuntu_system_tests.helpers.webbrowser import (
    fixture_setup as webbrowser_fixtures
)

from ubuntu_system_tests.tests import base_webbrowser


logger = logging.getLogger(__name__)
PageUrls = namedtuple('PageUrls', ['entered_url',
                                   'title_url',
                                   'actual_url',
                                   'page_title'])
FileDesc = namedtuple('FileDesc', ['file_name',
                                   'sha1'])

APP = 'webbrowser-app'
PARAMS = '--desktop_file_hint=/usr/share/applications/webbrowser-app.desktop'
ARGS = ['--new-session']
QT = 'qt'

test_data = {
    'start_page': PageUrls(
        entered_url='start.ubuntu.com',
        title_url='start.ubuntu.com',
        actual_url='http://start.ubuntu.com/current/Google/?sourceid=hp',
        page_title='Ubuntu Start Page'),
    'page1': PageUrls(
        entered_url='people.canonical.com/~platform-qa/page1.html',
        title_url='people.canonical.com',
        actual_url='http://people.canonical.com/~platform-qa/page1.html',
        page_title='First page'),
    'page2': PageUrls(
        entered_url='lillypilly.canonical.com/~platform-qa/page2.html',
        title_url='lillypilly.canonical.com',
        actual_url='http://lillypilly.canonical.com/~platform-qa/page2.html',
        page_title='Second page'),
    'page3': PageUrls(
        entered_url='lillypilly.canonical.com/~platform-qa/page3.html',
        title_url='lillypilly.canonical.com',
        actual_url='http://lillypilly.canonical.com/~platform-qa/page3.html',
        page_title='Third page'),
    'ubuntu': PageUrls(
        entered_url='www.ubuntu.com',
        title_url='ubuntu.com',
        actual_url='http://www.ubuntu.com/',
        page_title='The leading OS for PC, tablet, phone and cloud | Ubuntu'),
    'canonical': PageUrls(
        entered_url='www.canonical.com',
        title_url='canonical.com',
        actual_url='http://www.canonical.com/',
        page_title='Canonical | The company behind Ubuntu'),
    'linux': PageUrls(
        entered_url='http://www.linux.org/',
        title_url='linux.org',
        actual_url='http://www.linux.org/',
        page_title='Linux.org'),
    'ubuntu_img': FileDesc(
        file_name='ubuntu.jpg',
        sha1='429cbd692c4abd3e5c785edf5dcf0993d21076b1')
}

data = load_test_metadata()
REFERENCE_DOWNLOAD_IMAGE = os.path.join(fs.DIR_TEST_DATA_IMAGES,
                                        data.images.img4)


class WebBrowserSystemTest(base_webbrowser.WebBrowserBase):
    """This Class tests the web browser app"""

    def setUp(self):
        super().setUp()

    def ensure_browser_actually_loaded(self, unity_proxy):
        """Fail the test if the browser app doesn't get past loading.

        Due to this bug[1] after a reboot or 2 of unity8 the browser is no
        longer launchable from the apps scope.
        Check for when this happens and if it does, fail the test with a nice
        message.

        [1] https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361

        """
        try:
            spinner = unity_proxy.select_single('ActivityIndicatorStyle')
            spinner.wait_until_destroyed()
        except exceptions.StateNotFoundError:
            # This is fine, it's already loaded
            pass
        except RuntimeError:
            self.fail("""Test failed due to browser not loading.
            Please see this bug for further information:
            https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361
            """)

    def test_navigate_forward_back(self):
        """Test the back and forward mechanism provided by the web browser"""
        self.browser.go_to_url(test_data.get('page1').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page1').actual_url)

        self.browser.go_to_url(test_data.get('page2').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page2').actual_url)

        self.browser.go_back()
        self.browser.wait_until_page_loaded(test_data.get('page1').actual_url)

        self.browser.go_forward()
        self.browser.wait_until_page_loaded(test_data.get('page2').actual_url)

    def test_navigate_backwards_quickly_no_crash(self):
        """Test quickly going back a few pages does not crash the web
        browser."""
        for page in ['page1', 'page2', 'ubuntu', 'canonical']:
            self.browser.go_to_url(test_data.get(page).entered_url)
            self.browser.wait_until_page_loaded(test_data.get(page).actual_url)

        for i in range(4):
            self.browser.go_back()

        self.browser.wait_until_page_loaded(
            test_data.get('start_page').actual_url)

    def test_history_view(self):
        """Test that after visit three web pages, all of them are listed in the
        history page of the web browser"""
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)

        self.browser.go_to_url(test_data.get('linux').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('linux').actual_url)

        self.browser.go_to_url(test_data.get('canonical').entered_url)
        self.browser.wait_until_page_loaded(
            test_data.get('canonical').actual_url)

        self.browser.open_history_view()

        titles = [url.title for url in self.browser.get_history_urls()]
        self.assertEqual(len(titles), 3)
        self.assertIn(test_data.get('canonical').title_url, titles)
        self.assertIn(test_data.get('linux').title_url, titles)
        self.assertIn(test_data.get('ubuntu').title_url, titles)

    def _go_to_url_wait_until_loaded(self, site):
        """Navigate to site url, wait for it to load and set bookmark state."""
        self.browser.go_to_url(site.entered_url)
        self.browser.wait_until_page_loaded(site.actual_url)

    def _set_bookmark_state(self, new_state):
        """Assert that current bookmark state is not equal to the specified
        state and then set the bookmark to the new state."""
        self.assertNotEqual(self.browser.chrome.bookmarked, new_state)
        bookmark_toggle = self.browser.address_bar.get_bookmark_toggle()
        self.browser.pointing_device.click_object(bookmark_toggle)
        self.assertThat(self.browser.chrome.bookmarked,
                        Eventually(Equals(new_state)))

    def test_create_bookmark_and_navigate(self):
        """Test that the user is able to create a bookmark and then navigate
        to it"""
        sites = [test_data.get('ubuntu'), test_data.get('canonical')]
        self._go_to_url_wait_until_loaded(sites[0])
        self._set_bookmark_state(True)
        self._go_to_url_wait_until_loaded(sites[1])
        self.assertFalse(self.browser.chrome.bookmarked)

        new_tab_page = self.browser.open_new_tab()
        bookmark_urls = new_tab_page.get_bookmarks_list().get_urls()
        self.assertCountEqual([sites[0].actual_url], bookmark_urls)

        expected_recent = [test_data.get('start_page')] + sites
        expected_recent_urls = [item.actual_url for item in expected_recent]

        recent_urls = new_tab_page.get_top_sites_list().get_urls()
        self.assertCountEqual(recent_urls, expected_recent_urls)

    def test_delete_bookmark(self):
        """Test that the user is able to create a bookmark and then delete
        it"""
        site = test_data.get('ubuntu')
        self._go_to_url_wait_until_loaded(site)
        self._set_bookmark_state(True)
        self._go_to_url_wait_until_loaded(site)
        self._set_bookmark_state(False)

        new_tab_page = self.browser.open_new_tab()
        self.assertCountEqual(new_tab_page.get_bookmarks_list().get_urls(), [])

        expected_recent = [test_data.get('start_page'), site]
        expected_recent_urls = [item.actual_url for item in expected_recent]
        recent_urls = new_tab_page.get_top_sites_list().get_urls()
        self.assertCountEqual(recent_urls, expected_recent_urls)

    def _assert_find_in_page_item_not_visible_in_drawer(self):
        """Open the drawer and ensure 'Find in page' item is not visible."""
        self.browser.open_drawer()
        self.assertFalse(autopilot.is_element(
            self.browser.chrome.get_drawer_action, 'findinpage'))

    def test_find_in_page_invisible_new_tabs_view(self):
        """
        1. Open a website
        2. Go to new tab page
        3. Open the drawer and ensure 'Find in page' item is not visible
        """
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)
        self.browser.open_new_tab()
        self._assert_find_in_page_item_not_visible_in_drawer()

    def _find_in_page_buttons_enabled(self, res):
        """ Indicate if both find in page buttons are in the correct enabled
        status
        """
        previous_button = self.browser.get_find_in_page_previous_button()
        next_button = self.browser.get_find_in_page_next_button()
        return (previous_button.enabled == res and next_button.enabled == res)

    def _check_find_in_page_initial_state(self):
        """ Validate if the browser is in find in page mode and it is in its
        initial state
        """
        self.assertTrue(wait_until(self.browser.is_find_in_page_enabled))
        self.assertTrue(wait_until(self.browser.is_keyboard_displayed))
        self.assertEqual('0/0', self.browser.get_find_in_page_counter_text())
        self.assertEqual('find in page',
                         self.browser.get_find_in_page_placeholder_text())
        self.assertTrue(self._find_in_page_buttons_enabled(False))

    def _check_find_in_page_disabled_state(self):
        """ Validate if the browser is not in find in page mode """
        self.assertTrue(wait_until(self.browser.is_find_in_page_disable))
        self.assertTrue(wait_until(self.browser.is_keyboard_hidden))

    def _go_to_next_match_and_validate(self, expected_match,
                                       expected_total_matches,
                                       backwards=False):
        """
        Go to the next or previous find in page match and validate
        the toolbar text.
        """
        if backwards:
            button = self.browser.get_find_in_page_previous_button()
        else:
            button = self.browser.get_find_in_page_next_button()
        self.browser.pointing_device.click_object(button)
        self.assertEqual(
            '{current_match}/{matches}'.format(
                current_match=expected_match,
                matches=expected_total_matches
            ),
            self.browser.get_find_in_page_counter_text()
        )

    def test_find_in_page_matches(self):
        """
        Validate it is possible to find matches using the
        browsers find in page mode
        """
        self.browser.go_to_url(test_data.get('page2').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page2').actual_url)

        self.browser.open_find_in_page()
        self._check_find_in_page_initial_state()

        # Get initial screenshot used to compare after find
        (x, y, w, h) = self.browser.get_html_section_dimensions()
        initial_img = self.get_screenshot_image(x, y, w, h)

        # Find a text that it is included many times in the html
        self.browser.find_in_page('e')
        expected_matches = 4
        self.assertEqual(
            '1/{matches}'.format(matches=expected_matches),
            self.browser.get_find_in_page_counter_text()
        )
        self.assertTrue(self._find_in_page_buttons_enabled(True))
        find_img = self.get_screenshot_image(x, y, w, h)
        self.assertFalse(images.images_equal(initial_img, find_img))

        # Go one back to get the last match
        prev_img = find_img
        self._go_to_next_match_and_validate(
            expected_matches, expected_matches,
            backwards=True
        )
        next_img = self.get_screenshot_image(x, y, w, h)
        self.assertFalse(images.images_equal(prev_img, next_img))

        # Go one forward and get the first match
        prev_img = next_img
        self._go_to_next_match_and_validate(1, expected_matches)
        next_img = self.get_screenshot_image(x, y, w, h)
        self.assertFalse(images.images_equal(prev_img, next_img))
        self.assertTrue(images.images_equal(find_img, next_img))

        # Find a text that it is not included in the html
        self.browser.find_in_page('xxxxx')
        not_find_img = self.get_screenshot_image(x, y, w, h)
        # Validate the html was highlighted through images comparison
        self.assertTrue(images.images_equal(initial_img, not_find_img))

        # Go back and check find in page section is not being displayed
        self.browser.go_back()
        self._check_find_in_page_disabled_state()

    def test_delete_bookmark_swipe(self):
        """
        1. Open a website and bookmark that page
        2. Go to bookmarks screen and delete the newly bookmarked page
        3. Go back to main browser screen and ensure current page is no longer
           bookmarked
        """
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)
        self.browser.bookmark_current_page()
        bookmarks_view = self.browser.open_bookmarks_view()
        bookmarks_view.delete_bookmark(test_data.get('ubuntu').actual_url)
        self.assertFalse(self.browser.is_current_page_bookmarked())


class WebBrowserWithMessagingSystemTest(base_webbrowser.WebBrowserBase):

    def setUp(self):
        super().setUp()
        self.useFixture(messaging_fixtures.MessagingTestEnvironment())
        if number_of_sims() > 1:
            self.useFixture(messaging_fixtures.SetDualSIMPromptConfig())

    def test_copy_url_and_paste_in_messaging_app(self):
        """
        1. Open webbrowser and open a website
        2. Copy the loaded page url and open messaging app
        3. Create new message and paste the copied text into the message field
        """
        self.browser.go_to_url(test_data.get('ubuntu').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('ubuntu').actual_url)
        self.browser.copy_url_text()

        messaging_app = messaging.launch_messaging_app_from_launcher()
        # Apparently the OSK stays visible if the app was switched while
        # the keyboard was on screen. Reported: LP: 1505630
        keyboard.Keyboard().dismiss()

        text_entry = messaging_app.paste_text_in_message_field()
        self.assertThat(
            text_entry.text,
            Eventually(Equals(test_data.get('ubuntu').actual_url)))


class BookmarksAndHistoryBrowserSystemTest(base_webbrowser.WebBrowserBase):

    def setUp(self):
        urls_metadata = data['urls']
        urls = [value for _, value in urls_metadata.items()]
        xdg_data_home = self.useFixture(
            webbrowser_fixtures.XdgDataHomeFixture())
        self.bookmarks = self.useFixture(
            webbrowser_fixtures.BookmarksDatabaseFixture(
                urls=urls, data_dir=xdg_data_home.path))
        self.history = self.useFixture(
            webbrowser_fixtures.HistoryDatabaseFixture(
                urls=urls, data_dir=xdg_data_home.path))
        super().setUp()

    def test_new_tab_bookmarks_and_top_sites_layout(self):
        """New tab view shows correct quantity of bookmarks and top sites."""

        new_tab_view = self.browser.open_new_tab()

        # Check default quantity of sites on Bookmarks page
        urls_list = new_tab_view.get_bookmarks_list()
        # NOTE that the "Homepage" url is not counted, hence 4 instead of 5
        self.assertEqual(4, len(urls_list.get_urls()))

        # Tap 'More' to validate detail view
        new_tab_view.show_more()
        more_urls_list = new_tab_view.get_bookmarks('')
        self.assertLess(9, len(more_urls_list))

        # Tap 'Less'
        new_tab_view.show_less()
        urls_list = new_tab_view.get_bookmarks_list()
        self.assertEqual(4, len(urls_list.get_urls()))

        # up to 10 "Top sites" will be displayed
        top_sites_items = new_tab_view.get_top_site_items()
        self.assertLessEqual(10, len(top_sites_items))

    def test_open_bookmark_from_bookmarks_view(self):
        """Test that the user can open an existing bookmark via the menu."""

        # Tap on drawer menu, tap on bookmarks
        bookmarks_view = self.browser.open_bookmarks_view()

        # Bookmarks page is displayed with list of bookmarked pages
        urls = bookmarks_view.get_bookmarks_urls()
        self.assertLess(3, len(urls))

        # Select one bookmarked page
        url = urls[3]
        # the url string
        url_url = url.url
        url.select()

        # The bookmarked page opens
        self.browser.wait_until_page_loaded(url=url_url)

        # Open Bookmarks page again by selecting Bookmarks from drop-down menu
        bookmarks_view = self.browser.open_bookmarks_view()

        # Bookmarks page is displayed with list of bookmarked pages again
        self.assertLess(0, len(bookmarks_view.get_bookmarks_urls()))

        # Tap on "Done" option at bottom-left corner
        bookmarks_view.done()


class DownloadWithBrowserSystemTest(base_webbrowser.WebBrowserBase):

    def setUp(self):
        super().setUp()

    def get_download_notification(self):
        notifications = self.shell.get_notifications_list()
        return notifications.get_download_finished_notification()

    def test_cancel_ongoing_download(self):
        """
        1. Open webbrowser and type in a url for a file to download
        2. When the download dialog appears, press 'Cancel'
        3. Enter the url again and hit return
        4. When the download dialog appears, press 'Download'
        5. Ensure 'Downloads' screen is shown and the transfers indicator
           becomes visible
        6. Cancel the download, ensure transfers indicator hides
        """
        self.browser.go_to_url(data.urls.download1.url)
        options = self.browser.get_download_options_dialog()
        options.cancel()

        self.browser.go_to_url(data.urls.download1.url)
        options = self.browser.get_download_options_dialog()
        options.download()

        self.assertThat(
            self.shell.is_indicator_item_hidden(TRANSFER_INDICATOR),
            Eventually(Equals(False)))

        downloads_page = self.browser.get_downloads_page()
        downloads_page.cancel_download(data.urls.download1.url)

        self.assertThat(
            self.shell.is_indicator_item_hidden(TRANSFER_INDICATOR),
            Eventually(Equals(True)))

    def test_download_file_supported_by_currently_installed_apps(self):
        """
        1. Open web browser and open a url to a file, for which an associated
           app is already installed on the device
        2. Wait for the download prompt to appear and tap on
           'Choose application' and select 'Media Player' on the next screen
        3. Tap on 'Open' button in the 'Download Complete' dialog and ensure
           the newly downloaded file starts playing in the media player
        """
        self.browser.go_to_url(data.urls.download3.url)

        options = self.browser.get_download_options_dialog()
        options.choose_application()

        picker = self.browser.get_open_with_dialog()
        picker.pick_media_player()

        self.assertThat(
            self.shell.is_indicator_item_hidden(TRANSFER_INDICATOR),
            Eventually(Equals(False)))

        # Wait for download complete notification
        notification = self.get_download_notification()
        notification.open()

        player = mediaplayer.get_media_player_proxy().player
        self.assertTrue(player.playing)

    def test_download_file_not_supported_by_currently_installed_apps(self):
        """
        1. Open the webbrowser and go to a url to download a file for which
           there is no associated app installed
        2. Once the download completes, press the 'back' button to go back
           to the main screen
        3. Make sure the start page is loaded
        """
        self.browser.go_to_url(data.urls.download4.url)
        options = self.browser.get_download_options_dialog()
        options.download()

        downloads_page = self.browser.get_downloads_page()
        self.assertTrue(wait_until(downloads_page.is_download_finished,
                                   data.urls.download4.url, timeout=60))

        downloads_page.go_back_to_main_screen()
        self.browser.wait_until_page_loaded(
            test_data.get('start_page').actual_url)

    def test_download_image(self):
        """Test to download an image from internet an save it locally"""
        img = test_data.get('ubuntu_img')

        # check there are no files in download folder
        self.assertEqual(
            len(fs.recursive_files_in_dir(fs.DIR_HOME_CACHE_GALLERY)), 0)

        # Go to page with the picture to download
        self.browser.go_to_url(test_data.get('page3').entered_url)
        self.browser.wait_until_page_loaded(test_data.get('page3').actual_url)

        # Click on the picture
        webview = self.browser.get_current_webview()
        self.browser.pointing_device.move_to_object(webview)
        self.browser.pointing_device.move(self.browser.pointing_device.x,
                                          self.browser.pointing_device.y / 2)
        self.browser.pointing_device.click(press_duration=3)

        # Select to save the image
        save_image = self.browser.get_popover_foreground_dialog_item(
            'Save image')
        self.browser.pointing_device.click_object(save_image)

        # Choose open in application option
        options = self.browser.get_download_options_dialog()
        options.choose_application()

        # Pick the gallery to see the picture after it is downlaoded
        picker = self.browser.get_open_with_dialog()
        picker.pick_gallery()

        # Wait for download complete notification
        notification = self.get_download_notification()

        # The downlod has completed, get the file path of the downloaded file
        download_path = fs.recursive_files_in_dir(fs.DIR_HOME_CACHE_GALLERY)[0]
        # Validates the file is the same as the original
        actual_sha1 = fs.calculate_file_sha1(download_path)
        self.assertEqual(img.sha1, actual_sha1)

        # Validates the file downloaded is moved to Pictures when opened
        notification.open()
        downloaded_file = os.path.join(
            fs.DIR_HOME_PICTURES_IMPORTED, img.file_name)
        self.assertTrue(wait_until(os.path.isfile, downloaded_file))

        # Validate the gallery-app is displaying the downloaded image
        events_view = gallery.get_gallery_events_view()
        self.assertTrue(events_view.is_image_displayed(img.file_name))

        # check the displayed image matches the reference image
        self.assertTrue(
            images.image_files_equal(
                downloaded_file, REFERENCE_DOWNLOAD_IMAGE))


class DownloadsHistoryWithBrowserSystemTest(base_webbrowser.WebBrowserBase):

    def setUp(self):
        super().setUp(load_dummy_downloads_db=True)

    def test_delete_downloads_history(self):
        """
        1. Open webbrowser app and go to 'Downloads' page
        2. Ensure pre-populated download items are visible
        3. Delete one of the downloads items and ensure it gets removed from
           the list
        4. Now select all downloaded items and select delete
        5. Ensure all items gets removed from the UI as well as ~/Downloads
        """
        downloads_page = self.browser.open_downloads_page()
        self.assertEquals(downloads_page.get_visible_downloads_count(), 6)

        downloads_page.delete_history_item(url=data.urls.download2.url)
        self.assertThat(lambda: downloads_page.get_visible_downloads_count(),
                        Eventually(Equals(5)))

        downloads_page.clear_history()
        self.assertThat(lambda: downloads_page.get_visible_downloads_count(),
                        Eventually(Equals(0)))
